@@ -48,7 +48,7 @@ STATIC int _drbd_md_sync_page_io(struct drbd_conf *mdev,
 	md_io.error = 0;
 
 	if (rw == WRITE && !test_bit(MD_NO_BARRIER, &mdev->flags))
-		rw |= (1<<BIO_RW_BARRIER);
+	    rw |= (1<<BIO_RW_BARRIER);
 	rw |= (1 << BIO_RW_SYNC);
 
  retry:
@@ -64,7 +64,7 @@ STATIC int _drbd_md_sync_page_io(struct drbd_conf *mdev,
 
 	dump_internal_bio("Md", mdev, bio, 0);
 
-	if (FAULT_ACTIVE(mdev, (rw & WRITE) ? DRBD_FAULT_MD_WR : DRBD_FAULT_MD_RD))
+	if (FAULT_ACTIVE(mdev, (rw & WRITE)? DRBD_FAULT_MD_WR:DRBD_FAULT_MD_RD))
 		bio_endio(bio, -EIO);
 	else
 		submit_bio(rw, bio);
@@ -74,8 +74,8 @@ STATIC int _drbd_md_sync_page_io(struct drbd_conf *mdev,
 	/* check for unsupported barrier op */
 	if (unlikely(md_io.error == -EOPNOTSUPP && bio_barrier(bio))) {
 		/* Try again with no barrier */
-		drbd_WARN("Barriers not supported on meta data device - disabling\n");
-		set_bit(MD_NO_BARRIER, &mdev->flags);
+		WARN("Barriers not supported on meta data device - disabling\n");
+		set_bit(MD_NO_BARRIER,&mdev->flags);
 		rw &= ~(1 << BIO_RW_BARRIER);
 		bio_put(bio);
 		goto retry;
@@ -113,16 +113,16 @@ int drbd_md_sync_page_io(struct drbd_conf *mdev, struct drbd_backing_dev *bdev,
 			if (!page)
 				return 0;
 
-			drbd_WARN("Meta data's bdev hardsect = %d != %d\n",
+			WARN("Meta data's bdev hardsect = %d != %d\n",
 			     hardsect, MD_HARDSECT);
-			drbd_WARN("Workaround engaged (has performace impact).\n");
+			WARN("Workaround engaged (has performace impact).\n");
 
 			mdev->md_io_tmpp = page;
 		}
 
-		mask = (hardsect / MD_HARDSECT) - 1;
-		D_ASSERT(mask == 1 || mask == 3 || mask == 7);
-		D_ASSERT(hardsect == (mask+1) * MD_HARDSECT);
+		mask = ( hardsect / MD_HARDSECT ) - 1;
+		D_ASSERT( mask == 1 || mask == 3 || mask == 7 );
+		D_ASSERT( hardsect == (mask+1) * MD_HARDSECT );
 		offset = sector & mask;
 		sector = sector & ~mask;
 		iop = mdev->md_io_tmpp;
@@ -227,9 +227,9 @@ struct lc_element *_al_get(struct drbd_conf *mdev, unsigned int enr)
 	/*
 	if (!al_ext) {
 		if (al_flags & LC_STARVING)
-			drbd_WARN("Have to wait for LRU element (AL too small?)\n");
+			WARN("Have to wait for LRU element (AL too small?)\n");
 		if (al_flags & LC_DIRTY)
-			drbd_WARN("Ongoing AL update (AL device too slow?)\n");
+			WARN("Ongoing AL update (AL device too slow?)\n");
 	}
 	*/
 
@@ -253,7 +253,7 @@ void drbd_al_begin_io(struct drbd_conf *mdev, sector_t sector)
 		    (int)BM_SECT_TO_EXT(sector));
 	       );
 
-	wait_event(mdev->al_wait, (al_ext = _al_get(mdev, enr)));
+	wait_event(mdev->al_wait, (al_ext = _al_get(mdev, enr)) );
 
 	if (al_ext->lc_number != enr) {
 		/* drbd_al_write_transaction(mdev,al_ext,enr);
@@ -313,20 +313,20 @@ void drbd_al_complete_io(struct drbd_conf *mdev, sector_t sector)
 int
 w_al_write_transaction(struct drbd_conf *mdev, struct drbd_work *w, int unused)
 {
-	struct update_al_work *aw = (struct update_al_work *)w;
+	struct update_al_work *aw = (struct update_al_work*)w;
 	struct lc_element *updated = aw->al_ext;
 	const unsigned int new_enr = aw->enr;
 	const unsigned int evicted = aw->old_enr;
 
-	struct al_transaction *buffer;
+	struct al_transaction* buffer;
 	sector_t sector;
-	int i, n, mx;
+	int i,n,mx;
 	unsigned int extent_nr;
-	u32 xor_sum = 0;
+	u32 xor_sum=0;
 
 	if (!inc_local(mdev)) {
 		ERR("inc_local() failed in w_al_write_transaction\n");
-		complete(&((struct update_al_work *)w)->event);
+		complete(&((struct update_al_work*)w)->event);
 		return 1;
 	}
 	/* do we have to do a bitmap write, first?
@@ -414,9 +414,9 @@ STATIC int drbd_al_read_tr(struct drbd_conf *mdev,
 	if (!drbd_md_sync_page_io(mdev, bdev, sector, READ))
 		return -1;
 
-	rv = (be32_to_cpu(b->magic) == DRBD_MAGIC);
+	rv = ( be32_to_cpu(b->magic) == DRBD_MAGIC );
 
-	for (i = 0; i < AL_EXTENTS_PT + 1; i++)
+	for (i = 0; i < AL_EXTENTS_PT+1; i++)
 		xor_sum ^= be32_to_cpu(b->updates[i].extent);
 	rv &= (xor_sum == be32_to_cpu(b->xor_sum));
 
@@ -461,6 +461,7 @@ int drbd_al_read_log(struct drbd_conf *mdev, struct drbd_backing_dev *bdev)
 			return 0;
 		}
 		cnr = be32_to_cpu(buffer->tr_number);
+		/* INFO("index %d valid tnr=%d\n",i,cnr); */
 
 		if (cnr == -1)
 			overflow = 1;
@@ -476,7 +477,7 @@ int drbd_al_read_log(struct drbd_conf *mdev, struct drbd_backing_dev *bdev)
 	}
 
 	if (from == -1 || to == -1) {
-		drbd_WARN("No usable activity log found.\n");
+		WARN("No usable activity log found.\n");
 
 		up(&mdev->md_io_mutex);
 		return 1;
@@ -559,7 +560,8 @@ STATIC BIO_ENDIO_TYPE atodb_endio BIO_ENDIO_ARGS(struct bio *bio, int error)
 	BIO_ENDIO_FN_START;
 	/* strange behaviour of some lower level drivers...
 	 * fail the request by clearing the uptodate flag,
-	 * but do not return any error?! */
+	 * but do not return any error?!
+	 * do we want to WARN() on this? */
 	if (!error && !uptodate)
 		error = -EIO;
 
@@ -604,7 +606,7 @@ STATIC int atodb_prepare_unless_covered(struct drbd_conf *mdev,
 	 * the last invocation iterates over all bios,
 	 * and finds the last NULL entry.
 	 */
-	while ((bio = bios[i])) {
+	while ( (bio = bios[i]) ) {
 		if (bio->bi_sector == on_disk_sector)
 			return 0;
 		i++;
@@ -630,9 +632,9 @@ STATIC int atodb_prepare_unless_covered(struct drbd_conf *mdev,
 	}
 
 	offset = S2W(enr);
-	drbd_bm_get_lel(mdev, offset,
-			min_t(size_t, S2W(1), drbd_bm_words(mdev) - offset),
-			kmap(page) + page_offset);
+	drbd_bm_get_lel( mdev, offset,
+			 min_t(size_t, S2W(1), drbd_bm_words(mdev) - offset),
+			 kmap(page) + page_offset );
 	kunmap(page);
 
 	bio->bi_private = wc;
@@ -711,7 +713,7 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 	for (i = 0; i < nr_elements; i++) {
 		if (bios[i] == NULL)
 			break;
-		if (FAULT_ACTIVE(mdev, DRBD_FAULT_MD_WR)) {
+		if (FAULT_ACTIVE( mdev, DRBD_FAULT_MD_WR )) {
 			bios[i]->bi_rw = WRITE;
 			bio_endio(bios[i], -EIO);
 		} else {
@@ -745,10 +747,11 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 	/* free everything by calling the endio callback directly. */
 	for (i = 0; i < nr_elements && bios[i]; i++)
 		bio_endio(bios[i], 0);
+
 	kfree(bios);
 
  submit_one_by_one:
-	drbd_WARN("Using the slow drbd_al_to_on_disk_bm()\n");
+	WARN("Using the slow drbd_al_to_on_disk_bm()\n");
 
 	for (i = 0; i < mdev->act_log->nr_elements; i++) {
 		enr = lc_entry(mdev->act_log, i)->lc_number;
@@ -756,7 +759,7 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 			continue;
 		/* Really slow: if we have al-extents 16..19 active,
 		 * sector 4 will be written four times! Synchronous! */
-		drbd_bm_write_sect(mdev, enr/AL_EXT_PER_BM_SECT);
+		drbd_bm_write_sect(mdev, enr/AL_EXT_PER_BM_SECT );
 	}
 
 	lc_unlock(mdev->act_log);
@@ -797,13 +800,12 @@ static inline int _try_lc_del(struct drbd_conf *mdev, struct lc_element *al_ext)
 
 	spin_lock_irq(&mdev->al_lock);
 	rv = (al_ext->refcnt == 0);
-	if (likely(rv))
-		lc_del(mdev->act_log, al_ext);
+	if (likely(rv)) lc_del(mdev->act_log, al_ext);
 	spin_unlock_irq(&mdev->al_lock);
 
-	MTRACE(TraceTypeALExts, TraceLvlMetrics,
-		if (unlikely(!rv))
-			INFO("Waiting for extent in drbd_al_shrink()\n");
+	MTRACE(TraceTypeALExts,TraceLvlMetrics,
+	       if(unlikely(!rv))
+		       INFO("Waiting for extent in drbd_al_shrink()\n");
 	       );
 
 	return rv;
@@ -819,7 +821,7 @@ void drbd_al_shrink(struct drbd_conf *mdev)
 	struct lc_element *al_ext;
 	int i;
 
-	D_ASSERT(test_bit(__LC_DIRTY, &mdev->act_log->flags));
+	D_ASSERT( test_bit(__LC_DIRTY, &mdev->act_log->flags) );
 
 	for (i = 0; i < mdev->act_log->nr_elements; i++) {
 		al_ext = lc_entry(mdev->act_log, i);
@@ -837,11 +839,11 @@ STATIC int w_update_odbm(struct drbd_conf *mdev, struct drbd_work *w, int unused
 
 	if (!inc_local(mdev)) {
 		if (DRBD_ratelimit(5*HZ, 5))
-			drbd_WARN("Can not update on disk bitmap, local IO disabled.\n");
+			WARN("Can not update on disk bitmap, local IO disabled.\n");
 		return 1;
 	}
 
-	drbd_bm_write_sect(mdev, udw->enr);
+	drbd_bm_write_sect(mdev, udw->enr );
 	dec_local(mdev);
 
 	kfree(udw);
@@ -851,9 +853,7 @@ STATIC int w_update_odbm(struct drbd_conf *mdev, struct drbd_work *w, int unused
 		case SyncSource:  case SyncTarget:
 		case PausedSyncS: case PausedSyncT:
 			drbd_resync_finished(mdev);
-		default:
-			/* nothing to do */
-			break;
+		default: /* nothing to do */;
 		}
 	}
 	drbd_bcast_sync_progress(mdev);
@@ -907,18 +907,18 @@ STATIC void drbd_try_clear_on_disk_bm(struct drbd_conf *mdev, sector_t sector,
 			 * since drbd_rs_begin_io() pulled it already in.
 			 *
 			 * But maybe an application write finished, and we set
-			 * something outside the resync lru_cache in sync.
+			 * something in outside the resync lru_cache in sync.
 			 */
 			int rs_left = drbd_bm_e_weight(mdev, enr);
 			if (ext->flags != 0) {
-				drbd_WARN("changing resync lce: %d[%u;%02lx]"
+				WARN("changing resync lce: %d[%u;%02lx]"
 				     " -> %d[%u;00]\n",
 				     ext->lce.lc_number, ext->rs_left,
 				     ext->flags, enr, rs_left);
 				ext->flags = 0;
 			}
 			if (ext->rs_failed) {
-				drbd_WARN("Kicking resync_lru element enr=%u "
+				WARN("Kicking resync_lru element enr=%u "
 				     "out with rs_failed=%d\n",
 				     ext->lce.lc_number, ext->rs_failed);
 				set_bit(WRITE_BM_AFTER_RESYNC, &mdev->flags);
@@ -939,7 +939,7 @@ STATIC void drbd_try_clear_on_disk_bm(struct drbd_conf *mdev, sector_t sector,
 				udw->w.cb = w_update_odbm;
 				drbd_queue_work_front(&mdev->data.work, &udw->w);
 			} else {
-				drbd_WARN("Could not kmalloc an udw\n");
+				WARN("Could not kmalloc an udw\n");
 				set_bit(WRITE_BM_AFTER_RESYNC, &mdev->flags);
 			}
 		}
@@ -974,7 +974,7 @@ void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector, int size,
 		return;
 	}
 	nr_sectors = drbd_get_capacity(mdev->this_bdev);
-	esector = sector + (size >> 9) - 1;
+	esector = sector + (size>>9) -1;
 
 	ERR_IF(sector >= nr_sectors) return;
 	ERR_IF(esector >= nr_sectors) esector = (nr_sectors-1);
@@ -1011,9 +1011,9 @@ void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector, int size,
 		if (jiffies - mdev->rs_mark_time > HZ*10) {
 			/* should be roling marks,
 			 * but we estimate only anyways. */
-			if (mdev->rs_mark_left != drbd_bm_total_weight(mdev) &&
+			if ( mdev->rs_mark_left != drbd_bm_total_weight(mdev) &&
 			    mdev->state.conn != PausedSyncT &&
-			    mdev->state.conn != PausedSyncS) {
+			    mdev->state.conn != PausedSyncS ) {
 				mdev->rs_mark_time = jiffies;
 				mdev->rs_mark_left = drbd_bm_total_weight(mdev);
 			}
@@ -1042,8 +1042,10 @@ void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector, int size,
 void __drbd_set_out_of_sync(struct drbd_conf *mdev, sector_t sector, int size,
 			    const char *file, const unsigned int line)
 {
-	unsigned long sbnr, ebnr, lbnr;
+	unsigned long sbnr, ebnr, lbnr, flags;
 	sector_t esector, nr_sectors;
+	unsigned int enr, count;
+	struct bm_extent* ext;
 
 	if (size <= 0 || (size & 0x1ff) != 0 || size > DRBD_MAX_SEGMENT_SIZE) {
 		ERR("sector: %llus, size: %d\n",
@@ -1055,7 +1057,7 @@ void __drbd_set_out_of_sync(struct drbd_conf *mdev, sector_t sector, int size,
 		return; /* no disk, no metadata, no bitmap to set bits in */
 
 	nr_sectors = drbd_get_capacity(mdev->this_bdev);
-	esector = sector + (size >> 9) - 1;
+	esector = sector + (size>>9) -1;
 
 	ERR_IF(sector >= nr_sectors)
 		goto out;
@@ -1077,7 +1079,14 @@ void __drbd_set_out_of_sync(struct drbd_conf *mdev, sector_t sector, int size,
 
 	/* ok, (capacity & 7) != 0 sometimes, but who cares...
 	 * we count rs_{total,left} in bits, not sectors.  */
-	drbd_bm_set_bits(mdev, sbnr, ebnr);
+	spin_lock_irqsave(&mdev->al_lock, flags);
+	count = drbd_bm_set_bits(mdev, sbnr, ebnr);
+
+	enr = BM_SECT_TO_EXT(sector);
+	ext = (struct bm_extent *) lc_find(mdev->resync, enr);
+	if (ext)
+		ext->rs_left += count;
+	spin_unlock_irqrestore(&mdev->al_lock, flags);
 
 out:
 	dec_local(mdev);
@@ -1114,7 +1123,7 @@ struct bm_extent *_bme_get(struct drbd_conf *mdev, unsigned int enr)
 
 	if (!bm_ext) {
 		if (rs_flags & LC_STARVING)
-			drbd_WARN("Have to wait for element"
+			WARN("Have to wait for element"
 			     " (resync LRU too small?)\n");
 		BUG_ON(rs_flags & LC_DIRTY);
 	}
@@ -1128,8 +1137,7 @@ static inline int _is_in_al(struct drbd_conf *mdev, unsigned int enr)
 	int rv = 0;
 
 	spin_lock_irq(&mdev->al_lock);
-	if (unlikely(enr == mdev->act_log->new_number))
-		rv = 1;
+	if (unlikely(enr == mdev->act_log->new_number)) rv = 1;
 	else {
 		al_ext = lc_find(mdev->act_log, enr);
 		if (al_ext) {
@@ -1168,20 +1176,19 @@ int drbd_rs_begin_io(struct drbd_conf *mdev, sector_t sector)
 		    (unsigned long long)sector, enr);
 	    );
 
-	sig = wait_event_interruptible(mdev->al_wait,
-			(bm_ext = _bme_get(mdev, enr)));
+	sig = wait_event_interruptible( mdev->al_wait,
+			(bm_ext = _bme_get(mdev, enr)) );
 	if (sig)
 		return 0;
 
-	if (test_bit(BME_LOCKED, &bm_ext->flags))
-		return 1;
+	if (test_bit(BME_LOCKED, &bm_ext->flags)) return 1;
 
 	for (i = 0; i < AL_EXT_PER_BM_SECT; i++) {
-		sig = wait_event_interruptible(mdev->al_wait,
-				!_is_in_al(mdev, enr * AL_EXT_PER_BM_SECT + i));
+		sig = wait_event_interruptible( mdev->al_wait,
+				!_is_in_al(mdev, enr*AL_EXT_PER_BM_SECT+i) );
 		if (sig) {
 			spin_lock_irq(&mdev->al_lock);
-			if (lc_put(mdev->resync, &bm_ext->lce) == 0) {
+			if ( lc_put(mdev->resync, &bm_ext->lce) == 0 ) {
 				clear_bit(BME_NO_WRITES, &bm_ext->flags);
 				mdev->resync_locked--;
 				wake_up(&mdev->al_wait);
@@ -1280,7 +1287,7 @@ int drbd_try_rs_begin_io(struct drbd_conf *mdev, sector_t sector)
 		if (!bm_ext) {
 			const unsigned long rs_flags = mdev->resync->flags;
 			if (rs_flags & LC_STARVING)
-				drbd_WARN("Have to wait for element"
+				WARN("Have to wait for element"
 				     " (resync LRU too small?)\n");
 			BUG_ON(rs_flags & LC_DIRTY);
 			goto try_again;
@@ -1350,7 +1357,7 @@ void drbd_rs_complete_io(struct drbd_conf *mdev, sector_t sector)
 		return;
 	}
 
-	if (lc_put(mdev->resync, (struct lc_element *)bm_ext) == 0) {
+	if ( lc_put(mdev->resync, (struct lc_element *)bm_ext) == 0 ) {
 		clear_bit(BME_LOCKED, &bm_ext->flags);
 		clear_bit(BME_NO_WRITES, &bm_ext->flags);
 		mdev->resync_locked--;
@@ -1372,7 +1379,7 @@ void drbd_rs_cancel_all(struct drbd_conf *mdev)
 
 	spin_lock_irq(&mdev->al_lock);
 
-	if (inc_local_if_state(mdev, Failed)) { /* Makes sure ->resync is there. */
+	if (inc_local_if_state(mdev,Failed)) { /* Makes sure ->resync is there. */
 		lc_reset(mdev->resync);
 		dec_local(mdev);
 	}
@@ -1406,7 +1413,7 @@ int drbd_rs_del_all(struct drbd_conf *mdev)
 			if (bm_ext->lce.lc_number == LC_FREE)
 				continue;
 			if (bm_ext->lce.lc_number == mdev->resync_wenr) {
-				INFO("dropping %u in drbd_rs_del_all, apparently"
+				INFO("dropping %u in drbd_rs_del_all, aparently"
 				     " got 'synced' by application io\n",
 				     mdev->resync_wenr);
 				D_ASSERT(!test_bit(BME_LOCKED, &bm_ext->flags));
@@ -1422,7 +1429,6 @@ int drbd_rs_del_all(struct drbd_conf *mdev)
 				spin_unlock_irq(&mdev->al_lock);
 				return -EAGAIN;
 			}
-			D_ASSERT(bm_ext->rs_left == 0);
 			D_ASSERT(!test_bit(BME_LOCKED, &bm_ext->flags));
 			D_ASSERT(!test_bit(BME_NO_WRITES, &bm_ext->flags));
 			lc_del(mdev->resync, &bm_ext->lce);
@@ -1459,7 +1465,7 @@ void drbd_rs_failed_io(struct drbd_conf *mdev, sector_t sector, int size)
 		return;
 	}
 	nr_sectors = drbd_get_capacity(mdev->this_bdev);
-	esector = sector + (size >> 9) - 1;
+	esector = sector + (size>>9) -1;
 
 	ERR_IF(sector >= nr_sectors) return;
 	ERR_IF(esector >= nr_sectors) esector = (nr_sectors-1);
