@@ -98,84 +98,53 @@ struct drbd_bitmap {
 	unsigned long  bm_flags;
 
 	/* debugging aid, in case we are still racy somewhere */
-	char          *bm_why;
-	struct task_struct *bm_task;
+	unsigned long  bm_line;
+	char          *bm_file;
 };
 
 /* definition of bits in bm_flags */
 #define BM_LOCKED 0
 #define BM_MD_IO_ERROR (BITS_PER_LONG-1) /* 31? 63? */
 
-static inline int bm_is_locked(struct drbd_bitmap *b)
-{
-	return test_bit(BM_LOCKED, &b->bm_flags);
-}
-
-#define bm_print_lock_info(m) __bm_print_lock_info(m, __func__)
-static void __bm_print_lock_info(struct drbd_conf *mdev, const char *func)
+void __drbd_bm_lock(struct drbd_conf *mdev, char *file, int line)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	if (!DRBD_ratelimit(5*HZ, 5))
-		return;
-	ERR("FIXME %s in %s, bitmap locked for '%s' by %s\n",
-	    current == mdev->receiver.task ? "receiver" :
-	    current == mdev->asender.task  ? "asender"  :
-	    current == mdev->worker.task   ? "worker"   : current->comm,
-	    func, b->bm_why ?: "?",
-	    b->bm_task == mdev->receiver.task ? "receiver" :
-	    b->bm_task == mdev->asender.task  ? "asender"  :
-	    b->bm_task == mdev->worker.task   ? "worker"   : "?");
-}
 
-void drbd_bm_lock(struct drbd_conf *mdev, char *why)
-{
-	struct drbd_bitmap *b = mdev->bitmap;
-	int trylock_failed;
-
-	if (!b) {
-		ERR("FIXME no bitmap in drbd_bm_lock!?\n");
-		return;
+	spin_lock_irq(&b->bm_lock);
+	if (!__test_and_set_bit(BM_LOCKED,&b->bm_flags)) {
+		b->bm_file = file;
+		b->bm_line = line;
+	} else if (DRBD_ratelimit(5*HZ,5)) {
+		ERR("%s:%d: bitmap already locked by %s:%lu\n",
+		    file, line, b->bm_file,b->bm_line);
+		/*
+		dump_stack();
+		ERR("This is no oops, but debug stack trace only.\n");
+		ERR("If you get this often, or in reproducable situations, "
+		    "notify <drbd-devel@linbit.com>\n");
+		*/
 	}
-
-	trylock_failed = down_trylock(&b->bm_change);
-
-	if (trylock_failed) {
-		DBG("%s going to '%s' but bitmap already locked for '%s' by %s\n",
-		    current == mdev->receiver.task ? "receiver" :
-		    current == mdev->asender.task  ? "asender"  :
-		    current == mdev->worker.task   ? "worker"   : "?",
-		    why, b->bm_why ?: "?",
-		    b->bm_task == mdev->receiver.task ? "receiver" :
-		    b->bm_task == mdev->asender.task  ? "asender"  :
-		    b->bm_task == mdev->worker.task   ? "worker"   : "?");
-		down(&b->bm_change);
-	}
-	if (__test_and_set_bit(BM_LOCKED,&b->bm_flags))
-		ERR("FIXME bitmap already locked in bm_lock\n");
-
-	b->bm_why  = why;
-	b->bm_task = current;
+	spin_unlock_irq(&b->bm_lock);
 }
 
 void drbd_bm_unlock(struct drbd_conf *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	if (!b) {
-		ERR("FIXME no bitmap in drbd_bm_unlock!?\n");
-		return;
+	spin_lock_irq(&b->bm_lock);
+	if (!__test_and_clear_bit(BM_LOCKED,&mdev->bitmap->bm_flags)) {
+		ERR("bitmap not locked in bm_unlock\n");
+	} else {
+		/* FIXME if we got a "is already locked" previously,
+		 * we unlock here even though we actually MUST NOT do so... */
+		b->bm_file = NULL;
+		b->bm_line = -1;
 	}
-
-	if (!__test_and_clear_bit(BM_LOCKED, &mdev->bitmap->bm_flags))
-		ERR("FIXME bitmap not locked in bm_unlock\n");
-
-	b->bm_why  = NULL;
-	b->bm_task = NULL;
-	up(&b->bm_change);
+	spin_unlock_irq(&b->bm_lock);
 }
 
 #if DUMP_MD >= 3
 /* debugging aid */
-STATIC void bm_end_info(struct drbd_conf *mdev, const char *where)
+void bm_end_info(struct drbd_conf *mdev, const char *where)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	size_t w = (b->bm_bits-1) >> LN2_BPL;
@@ -185,7 +154,7 @@ STATIC void bm_end_info(struct drbd_conf *mdev, const char *where)
 	w++;
 
 	if (w < b->bm_words) {
-		D_ASSERT(w == b->bm_words - 1);
+		D_ASSERT(w == b->bm_words -1);
 		INFO("bm[%d]=0x%lX\n", w, b->bm[w]);
 	}
 }
@@ -256,7 +225,7 @@ void drbd_bm_cleanup(struct drbd_conf *mdev)
  */
 STATIC int bm_clear_surplus(struct drbd_bitmap *b)
 {
-	const unsigned long mask = (1UL << (b->bm_bits & (BITS_PER_LONG-1))) - 1;
+	const unsigned long mask = (1UL << (b->bm_bits & (BITS_PER_LONG-1))) -1;
 	size_t w = b->bm_bits >> LN2_BPL;
 	int cleared = 0;
 
@@ -275,7 +244,7 @@ STATIC int bm_clear_surplus(struct drbd_bitmap *b)
 
 STATIC void bm_set_surplus(struct drbd_bitmap *b)
 {
-	const unsigned long mask = (1UL << (b->bm_bits & (BITS_PER_LONG-1))) - 1;
+	const unsigned long mask = (1UL << (b->bm_bits & (BITS_PER_LONG-1))) -1;
 	size_t w = b->bm_bits >> LN2_BPL;
 
 	if (w < b->bm_words)
@@ -291,10 +260,9 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 	unsigned long *ep = b->bm + b->bm_words;
 	unsigned long bits = 0;
 
-	while (bm < ep) {
+	while ( bm < ep ) {
 #ifndef __LITTLE_ENDIAN
-		if (swap_endian)
-			*bm = lel_to_cpu(*bm);
+		if (swap_endian) *bm = lel_to_cpu(*bm);
 #endif
 		bits += hweight_long(*bm++);
 	}
@@ -304,28 +272,22 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 
 static inline unsigned long bm_count_bits(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 0);
+	return __bm_count_bits(b,0);
 }
 
 static inline unsigned long bm_count_bits_swap_endian(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 1);
+	return __bm_count_bits(b,1);
 }
 
 
-void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line)
+void _drbd_bm_recount_bits(struct drbd_conf *mdev, char* file, int line)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long flags, bits;
 
 	ERR_IF(!b) return;
 
-	/* IMO this should be inside drbd_bm_lock/unlock.
-	 * Unfortunately it is used outside of the locks.
-	 * And I'm not yet sure where we need to place the
-	 * lock/unlock correctly.
-	 */
-
 	spin_lock_irqsave(&b->bm_lock, flags);
 	bits = bm_count_bits(b);
 	if (bits != b->bm_set) {
@@ -354,7 +316,9 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 
 	ERR_IF(!b) return -ENOMEM;
 
-	drbd_bm_lock(mdev, "resize");
+	ERR_IF (down_trylock(&b->bm_change)) {
+		down(&b->bm_change);
+	}
 
 	INFO("drbd_bm_resize called with capacity == %llu\n",
 			(unsigned long long)capacity);
@@ -388,18 +352,18 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 			dec_local(mdev);
 		}
 
-		growing = bits > b->bm_bits;
 		if (words == b->bm_words) {
 			/* optimize: capacity has changed,
-			 * but only within one ulong64 word worth of bits.
-			 * no allocation needed, just update the
-			 * bm_dev_capacity and bm_bits members,
-			 * and set the new bits, if any */
+			 * but only within one long word worth of bits.
+			 * just update the bm_dev_capacity and bm_bits members.
+			 */
 			spin_lock_irq(&b->bm_lock);
-			bm_set_surplus(b);
-			if (growing)
-				b->bm_set += bits - b->bm_bits;
-			goto done;
+			b->bm_bits    = bits;
+			b->bm_dev_capacity = capacity;
+			b->bm_set -= bm_clear_surplus(b);
+			bm_end_info(mdev, __FUNCTION__ );
+			spin_unlock_irq(&b->bm_lock);
+			goto out;
 		} else {
 			/* one extra long to catch off by one errors */
 			bytes = (words+1)*sizeof(long);
@@ -416,41 +380,35 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 		/* brgs. move several MB within spinlock...
 		 * FIXME this should go into userspace! */
 		if (obm) {
-			/* adjust for possibly partially used
-			 * last word of old bitmap. */
 			bm_set_surplus(b);
 			D_ASSERT(b->bm[b->bm_words] == DRBD_MAGIC);
 			memcpy(nbm, obm, min_t(size_t, b->bm_words, words)
 								*sizeof(long));
 		}
+		growing = words > b->bm_words;
 		if (growing) {
-			/* set all newly allocated bits */
-			memset(nbm + b->bm_words, 0xff,
-				(words - b->bm_words) * sizeof(long));
-			/* yes, I know, this is not the same number as was set by this memset.
-			 * bm_set_surplus above before the memcpy,
-			 * and bm_clear_surplus below after the new assignments
-			 * make sure that this is indeed the amount of newly set bits */
+			/* set all newly allocated bits
+			 * start at -1, just to be sure. */
+			memset( nbm + (b->bm_words?:1)-1 , 0xff,
+				(words - ((b->bm_words?:1)-1)) * sizeof(long) );
 			b->bm_set  += bits - b->bm_bits;
 		}
 		nbm[words] = DRBD_MAGIC;
 		b->bm = nbm;
+		b->bm_bits  = bits;
 		b->bm_words = words;
- done:
-		b->bm_bits = bits;
 		b->bm_dev_capacity = capacity;
-		/* finally clear possibly only partially used last words */
 		bm_clear_surplus(b);
 		if (!growing)
 			b->bm_set = bm_count_bits(b);
-		bm_end_info(mdev, __func__);
+		bm_end_info(mdev, __FUNCTION__ );
 		spin_unlock_irq(&b->bm_lock);
 		INFO("resync bitmap: bits=%lu words=%lu\n", bits, words);
 	}
  free_obm:
 	vfree(obm); /* vfree(NULL) is noop */
  out:
-	drbd_bm_unlock(mdev);
+	up(&b->bm_change);
 	return err;
 }
 
@@ -479,27 +437,35 @@ unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
 
 size_t drbd_bm_words(struct drbd_conf *mdev)
 {
-	ERR_IF(!mdev->bitmap) return 0;
-	return mdev->bitmap->bm_words;
+	struct drbd_bitmap *b = mdev->bitmap;
+	ERR_IF(!b) return 0;
+	return b->bm_words;
 }
 
+unsigned long drbd_bm_bits(struct drbd_conf *mdev)
+{
+	struct drbd_bitmap *b = mdev->bitmap;
+	ERR_IF(!b) return 0;
+
+	return b->bm_bits;
+}
+
+
 /* merge number words from buffer into the bitmap starting at offset.
  * buffer[i] is expected to be little endian unsigned long.
- * bitmap must be locked by drbd_bm_lock.
- * currently only used from receive_bitmap.
  */
-void drbd_bm_merge_lel(struct drbd_conf *mdev, size_t offset, size_t number,
-			unsigned long *buffer)
+void drbd_bm_merge_lel( struct drbd_conf *mdev, size_t offset, size_t number,
+			unsigned long *buffer )
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long *bm;
 	unsigned long word, bits;
 	size_t n = number;
 
-	ERR_IF(!b) return;
-	ERR_IF(!b->bm) return;
 	if (number == 0)
 		return;
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 	WARN_ON(offset        >= b->bm_words);
 	WARN_ON(offset+number >  b->bm_words);
 	WARN_ON(number > PAGE_SIZE/sizeof(long));
@@ -519,7 +485,7 @@ void drbd_bm_merge_lel(struct drbd_conf *mdev, size_t offset, size_t number,
 	 */
 	if (offset+number == b->bm_words) {
 		b->bm_set -= bm_clear_surplus(b);
-		bm_end_info(mdev, __func__);
+		bm_end_info(mdev, __FUNCTION__ );
 	}
 	spin_unlock_irq(&b->bm_lock);
 }
@@ -528,27 +494,30 @@ void drbd_bm_merge_lel(struct drbd_conf *mdev, size_t offset, size_t number,
  * buffer[i] will be little endian unsigned long.
  */
 void drbd_bm_get_lel(struct drbd_conf *mdev, size_t offset, size_t number,
-		     unsigned long *buffer)
+		     unsigned long *buffer )
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long *bm;
 
+	if (number == 0)
+		return;
 	ERR_IF(!b) return;
 	ERR_IF(!b->bm) return;
-	spin_lock_irq(&b->bm_lock);
-	if ((offset        >= b->bm_words) ||
-	    (offset+number >  b->bm_words) ||
-	    (number > PAGE_SIZE/sizeof(long)) ||
-	    (number <= 0))
+	if ( (offset        >= b->bm_words) ||
+	     (offset+number >  b->bm_words) ||
+	     (number > PAGE_SIZE/sizeof(long)) ||
+	     (number <= 0) ) {
+		/* yes, there is "%z", but that gives compiler warnings... */
 		ERR("offset=%lu number=%lu bm_words=%lu\n",
 			(unsigned long)	offset,
 			(unsigned long)	number,
 			(unsigned long) b->bm_words);
-	else {
-		bm = b->bm + offset;
-		while (number--)
-			*buffer++ = cpu_to_lel(*bm++);
+		return;
 	}
+
+	spin_lock_irq(&b->bm_lock);
+	bm = b->bm + offset;
+	while (number--) *buffer++ = cpu_to_lel(*bm++);
 	spin_unlock_irq(&b->bm_lock);
 }
 
@@ -572,7 +541,6 @@ static BIO_ENDIO_TYPE bm_async_io_complete BIO_ENDIO_ARGS(struct bio *bio, int e
 	int uptodate = bio_flagged(bio, BIO_UPTODATE);
 
 	BIO_ENDIO_FN_START;
-
 	/* strange behaviour of some lower level drivers...
 	 * fail the request by clearing the uptodate flag,
 	 * but do not return any error?!
@@ -621,7 +589,7 @@ STATIC void bm_page_io_async(struct drbd_conf *mdev, struct drbd_bitmap *b, int
 	bio->bi_private = b;
 	bio->bi_end_io = bm_async_io_complete;
 
-	if (FAULT_ACTIVE(mdev, (rw & WRITE) ? DRBD_FAULT_MD_WR : DRBD_FAULT_MD_RD)) {
+	if (FAULT_ACTIVE(mdev, (rw&WRITE)?DRBD_FAULT_MD_WR:DRBD_FAULT_MD_RD)) {
 		bio->bi_rw |= rw;
 		bio_endio(bio, -EIO);
 	} else {
@@ -639,7 +607,7 @@ void bm_cpu_to_lel(struct drbd_bitmap *b)
 	 * this may be optimized by using
 	 * cpu_to_lel(-1) == -1 and cpu_to_lel(0) == 0;
 	 * the following is still not optimal, but better than nothing */
-	unsigned long *end = b->bm+b->bm_words;
+	const unsigned long *end = b->bm+b->bm_words;
 	unsigned long *bm;
 	if (b->bm_set == 0) {
 		/* no page at all; avoid swap if all is 0 */
@@ -670,13 +638,17 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 	char ppb[10];
 	int err = 0;
 
-	WARN_ON(!bm_is_locked(b));
-
-	/* no spinlock here, the drbd_bm_lock should be enough! */
-
 	bm_words  = drbd_bm_words(mdev);
 	num_pages = (bm_words*sizeof(long) + PAGE_SIZE-1) >> PAGE_SHIFT;
 
+	/* OK, I manipulate the bitmap low level,
+	 * and I expect to be the exclusive user.
+	 * If not, I am really in a bad mood...
+	 * to catch such bugs early, make all people who want to access the
+	 * bitmap while I read/write it dereference a NULL pointer :->
+	 */
+	mdev->bitmap = NULL;
+
 	/* on disk bitmap is little endian */
 	if (rw == WRITE)
 		bm_cpu_to_lel(b);
@@ -691,11 +663,8 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 
 	drbd_blk_run_queue(bdev_get_queue(mdev->bc->md_bdev));
 	wait_event(b->bm_io_wait, atomic_read(&b->bm_async_io) == 0);
-
-	MTRACE(TraceTypeMDIO, TraceLvlSummary,
-	       INFO("%s of bitmap took %lu jiffies\n",
-		    rw == READ ? "reading" : "writing", jiffies - now);
-	       );
+	INFO("%s of bitmap took %lu jiffies\n",
+	     rw == READ ? "reading" : "writing", jiffies - now);
 
 	if (test_bit(BM_MD_IO_ERROR, &b->bm_flags)) {
 		ALERT("we had at least one MD IO ERROR during bitmap IO\n");
@@ -709,7 +678,7 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 		/* swap back endianness */
 		bm_lel_to_cpu(b);
 		/* flush bitmap to stable storage */
-		if (!test_bit(MD_NO_BARRIER, &mdev->flags))
+		if (!test_bit(MD_NO_BARRIER,&mdev->flags))
 			blkdev_issue_flush(mdev->bc->md_bdev, NULL);
 	} else /* rw == READ */ {
 		/* just read, if neccessary adjust endianness */
@@ -719,6 +688,12 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 	}
 	now = b->bm_set;
 
+	/* ok, done,
+	 * now it is visible again
+	 */
+
+	mdev->bitmap = b;
+
 	INFO("%s (%lu bits) marked out-of-sync by on disk bit-map.\n",
 	     ppsize(ppb, now << (BM_BLOCK_SIZE_B-10)), now);
 
@@ -780,14 +755,14 @@ int drbd_bm_write_sect(struct drbd_conf *mdev, unsigned long enr) __must_hold(lo
 #endif
 	if (num_words < S2W(1))
 		memset(page_address(mdev->md_io_page), 0, MD_HARDSECT);
-	drbd_bm_get_lel(mdev, offset, num_words,
-			page_address(mdev->md_io_page));
+	drbd_bm_get_lel( mdev, offset, num_words,
+			 page_address(mdev->md_io_page) );
 	if (!drbd_md_sync_page_io(mdev, mdev->bc, on_disk_sector, WRITE)) {
 		int i;
 		err = -EIO;
-		ERR("IO ERROR writing bitmap sector %lu "
-		    "(meta-disk sector %llus)\n",
-		    enr, (unsigned long long)on_disk_sector);
+		ERR( "IO ERROR writing bitmap sector %lu "
+		     "(meta-disk sector %llus)\n",
+		     enr, (unsigned long long)on_disk_sector );
 		drbd_chk_io_error(mdev, 1, TRUE);
 		drbd_io_error(mdev, TRUE);
 		for (i = 0; i < AL_EXT_PER_BM_SECT; i++)
@@ -805,8 +780,6 @@ void drbd_bm_reset_find(struct drbd_conf *mdev)
 	ERR_IF(!b) return;
 
 	spin_lock_irq(&b->bm_lock);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 	b->bm_fo = 0;
 	spin_unlock_irq(&b->bm_lock);
 
@@ -826,8 +799,6 @@ unsigned long drbd_bm_find_next(struct drbd_conf *mdev)
 	ERR_IF(!b->bm) return i;
 
 	spin_lock_irq(&b->bm_lock);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 	if (b->bm_fo < b->bm_bits)
 		i = find_next_bit(b->bm, b->bm_bits, b->bm_fo);
 	else if (b->bm_fo > b->bm_bits)
@@ -857,8 +828,7 @@ void drbd_bm_set_find(struct drbd_conf *mdev, unsigned long i)
 
 int drbd_bm_rs_done(struct drbd_conf *mdev)
 {
-	D_ASSERT(mdev->bitmap);
-	return mdev->bitmap->bm_fo >= mdev->bitmap->bm_bits;
+	return (mdev->bitmap->bm_fo >= mdev->bitmap->bm_bits);
 }
 
 /* returns number of bits actually changed.
@@ -875,9 +845,7 @@ static int bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
 	ERR_IF(!b) return 1;
 	ERR_IF(!b->bm) return 1;
 
-	spin_lock_irqsave(&b->bm_lock, flags);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
+	spin_lock_irqsave(&b->bm_lock,flags);
 	for (bitnr = s; bitnr <= e; bitnr++) {
 		ERR_IF (bitnr >= b->bm_bits) {
 			ERR("bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
@@ -921,8 +889,6 @@ int drbd_bm_test_bit(struct drbd_conf *mdev, const unsigned long bitnr)
 	ERR_IF(!b->bm) return 0;
 
 	spin_lock_irqsave(&b->bm_lock, flags);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 	if (bitnr < b->bm_bits) {
 		i = test_bit(bitnr, b->bm) ? 1 : 0;
 	} else if (bitnr == b->bm_bits) {
@@ -951,15 +917,15 @@ int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsi
 	ERR_IF(!b) return 1;
 	ERR_IF(!b->bm) return 1;
 
-	spin_lock_irqsave(&b->bm_lock, flags);
-	for (bitnr = s; bitnr <= e; bitnr++) {
+	spin_lock_irqsave(&b->bm_lock,flags);
+	for (bitnr = s; bitnr <=e; bitnr++) {
 		ERR_IF (bitnr >= b->bm_bits) {
-			ERR("bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
+			ERR("bitnr=%lu bm_bits=%lu\n",bitnr, b->bm_bits);
 		} else {
 			c += (0 != test_bit(bitnr, b->bm));
 		}
 	}
-	spin_unlock_irqrestore(&b->bm_lock, flags);
+	spin_unlock_irqrestore(&b->bm_lock,flags);
 	return c;
 }
 
@@ -987,8 +953,6 @@ int drbd_bm_e_weight(struct drbd_conf *mdev, unsigned long enr)
 	ERR_IF(!b) return 0;
 	ERR_IF(!b->bm) return 0;
 	spin_lock_irqsave(&b->bm_lock, flags);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 
 	s = S2W(enr);
 	e = min((size_t)S2W(enr+1), b->bm_words);
@@ -996,8 +960,7 @@ int drbd_bm_e_weight(struct drbd_conf *mdev, unsigned long enr)
 	if (s < b->bm_words) {
 		const unsigned long *w = b->bm+s;
 		int n = e-s;
-		while (n--)
-			count += hweight_long(*w++);
+		while (n--) count += hweight_long(*w++);
 	} else {
 		ERR("start offset (%d) too large in drbd_bm_e_weight\n", s);
 	}
@@ -1018,8 +981,6 @@ unsigned long drbd_bm_ALe_set_all(struct drbd_conf *mdev, unsigned long al_enr)
 	ERR_IF(!b->bm) return 0;
 
 	spin_lock_irq(&b->bm_lock);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 	weight = b->bm_set;
 
 	s = al_enr * BM_WORDS_PER_AL_EXT;
@@ -1028,8 +989,7 @@ unsigned long drbd_bm_ALe_set_all(struct drbd_conf *mdev, unsigned long al_enr)
 	if (s < b->bm_words) {
 		const unsigned long *w = b->bm+s;
 		int n = e-s;
-		while (n--)
-			count += hweight_long(*w++);
+		while (n--) count += hweight_long(*w++);
 		n = e-s;
 		memset(b->bm+s, -1, n*sizeof(long));
 		b->bm_set += n*BITS_PER_LONG - count;
