@@ -57,24 +57,23 @@
 /* Compatibility for older kernels */
 #ifndef __acquires
 # ifdef __CHECKER__
-#  define __acquires(x)	__attribute__((context(x,0,1)))
-#  define __releases(x)	__attribute__((context(x,1,0)))
-#  define __acquire(x)	__context__(x,1)
-#  define __release(x)	__context__(x,-1)
-#  define __cond_lock(x,c)	((c) ? ({ __acquire(x); 1; }) : 0)
+#  define __acquires(x) __attribute__((context(x,0,1)))
+#  define __releases(x) __attribute__((context(x,1,0)))
+#  define __acquire(x)  __context__(x,1)
+#  define __release(x)  __context__(x,-1)
+#  define __cond_lock(x,c)      ((c) ? ({ __acquire(x); 1; }) : 0)
 # else
 #  define __acquires(x)
 #  define __releases(x)
-#  define __acquire(x)	(void)0
-#  define __release(x)	(void)0
+#  define __acquire(x)  (void)0
+#  define __release(x)  (void)0
 #  define __cond_lock(x,c) (c)
 # endif
 #endif
 
 /* module parameter, defined in drbd_main.c */
-extern unsigned int minor_count;
+extern int minor_count;
 extern int allow_oos;
-extern unsigned int cn_idx;
 
 #ifdef DRBD_ENABLE_FAULTS
 extern int enable_faults;
@@ -92,7 +91,7 @@ extern char usermode_helper[];
 #include <linux/blkdev.h>
 #include <linux/bio.h>
 
-/* XXX do we need this? */
+// XXX do we need this?
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -120,6 +119,10 @@ extern char usermode_helper[];
  * freshly allocated EEs get !ID_VACANT (== 1)
  * so if it says "cannot dereference null pointer at adress 0x00000001",
  * it is most likely one of these :( */
+
+#define ID_IN_SYNC      (4711ULL)
+#define ID_OUT_OF_SYNC  (4712ULL)
+
 #define ID_SYNCER (-1ULL)
 #define ID_VACANT 0
 #define is_syncer_block_id(id) ((id) == ID_SYNCER)
@@ -143,11 +146,11 @@ struct drbd_conf;
  *************************/
 
 /* handy macro: DUMPP(somepointer) */
-#define DUMPP(A)   ERR(#A " = %p in %s:%d\n", (A), __FILE__, __LINE__);
-#define DUMPLU(A)  ERR(#A " = %lu in %s:%d\n", (unsigned long)(A), __FILE__, __LINE__);
-#define DUMPLLU(A) ERR(#A " = %llu in %s:%d\n", (unsigned long long)(A), __FILE__, __LINE__);
-#define DUMPLX(A)  ERR(#A " = %lx in %s:%d\n", (A), __FILE__, __LINE__);
-#define DUMPI(A)   ERR(#A " = %d in %s:%d\n", (int)(A), __FILE__, __LINE__);
+#define DUMPP(A)   ERR( #A " = %p in %s:%d\n", (A), __FILE__, __LINE__);
+#define DUMPLU(A)  ERR( #A " = %lu in %s:%d\n", (unsigned long)(A), __FILE__, __LINE__);
+#define DUMPLLU(A) ERR( #A " = %llu in %s:%d\n", (unsigned long long)(A), __FILE__, __LINE__);
+#define DUMPLX(A)  ERR( #A " = %lx in %s:%d\n", (A), __FILE__, __LINE__);
+#define DUMPI(A)   ERR( #A " = %d in %s:%d\n", (int)(A), __FILE__, __LINE__);
 
 #define DUMPST(A) DUMPLLU((unsigned long long)(A))
 
@@ -165,14 +168,15 @@ struct drbd_conf;
 #define D_DUMPI(A)
 #endif
 
+/* Info: do not remove the spaces around the "," before ##
+ *	 Otherwise this is not portable from gcc-2.95 to gcc-3.3 */
 #define PRINTK(level, fmt, args...) \
 	printk(level "drbd%d: " fmt, \
 		mdev->minor , ##args)
 
 #define ALERT(fmt, args...) PRINTK(KERN_ALERT, fmt , ##args)
 #define ERR(fmt, args...)   PRINTK(KERN_ERR, fmt , ##args)
-/* nowadays, WARN() is defined as BUG() without crash in bug.h */
-#define drbd_WARN(fmt, args...)  PRINTK(KERN_WARNING, fmt , ##args)
+#define WARN(fmt, args...)  PRINTK(KERN_WARNING, fmt , ##args)
 #define INFO(fmt, args...)  PRINTK(KERN_INFO, fmt , ##args)
 #define DBG(fmt, args...)   PRINTK(KERN_DEBUG, fmt , ##args)
 
@@ -197,8 +201,8 @@ struct drbd_conf;
 		missed = 0;					\
 		toks -= ratelimit_jiffies;			\
 		if (lost)					\
-			drbd_WARN("%d messages suppressed in %s:%d.\n", \
-				lost, __FILE__, __LINE__);	\
+			WARN("%d messages suppressed in %s:%d.\n", \
+				lost , __FILE__ , __LINE__ );	\
 		__ret = 1;					\
 	} else {						\
 		missed++;					\
@@ -209,7 +213,7 @@ struct drbd_conf;
 
 
 #ifdef DBG_ASSERTS
-extern void drbd_assert_breakpoint(struct drbd_conf *, char *, char *, int);
+extern void drbd_assert_breakpoint(struct drbd_conf *, char *, char *, int );
 # define D_ASSERT(exp)	if (!(exp)) \
 	 drbd_assert_breakpoint(mdev, #exp, __FILE__, __LINE__)
 #else
@@ -217,9 +221,9 @@ extern void drbd_assert_breakpoint(struct drbd_conf *, char *, char *, int);
 	 ERR("ASSERT( " #exp " ) in %s:%d\n", __FILE__, __LINE__)
 #endif
 #define ERR_IF(exp) if (({				\
-	int _b = (exp) != 0;				\
+	int _b = (exp)!=0;				\
 	if (_b) ERR("%s: (%s) in %s:%d\n",		\
-		__func__, #exp, __FILE__, __LINE__);	\
+		__func__, #exp, __FILE__,__LINE__);	\
 	 _b;						\
 	}))
 
@@ -241,9 +245,9 @@ extern unsigned int
 _drbd_insert_fault(struct drbd_conf *mdev, unsigned int type);
 static inline int
 drbd_insert_fault(struct drbd_conf *mdev, unsigned int type) {
-    return fault_rate &&
+    return (fault_rate &&
 	    (enable_faults & (1<<type)) &&
-	    _drbd_insert_fault(mdev, type);
+	    _drbd_insert_fault(mdev, type));
 }
 #define FAULT_ACTIVE(_m, _t) (drbd_insert_fault((_m), (_t)))
 
@@ -253,9 +257,9 @@ drbd_insert_fault(struct drbd_conf *mdev, unsigned int type) {
 
 #include <linux/stringify.h>
 /* integer division, round _UP_ to the next integer */
-#define div_ceil(A, B) ((A)/(B) + ((A)%(B) ? 1 : 0))
+#define div_ceil(A, B) ( (A)/(B) + ((A)%(B) ? 1 : 0) )
 /* usual integer division */
-#define div_floor(A, B) ((A)/(B))
+#define div_floor(A, B) ( (A)/(B) )
 
 /*
  * Compatibility Section
@@ -266,7 +270,7 @@ drbd_insert_fault(struct drbd_conf *mdev, unsigned int type) {
 #define RECALC_SIGPENDING()	    recalc_sigpending();
 
 #if defined(DBG_SPINLOCKS) && defined(__SMP__)
-# define MUST_HOLD(lock) if (!spin_is_locked(lock)) ERR("Not holding lock! in %s\n", __func__);
+# define MUST_HOLD(lock) if (!spin_is_locked(lock)) { ERR("Not holding lock! in %s\n", __FUNCTION__ ); }
 #else
 # define MUST_HOLD(lock)
 #endif
@@ -286,8 +290,8 @@ drbd_insert_fault(struct drbd_conf *mdev, unsigned int type) {
 	({ typecheck(struct drbd_conf*, x); \
 	  (x)->magic = (long)(x) ^ DRBD_MAGIC; })
 #define IS_VALID_MDEV(x)  \
-	(typecheck(struct drbd_conf*, x) && \
-	  ((x) ? (((x)->magic ^ DRBD_MAGIC) == (long)(x)) : 0))
+	( typecheck(struct drbd_conf*, x) && \
+	  ((x) ? (((x)->magic ^ DRBD_MAGIC) == (long)(x)):0))
 
 /* drbd_meta-data.c (still in drbd_main.c) */
 /* 4th incarnation of the disk layout. */
@@ -334,6 +338,10 @@ enum Drbd_Packet_Cmd {
 	StateChgReply,
 	LAST_ASENDER_CMD = StateChgReply,
 
+	OVRequest,
+	OVReply,
+	OVResult, // Exception to the FIRST/LAST ASENDER_CMD
+
 	MAX_CMD,
 	MayIgnore = 0x100, /* Flag to test if (cmd > MayIgnore) ... */
 	MAX_OPT_CMD,
@@ -382,24 +390,27 @@ static inline const char *cmdname(enum Drbd_Packet_Cmd cmd)
 		[NegRSDReply]	   = "NegRSDReply",
 		[BarrierAck]	   = "BarrierAck",
 		[StateChgRequest]  = "StateChgRequest",
-		[StateChgReply]    = "StateChgReply"
+		[StateChgReply]    = "StateChgReply",
+		[OVRequest]        = "OVRequest",
+		[OVReply]          = "OVReply",
+		[OVResult]         = "OVResult",
 	};
 
 	if (Data > cmd || cmd >= MAX_CMD) {
-		switch (cmd) {
-		case HandShakeM:
-			return "HandShakeM";
-			break;
-		case HandShakeS:
-			return "HandShakeS";
-			break;
-		case HandShake:
-			return "HandShake";
-			break;
-		default:
-			return "Unknown";
-			break;
-		}
+	    switch (cmd) {
+	    case HandShakeM:
+		return "HandShakeM";
+		break;
+	    case HandShakeS:
+		return "HandShakeS";
+		break;
+	    case HandShake:
+		return "HandShake";
+		break;
+	    default:
+		return "Unknown";
+		break;
+	    }
 	}
 	return cmdnames[cmd];
 }
@@ -486,14 +497,16 @@ struct Drbd_BlockRequest_Packet {
 
 struct Drbd_HandShake_Packet {
 	struct Drbd_Header head;	/* 8 bytes */
-	u32 protocol_version;
+	u32 protocol_min;
 	u32 feature_flags;
+	u32 protocol_max;
 
 	/* should be more than enough for future enhancements
 	 * for now, feature_flags and the reserverd array shall be zero.
 	 */
 
-	u64 reserverd[8];
+	u32 _pad;
+	u64 reserverd[7];
 } __attribute((packed));
 /* 80 bytes, FIXED for the next century */
 
@@ -512,6 +525,9 @@ struct Drbd_BarrierAck_Packet {
 struct Drbd_SyncParam_Packet {
 	struct Drbd_Header head;
 	u32 rate;
+
+	      /* Since protocol version 88 and higher. */
+	char online_verify_alg[0];
 } __attribute((packed));
 
 struct Drbd_Protocol_Packet {
@@ -522,6 +538,10 @@ struct Drbd_Protocol_Packet {
 	u32 after_sb_2p;
 	u32 want_lose;
 	u32 two_primaries;
+
+              /* Since protocol version 87 and higher. */
+	char integrity_alg[0];
+
 } __attribute((packed));
 
 struct Drbd_GenCnt_Packet {
@@ -594,7 +614,6 @@ union Drbd_Polymorph_Packet {
 } __attribute((packed));
 
 /**********************************************************************/
-
 enum Drbd_thread_state {
 	None,
 	Running,
@@ -609,6 +628,7 @@ struct Drbd_thread {
 	enum Drbd_thread_state t_state;
 	int (*function) (struct Drbd_thread *);
 	struct drbd_conf *mdev;
+	struct mutex task_mutex;
 };
 
 static inline enum Drbd_thread_state get_t_state(struct Drbd_thread *thi)
@@ -700,6 +720,11 @@ struct Tl_epoch_entry {
 	u64    block_id;
 };
 
+struct digest_info {
+	int digest_size;
+	void *digest;
+};
+
 /* ee flag bits */
 enum {
 	__EE_CALL_AL_COMPLETE_IO,
@@ -737,11 +762,8 @@ enum {
 				   so don't even try */
 	MD_NO_BARRIER,		/* meta data device does not support barriers,
 				   so don't even try */
-	SUSPEND_IO,		/* suspend application io */
-	BITMAP_IO,		/* suspend application io;
-				   once no more io in flight, start bitmap io */
+	BITMAP_IO,		/* Let user IO drain */
 	BITMAP_IO_QUEUED,       /* Started bitmap IO */
-	RESYNC_AFTER_NEG,       /* Resync after online grow after the attach&negotiate finished. */
 };
 
 struct drbd_bitmap; /* opaque for drbd_conf */
@@ -770,7 +792,7 @@ struct drbd_work_queue {
 struct drbd_socket {
 	struct drbd_work_queue work;
 	struct semaphore  mutex;
-	struct socket	 *socket;
+	struct socket    *socket;
 	/* this way we get our
 	 * send/receive buffers off the stack */
 	union Drbd_Polymorph_Packet sbuf;
@@ -795,11 +817,11 @@ struct drbd_md {
 	 */
 };
 
-/* for sync_conf and other types... */
+// for sync_conf and other types...
 #define NL_PACKET(name, number, fields) struct name { fields };
 #define NL_INTEGER(pn,pr,member) int member;
 #define NL_INT64(pn,pr,member) __u64 member;
-#define NL_BIT(pn,pr,member)   unsigned member:1;
+#define NL_BIT(pn,pr,member)   unsigned member : 1;
 #define NL_STRING(pn,pr,member,len) unsigned char member[len]; int member ## _len;
 #include "linux/drbd_nl.h"
 
@@ -821,7 +843,6 @@ struct drbd_md_io {
 
 struct bm_io_work {
 	struct drbd_work w;
-	char *why;
 	int (*io_fn)(struct drbd_conf *mdev);
 	void (*done)(struct drbd_conf *mdev, int rv);
 };
@@ -845,6 +866,7 @@ struct drbd_conf {
 
 	struct drbd_socket data; /* data/barrier/cstate/parameter packets */
 	struct drbd_socket meta; /* ping/ack (metadata) packets */
+	int agreed_pro_version;  /* actually used protocol version */
 	unsigned long last_received; /* in jiffies, either socket */
 	unsigned int ko_count;
 	struct drbd_work  resync_work,
@@ -875,7 +897,6 @@ struct drbd_conf {
 	struct drbd_barrier *unused_spare_barrier; /* for pre-allocation */
 	struct drbd_barrier *newest_barrier;
 	struct drbd_barrier *oldest_barrier;
-	struct list_head out_of_sequence_requests;
 	struct hlist_head *tl_hash;
 	unsigned int tl_hash_s;
 
@@ -892,6 +913,12 @@ struct drbd_conf {
 	/* marks's time [unit jiffies] */
 	unsigned long rs_mark_time;
 
+	sector_t ov_position;
+	sector_t ov_last_oos_start; /* Start sector of out of sync range */
+	sector_t ov_last_oos_size;  /* size of out-of-sync range in sectors */
+	unsigned long ov_left;
+	struct crypto_hash *verify_tfm;
+
 	struct Drbd_thread receiver;
 	struct Drbd_thread worker;
 	struct Drbd_thread asender;
@@ -935,18 +962,24 @@ struct drbd_conf {
 	int al_tr_cycle;
 	int al_tr_pos;   /* position of the next transaction in the journal */
 	struct crypto_hash *cram_hmac_tfm;
+	struct crypto_hash *integrity_w_tfm; /* to be used by the worker thread */
+	struct crypto_hash *integrity_r_tfm; /* to be used by the receiver thread */
+	void *int_dig_out;
+	void *int_dig_in;
+	void *int_dig_vv;
 	wait_queue_head_t seq_wait;
 	atomic_t packet_seq;
 	unsigned int peer_seq;
 	spinlock_t peer_seq_lock;
 	int minor;
 	unsigned long comm_bm_set; /* communicated number of set bits. */
+	cpumask_t cpu_mask;
 	struct bm_io_work bm_io_work;
 	u64 ed_uuid; /* UUID of the exposed data */
 	struct mutex state_mutex;
 };
 
-static inline struct drbd_conf *minor_to_mdev(unsigned int minor)
+static inline struct drbd_conf *minor_to_mdev(int minor)
 {
 	struct drbd_conf *mdev;
 
@@ -1009,9 +1042,16 @@ extern int _drbd_request_state(struct drbd_conf *, union drbd_state_t,
 extern int _drbd_set_state(struct drbd_conf *, union drbd_state_t,
 			   enum chg_state_flags, struct completion *done);
 extern void print_st_err(struct drbd_conf *, union drbd_state_t,
-			union drbd_state_t, int);
+			union drbd_state_t, int );
 extern int  drbd_thread_start(struct Drbd_thread *thi);
 extern void _drbd_thread_stop(struct Drbd_thread *thi, int restart, int wait);
+#ifdef CONFIG_SMP
+extern void drbd_thread_set_cpu(struct Drbd_thread *thi, cpumask_t cpu_mask);
+extern cpumask_t drbd_calc_cpu_mask(struct drbd_conf *mdev);
+#else
+#define drbd_thread_set_cpu(A,B) ({})
+#define drbd_calc_cpu_mask(A) CPU_MASK_ALL
+#endif
 extern void drbd_free_resources(struct drbd_conf *mdev);
 extern void tl_release(struct drbd_conf *mdev, unsigned int barrier_nr,
 		       unsigned int set_size);
@@ -1046,6 +1086,8 @@ extern int drbd_send_ack_rp(struct drbd_conf *mdev, enum Drbd_Packet_Cmd cmd,
 			struct Drbd_BlockRequest_Packet *rp);
 extern int drbd_send_ack_dp(struct drbd_conf *mdev, enum Drbd_Packet_Cmd cmd,
 			struct Drbd_Data_Packet *dp);
+extern int drbd_send_ack_ex(struct drbd_conf *mdev, enum Drbd_Packet_Cmd cmd,
+			    sector_t sector, int blksize, u64 block_id);
 extern int _drbd_send_page(struct drbd_conf *mdev, struct page *page,
 			int offset, size_t size);
 extern int drbd_send_block(struct drbd_conf *mdev, enum Drbd_Packet_Cmd cmd,
@@ -1055,6 +1097,12 @@ extern int _drbd_send_barrier(struct drbd_conf *mdev,
 			struct drbd_barrier *barrier);
 extern int drbd_send_drequest(struct drbd_conf *mdev, int cmd,
 			      sector_t sector, int size, u64 block_id);
+extern int drbd_send_drequest_csum(struct drbd_conf *mdev,
+				   sector_t sector,int size,
+				   void *digest, int digest_size,
+				   enum Drbd_Packet_Cmd cmd);
+extern int drbd_send_ov_request(struct drbd_conf *mdev,sector_t sector,int size);
+
 extern int drbd_send_bitmap(struct drbd_conf *mdev);
 extern int _drbd_send_bitmap(struct drbd_conf *mdev);
 extern int drbd_send_sr_reply(struct drbd_conf *mdev, int retcode);
@@ -1077,10 +1125,9 @@ extern int drbd_md_test_flag(struct drbd_backing_dev *, int);
 extern void drbd_md_mark_dirty(struct drbd_conf *mdev);
 extern void drbd_queue_bitmap_io(struct drbd_conf *mdev,
 				 int (*io_fn)(struct drbd_conf *),
-				 void (*done)(struct drbd_conf *, int),
-				 char *why);
+				 void (*done)(struct drbd_conf *, int));
 extern int drbd_bmio_set_n_write(struct drbd_conf *mdev);
-extern int drbd_bitmap_io(struct drbd_conf *mdev, int (*io_fn)(struct drbd_conf *), char *why);
+extern int drbd_bitmap_io(struct drbd_conf *mdev, int (*io_fn)(struct drbd_conf *));
 
 
 /* Meta data layout
@@ -1088,7 +1135,7 @@ extern int drbd_bitmap_io(struct drbd_conf *mdev, int (*io_fn)(struct drbd_conf
    * either at the end of the backing device
    * or on a seperate meta data device. */
 
-#define MD_RESERVED_SECT (128LU << 11)  /* 128 MB, unit sectors */
+#define MD_RESERVED_SECT ( 128LU << 11 )  /* 128 MB, unit sectors */
 /* The following numbers are sectors */
 #define MD_AL_OFFSET 8	    /* 8 Sectors after start of meta area */
 #define MD_AL_MAX_SIZE 64   /* = 32 kb LOG  ~ 3776 extents ~ 14 GB Storage */
@@ -1140,7 +1187,7 @@ struct bm_extent {
 #define BM_BLOCK_SIZE	 (1<<BM_BLOCK_SIZE_B)
 /* (9+3) : 512 bytes @ 8 bits; representing 16M storage
  * per sector of on disk bitmap */
-#define BM_EXT_SIZE_B	 (BM_BLOCK_SIZE_B + MD_HARDSECT_B + 3)  /* = 24 */
+#define BM_EXT_SIZE_B	 (BM_BLOCK_SIZE_B + MD_HARDSECT_B + 3 )  /* = 24 */
 #define BM_EXT_SIZE	 (1<<BM_EXT_SIZE_B)
 
 #if (BM_EXT_SIZE_B != 24) || (BM_BLOCK_SIZE_B != 12)
@@ -1163,12 +1210,12 @@ struct bm_extent {
 #define BM_SECT_PER_EXT     (1ULL << (BM_EXT_SIZE_B-9))
 
 /* in one sector of the bitmap, we have this many activity_log extents. */
-#define AL_EXT_PER_BM_SECT  (1 << (BM_EXT_SIZE_B - AL_EXTENT_SIZE_B))
+#define AL_EXT_PER_BM_SECT  (1 << (BM_EXT_SIZE_B - AL_EXTENT_SIZE_B) )
 #define BM_WORDS_PER_AL_EXT (1 << (AL_EXTENT_SIZE_B-BM_BLOCK_SIZE_B-LN2_BPL))
 
 
-#define BM_BLOCKS_PER_BM_EXT_B (BM_EXT_SIZE_B - BM_BLOCK_SIZE_B)
-#define BM_BLOCKS_PER_BM_EXT_MASK  ((1<<BM_BLOCKS_PER_BM_EXT_B) - 1)
+#define BM_BLOCKS_PER_BM_EXT_B ( BM_EXT_SIZE_B - BM_BLOCK_SIZE_B )
+#define BM_BLOCKS_PER_BM_EXT_MASK  ( (1<<BM_BLOCKS_PER_BM_EXT_B) - 1 )
 
 /* I want the packet to fit within one page
  * THINK maybe use a special bitmap header,
@@ -1176,10 +1223,6 @@ struct bm_extent {
  * Do not use PAGE_SIZE here! Use a architecture agnostic constant!
  */
 #define BM_PACKET_WORDS ((4096-sizeof(struct Drbd_Header))/sizeof(long))
-#if (PAGE_SIZE < 4096)
-/* drbd_send_bitmap / receive_bitmap would break horribly */
-#error "PAGE_SIZE too small"
-#endif
 
 /* the extent in "PER_EXTENT" below is an activity log extent
  * we need that many (long words/bytes) to store the bitmap
@@ -1197,17 +1240,17 @@ struct bm_extent {
 
 #define DRBD_MAX_SECTORS_32 (0xffffffffLU)
 #define DRBD_MAX_SECTORS_BM \
-	  ((MD_RESERVED_SECT - MD_BM_OFFSET) * (1LL<<(BM_EXT_SIZE_B-9)))
+	  ( (MD_RESERVED_SECT - MD_BM_OFFSET) * (1LL<<(BM_EXT_SIZE_B-9)) )
 #if DRBD_MAX_SECTORS_BM < DRBD_MAX_SECTORS_32
 #define DRBD_MAX_SECTORS      DRBD_MAX_SECTORS_BM
 #define DRBD_MAX_SECTORS_FLEX DRBD_MAX_SECTORS_BM
-#elif !defined(CONFIG_LBD) && BITS_PER_LONG == 32
+#elif   !defined(CONFIG_LBD) && BITS_PER_LONG == 32
 #define DRBD_MAX_SECTORS      DRBD_MAX_SECTORS_32
 #define DRBD_MAX_SECTORS_FLEX DRBD_MAX_SECTORS_32
 #else
 #define DRBD_MAX_SECTORS      DRBD_MAX_SECTORS_BM
-/* 8 TB in units of sectors */
-#define DRBD_MAX_SECTORS_FLEX (1ULL<<(31+BM_BLOCK_SIZE_B-9))
+/* 16 TB in units of sectors */
+#define DRBD_MAX_SECTORS_FLEX (1ULL<<(32+BM_BLOCK_SIZE_B-9))
 #endif
 
 /* Sector shift value for the "hash" functions of tl_hash and ee_hash tables.
@@ -1236,6 +1279,7 @@ extern int  drbd_bm_write(struct drbd_conf *mdev) __must_hold(local);
 extern unsigned long drbd_bm_ALe_set_all(struct drbd_conf *mdev,
 		unsigned long al_enr);
 extern size_t	     drbd_bm_words(struct drbd_conf *mdev);
+extern unsigned long drbd_bm_bits(struct drbd_conf *mdev);
 extern sector_t      drbd_bm_capacity(struct drbd_conf *mdev);
 extern unsigned long drbd_bm_find_next(struct drbd_conf *mdev);
 extern void drbd_bm_set_find(struct drbd_conf *mdev, unsigned long i);
@@ -1248,12 +1292,13 @@ extern void drbd_bm_merge_lel(struct drbd_conf *mdev, size_t offset,
 extern void drbd_bm_get_lel(struct drbd_conf *mdev, size_t offset,
 		size_t number, unsigned long *buffer);
 
-extern void drbd_bm_lock(struct drbd_conf *mdev, char *why);
+extern void __drbd_bm_lock(struct drbd_conf *mdev, char *file, int line);
 extern void drbd_bm_unlock(struct drbd_conf *mdev);
+#define drbd_bm_lock(mdev)    __drbd_bm_lock(mdev, __FILE__, __LINE__ )
 
 extern void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line);
 #define drbd_bm_recount_bits(mdev) \
-	_drbd_bm_recount_bits(mdev, __FILE__, __LINE__)
+	_drbd_bm_recount_bits(mdev, __FILE__, __LINE__ )
 extern int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e);
 /* drbd_main.c */
 
@@ -1261,6 +1306,7 @@ extern int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, con
  * because of kmem_cache_t weirdness */
 #include "drbd_wrappers.h"
 
+extern int minor_count;
 extern struct kmem_cache *drbd_request_cache;
 extern struct kmem_cache *drbd_ee_cache;
 extern mempool_t *drbd_request_mempool;
@@ -1276,6 +1322,7 @@ extern struct drbd_conf *drbd_new_device(int minor);
 /* Dynamic tracing framework */
 #ifdef ENABLE_DYNAMIC_TRACE
 
+extern int proc_details;
 extern int trace_type;
 extern int trace_devs;
 extern int trace_level;
@@ -1298,17 +1345,16 @@ enum {
 	TraceTypeNl	= 0x00000040,
 	TraceTypeALExts = 0x00000080,
 	TraceTypeIntRq  = 0x00000100,
-	TraceTypeMDIO   = 0x00000200,
 };
 
 static inline int
 is_trace(unsigned int type, unsigned int level) {
-	return (trace_level >= level) && (type & trace_type);
+	return ((trace_level >= level) && (type & trace_type));
 }
 static inline int
 is_mdev_trace(struct drbd_conf *mdev, unsigned int type, unsigned int level) {
-	return is_trace(type, level) &&
-		((1 << mdev_to_minor(mdev)) & trace_devs);
+	return (is_trace(type, level) &&
+		( ( 1 << mdev_to_minor(mdev)) & trace_devs));
 }
 
 #define MTRACE(type, lvl, code...) \
@@ -1352,9 +1398,8 @@ static inline void dump_bio(struct drbd_conf *mdev,
 		);
 }
 
-static inline void dump_internal_bio(const char *pfx, struct drbd_conf *mdev, struct bio *bio, int complete)
-{
-	MTRACE(TraceTypeIntRq, TraceLvlSummary,
+static inline void dump_internal_bio(const char *pfx, struct drbd_conf *mdev, struct bio *bio, int complete) {
+	MTRACE(TraceTypeIntRq,TraceLvlSummary,
 	       _dump_bio(pfx, mdev, bio, complete);
 		);
 }
@@ -1386,19 +1431,11 @@ dump_packet(struct drbd_conf *mdev, struct socket *sock,
 /* drbd_req */
 extern int drbd_make_request_26(struct request_queue *q, struct bio *bio);
 extern int drbd_read_remote(struct drbd_conf *mdev, struct drbd_request *req);
-extern int drbd_merge_bvec(struct request_queue *q,
-#ifdef HAVE_bvec_merge_data
-		struct bvec_merge_data *bvm,
-#else
-		struct bio *bvm,
-#endif
-		struct bio_vec *bvec);
+extern int drbd_merge_bvec(struct request_queue *, struct bio *, struct bio_vec *);
 extern int is_valid_ar_handle(struct drbd_request *, sector_t);
 
 
 /* drbd_nl.c */
-extern void drbd_suspend_io(struct drbd_conf *mdev);
-extern void drbd_resume_io(struct drbd_conf *mdev);
 extern char *ppsize(char *buf, unsigned long long size);
 extern sector_t drbd_new_dev_size(struct drbd_conf *,
 		struct drbd_backing_dev *);
@@ -1424,11 +1461,28 @@ extern int drbd_resync_finished(struct drbd_conf *mdev);
 /* maybe rather drbd_main.c ? */
 extern int drbd_md_sync_page_io(struct drbd_conf *mdev,
 		struct drbd_backing_dev *bdev, sector_t sector, int rw);
+extern void drbd_ov_oos_found(struct drbd_conf*, sector_t, int);
+
+static inline void ov_oos_print(struct drbd_conf *mdev)
+{
+	if (mdev->ov_last_oos_size) {
+		ERR("Out of sync: start=%llu, size=%lu (sectors)\n",
+		     (unsigned long long)mdev->ov_last_oos_start,
+		     (unsigned long)mdev->ov_last_oos_size);
+	}
+	mdev->ov_last_oos_size=0;
+}
+
+
+void drbd_csum(struct drbd_conf *, struct crypto_hash *, struct bio *, void *);
 /* worker callbacks */
 extern int w_req_cancel_conflict(struct drbd_conf *, struct drbd_work *, int);
 extern int w_read_retry_remote(struct drbd_conf *, struct drbd_work *, int);
 extern int w_e_end_data_req(struct drbd_conf *, struct drbd_work *, int);
 extern int w_e_end_rsdata_req(struct drbd_conf *, struct drbd_work *, int);
+extern int w_e_end_ov_reply(struct drbd_conf *, struct drbd_work *, int);
+extern int w_e_end_ov_req(struct drbd_conf *, struct drbd_work *, int);
+extern int w_ov_finished(struct drbd_conf *, struct drbd_work *, int);
 extern int w_resync_inactive(struct drbd_conf *, struct drbd_work *, int);
 extern int w_resume_next_sg(struct drbd_conf *, struct drbd_work *, int);
 extern int w_io_error(struct drbd_conf *, struct drbd_work *, int);
@@ -1456,46 +1510,16 @@ extern void _drbd_wait_ee_list_empty(struct drbd_conf *mdev,
 extern void drbd_set_recv_tcq(struct drbd_conf *mdev, int tcq_enabled);
 extern void _drbd_clear_done_ee(struct drbd_conf *mdev);
 
-/* yes, there is kernel_setsockopt, but only since 2.6.18. we don't need to
- * mess with get_fs/set_fs, we know we are KERNEL_DS always. */
-static inline int drbd_setsockopt(struct socket *sock, int level, int optname,
-			char __user *optval, int optlen)
-{
-	int err;
-	if (level == SOL_SOCKET)
-		err = sock_setsockopt(sock, level, optname, optval, optlen);
-	else
-		err = sock->ops->setsockopt(sock, level, optname, optval,
-					    optlen);
-	return err;
-}
-
 static inline void drbd_tcp_cork(struct socket *sock)
 {
 	int __user val = 1;
-	(void) drbd_setsockopt(sock, SOL_TCP, TCP_CORK,
-			(char __user *)&val, sizeof(val));
+	tcp_setsockopt(sock->sk, SOL_TCP, TCP_CORK, (char __user *)&val, sizeof(val) );
 }
 
-static inline void drbd_tcp_uncork(struct socket *sock)
+static inline void drbd_tcp_flush(struct socket *sock)
 {
 	int __user val = 0;
-	(void) drbd_setsockopt(sock, SOL_TCP, TCP_CORK,
-			(char __user *)&val, sizeof(val));
-}
-
-static inline void drbd_tcp_nodelay(struct socket *sock)
-{
-	int __user val = 1;
-	(void) drbd_setsockopt(sock, SOL_TCP, TCP_NODELAY,
-			(char __user *)&val, sizeof(val));
-}
-
-static inline void drbd_tcp_quickack(struct socket *sock)
-{
-	int __user val = 1;
-	(void) drbd_setsockopt(sock, SOL_TCP, TCP_QUICKACK,
-			(char __user *)&val, sizeof(val));
+	tcp_setsockopt(sock->sk, SOL_TCP, TCP_CORK, (char __user *)&val, sizeof(val) );
 }
 
 /* drbd_proc.c */
@@ -1518,11 +1542,11 @@ extern int drbd_al_read_log(struct drbd_conf *mdev, struct drbd_backing_dev *);
 extern void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector,
 		int size, const char *file, const unsigned int line);
 #define drbd_set_in_sync(mdev, sector, size) \
-	__drbd_set_in_sync(mdev, sector, size, __FILE__, __LINE__)
+	__drbd_set_in_sync(mdev, sector, size, __FILE__, __LINE__ )
 extern void __drbd_set_out_of_sync(struct drbd_conf *mdev, sector_t sector,
 		int size, const char *file, const unsigned int line);
 #define drbd_set_out_of_sync(mdev, sector, size) \
-	__drbd_set_out_of_sync(mdev, sector, size, __FILE__, __LINE__)
+	__drbd_set_out_of_sync(mdev, sector, size, __FILE__, __LINE__ )
 extern void drbd_al_apply_to_bm(struct drbd_conf *mdev);
 extern void drbd_al_to_on_disk_bm(struct drbd_conf *mdev);
 extern void drbd_al_shrink(struct drbd_conf *mdev);
@@ -1534,6 +1558,10 @@ void drbd_nl_cleanup(void);
 int __init drbd_nl_init(void);
 void drbd_bcast_state(struct drbd_conf *mdev, union drbd_state_t);
 void drbd_bcast_sync_progress(struct drbd_conf *mdev);
+void drbd_bcast_ee(struct drbd_conf *mdev,
+		const char *reason, const int dgs,
+		const char* seen_hash, const char* calc_hash,
+		const struct Tl_epoch_entry* e);
 
 /*
  * inline helper functions
@@ -1654,31 +1682,25 @@ static inline sector_t drbd_md_last_sector(struct drbd_backing_dev *bdev)
 	switch (bdev->dc.meta_dev_idx) {
 	case DRBD_MD_INDEX_INTERNAL:
 	case DRBD_MD_INDEX_FLEX_INT:
-		return bdev->md.md_offset + MD_AL_OFFSET - 1;
+		return bdev->md.md_offset + MD_AL_OFFSET -1;
 	case DRBD_MD_INDEX_FLEX_EXT:
 	default:
 		return bdev->md.md_offset + bdev->md.md_size_sect;
 	}
 }
 
-/* returns the capacity we announce to out peer.
- * we clip ourselves at the various MAX_SECTORS, because if we don't,
- * current implementation will oops sooner or later */
+/* returns the capacity we announce to out peer */
 static inline sector_t drbd_get_max_capacity(struct drbd_backing_dev *bdev)
 {
 	switch (bdev->dc.meta_dev_idx) {
 	case DRBD_MD_INDEX_INTERNAL:
 	case DRBD_MD_INDEX_FLEX_INT:
 		return drbd_get_capacity(bdev->backing_bdev)
-			? min_t(sector_t, DRBD_MAX_SECTORS_FLEX,
-					drbd_md_first_sector(bdev))
+			? drbd_md_first_sector(bdev)
 			: 0;
 	case DRBD_MD_INDEX_FLEX_EXT:
-		return min_t(sector_t, DRBD_MAX_SECTORS_FLEX,
-				drbd_get_capacity(bdev->backing_bdev));
 	default:
-		return min_t(sector_t, DRBD_MAX_SECTORS,
-				drbd_get_capacity(bdev->backing_bdev));
+		return drbd_get_capacity(bdev->backing_bdev);
 	}
 }
 
@@ -1818,7 +1840,7 @@ static inline void inc_ap_pending(struct drbd_conf *mdev)
 		    atomic_read(&mdev->which))
 
 #define dec_ap_pending(mdev)	do {				\
-	typecheck(struct drbd_conf *, mdev);			\
+	typecheck(struct drbd_conf *, mdev);				\
 	if (atomic_dec_and_test(&mdev->ap_pending_cnt))		\
 		wake_up(&mdev->misc_wait);			\
 	ERR_IF_CNT_IS_NEGATIVE(ap_pending_cnt); } while (0)
@@ -1835,7 +1857,7 @@ static inline void inc_rs_pending(struct drbd_conf *mdev)
 }
 
 #define dec_rs_pending(mdev)	do {				\
-	typecheck(struct drbd_conf *, mdev);			\
+	typecheck(struct drbd_conf *, mdev);				\
 	atomic_dec(&mdev->rs_pending_cnt);			\
 	ERR_IF_CNT_IS_NEGATIVE(rs_pending_cnt); } while (0)
 
@@ -1854,12 +1876,12 @@ static inline void inc_unacked(struct drbd_conf *mdev)
 }
 
 #define dec_unacked(mdev)	do {				\
-	typecheck(struct drbd_conf *, mdev);			\
+	typecheck(struct drbd_conf *, mdev);				\
 	atomic_dec(&mdev->unacked_cnt);				\
 	ERR_IF_CNT_IS_NEGATIVE(unacked_cnt); } while (0)
 
 #define sub_unacked(mdev, n)	do {				\
-	typecheck(struct drbd_conf *, mdev);			\
+	typecheck(struct drbd_conf *, mdev);				\
 	atomic_sub(n, &mdev->unacked_cnt);			\
 	ERR_IF_CNT_IS_NEGATIVE(unacked_cnt); } while (0)
 
@@ -1906,7 +1928,7 @@ static inline int _inc_local_if_state(struct drbd_conf *mdev, enum drbd_disk_sta
 	int io_allowed;
 
 	atomic_inc(&mdev->local_cnt);
-	io_allowed = (mdev->state.disk >= mins);
+	io_allowed = (mdev->state.disk >= mins );
 	if (!io_allowed)
 		dec_local(mdev);
 	return io_allowed;
@@ -1939,7 +1961,7 @@ static inline void drbd_get_syncer_progress(struct drbd_conf *mdev,
 		 * for now, just prevent in-kernel buffer overflow.
 		 */
 		smp_rmb();
-		drbd_WARN("cs:%s rs_left=%lu > rs_total=%lu (rs_failed %lu)\n",
+		WARN("cs:%s rs_left=%lu > rs_total=%lu (rs_failed %lu)\n",
 				conns_to_name(mdev->state.conn),
 				*bits_left, mdev->rs_total, mdev->rs_failed);
 		*per_mil_done = 0;
@@ -1966,82 +1988,18 @@ static inline int drbd_get_max_buffers(struct drbd_conf *mdev)
 	return mxb;
 }
 
-static inline int drbd_state_is_stable(union drbd_state_t s)
-{
-
-	/* DO NOT add a default clause, we want the compiler to warn us
-	 * for any newly introduced state we may have forgotten to add here */
-
-	switch ((enum drbd_conns)s.conn) {
-	/* new io only accepted when there is no connection, ... */
-	case StandAlone:
-	case WFConnection:
-	/* ... or there is a well established connection. */
-	case Connected:
-	case SyncSource:
-	case SyncTarget:
-	case PausedSyncS:
-	case PausedSyncT:
-		/* maybe stable, look at the disk state */
-		break;
-
-	/* no new io accepted during tansitional states
-	 * like handshake or teardown */
-	case Disconnecting:
-	case Unconnected:
-	case Timeout:
-	case BrokenPipe:
-	case NetworkFailure:
-	case ProtocolError:
-	case TearDown:
-	case WFReportParams:
-	case StartingSyncS:
-	case StartingSyncT:
-	case WFBitMapS:
-	case WFBitMapT:
-	case WFSyncUUID:
-	case conn_mask:
-		/* not "stable" */
-		return 0;
-	}
-
-	switch ((enum drbd_disk_state)s.disk) {
-	case Diskless:
-	case Inconsistent:
-	case Outdated:
-	case Consistent:
-	case UpToDate:
-		/* disk state is stable as well. */
-		break;
-
-	/* no new io accepted during tansitional states */
-	case Attaching:
-	case Failed:
-	case Negotiating:
-	case DUnknown:
-	case disk_mask:
-		/* not "stable" */
-		return 0;
-	}
-
-	return 1;
-}
-
 static inline int __inc_ap_bio_cond(struct drbd_conf *mdev)
 {
+	const unsigned int cs = mdev->state.conn;
+	const unsigned int ds = mdev->state.disk;
 	int mxb = drbd_get_max_buffers(mdev);
-
 	if (mdev->state.susp)
 		return 0;
-	if (test_bit(SUSPEND_IO, &mdev->flags))
+	/* to avoid deadlock or bitmap corruption, we need to lock out
+	 * application io during attaching and bitmap exchange */
+	if (Attaching <= ds && ds <= Negotiating)
 		return 0;
-
-	/* to avoid potential deadlock or bitmap corruption,
-	 * in various places, we only allow new application io
-	 * to start during "stable" states. */
-
-	/* no new io accepted when attaching or detaching the disk */
-	if (!drbd_state_is_stable(mdev->state))
+	if (cs == WFBitMapS || cs == WFBitMapT || cs == WFReportParams || cs == WFSyncUUID)
 		return 0;
 
 	/* since some older kernels don't have atomic_add_unless,
@@ -2050,13 +2008,14 @@ static inline int __inc_ap_bio_cond(struct drbd_conf *mdev)
 		return 0;
 	if (test_bit(BITMAP_IO, &mdev->flags))
 		return 0;
+	atomic_inc(&mdev->ap_bio_cnt);
 	return 1;
 }
 
 /* I'd like to use wait_event_lock_irq,
  * but I'm not sure when it got introduced,
  * and not sure when it has 3 or 4 arguments */
-static inline void inc_ap_bio(struct drbd_conf *mdev, int one_or_two)
+static inline void inc_ap_bio(struct drbd_conf *mdev)
 {
 	/* compare with after_state_ch,
 	 * os.conn != WFBitMapS && ns.conn == WFBitMapS */
@@ -2078,7 +2037,6 @@ static inline void inc_ap_bio(struct drbd_conf *mdev, int one_or_two)
 		finish_wait(&mdev->misc_wait, &wait);
 		spin_lock_irq(&mdev->req_lock);
 	}
-	atomic_add(one_or_two, &mdev->ap_bio_cnt);
 	spin_unlock_irq(&mdev->req_lock);
 }
 
@@ -2088,9 +2046,6 @@ static inline void dec_ap_bio(struct drbd_conf *mdev)
 	int ap_bio = atomic_dec_return(&mdev->ap_bio_cnt);
 
 	D_ASSERT(ap_bio >= 0);
-	/* this currently does wake_up for every dec_ap_bio!
-	 * maybe rather introduce some type of hysteresis?
-	 * e.g. (ap_bio == mxb/2 || ap_bio == 0) ? */
 	if (ap_bio < mxb)
 		wake_up(&mdev->misc_wait);
 	if (ap_bio == 0 && test_bit(BITMAP_IO, &mdev->flags)) {
@@ -2103,9 +2058,8 @@ static inline void drbd_set_ed_uuid(struct drbd_conf *mdev, u64 val)
 {
 	mdev->ed_uuid = val;
 
-	MTRACE(TraceTypeUuid, TraceLvlMetrics,
-	       INFO(" exposed data uuid now %016llX\n",
-		    (unsigned long long)val);
+	MTRACE(TraceTypeUuid,TraceLvlMetrics,
+	       INFO(" exposed data uuid now %016llX\n",val);
 		);
 }
 
@@ -2116,7 +2070,7 @@ static inline int seq_cmp(u32 a, u32 b)
 	 * we'd have to
 	 *  a <<= 8; b <<= 8;
 	 */
-	return (s32)(a) - (s32)(b);
+	return ((s32)(a) - (s32)(b));
 }
 #define seq_lt(a, b) (seq_cmp((a), (b)) < 0)
 #define seq_gt(a, b) (seq_cmp((a), (b)) > 0)
