@@ -35,6 +35,7 @@
 #include <linux/connector.h>
 #include <linux/drbd.h>
 #include <linux/blkpg.h>
+#include <linux/cpumask.h>
 
 #include "drbd_int.h"
 #include <linux/drbd_tag_magic.h>
@@ -51,9 +52,9 @@ STATIC int name ## _from_tags (struct drbd_conf *mdev, \
 	int tag; \
 	int dlen; \
 	\
-	while ((tag = *tags++) != TT_END) { \
+	while ( (tag = *tags++) != TT_END ) { \
 		dlen = *tags++; \
-		switch (tag_number(tag)) { \
+		switch ( tag_number(tag) ) { \
 		fields \
 		default: \
 			if (tag & T_MANDATORY) { \
@@ -79,17 +80,12 @@ STATIC int name ## _from_tags (struct drbd_conf *mdev, \
 		 break;
 #define NL_STRING(pn, pr, member, len) \
 	case pn: /* D_ASSERT( tag_type(tag) == TT_STRING ); */ \
-		if (dlen > len) { \
-			ERR("arg too long: %s (%u wanted, max len: %u bytes)\n", \
-				#member, dlen, (unsigned int)len); \
-			return 0; \
-		} \
 		 arg->member ## _len = dlen; \
 		 memcpy(arg->member, tags, min_t(size_t, dlen, len)); \
 		 break;
 #include "linux/drbd_nl.h"
 
-/* Generate the struct to tag_list functions */
+// Generate the struct to tag_list functions
 #define NL_PACKET(name, number, fields) \
 STATIC unsigned short* \
 name ## _to_tags (struct drbd_conf *mdev, \
@@ -163,7 +159,7 @@ STATIC void nl_trace_reply(void *data)
 	printk(KERN_INFO "drbd%d: "
 	       "Netlink: >> %s (%d) - seq: %x, ack: %x, len: %x\n",
 	       nlp->minor,
-	       nlp->packet_type == P_nl_after_last_packet ?
+	       nlp->packet_type == P_nl_after_last_packet?
 		   "Empty-Reply" : nl_packet_name(nlp->packet_type),
 	       nlp->packet_type,
 	       req->seq, req->ack, req->len);
@@ -173,7 +169,6 @@ int drbd_khelper(struct drbd_conf *mdev, char *cmd)
 {
 	char mb[12];
 	char *argv[] = {usermode_helper, cmd, mb, NULL };
-	int ret;
 	static char *envp[] = { "HOME=/",
 				"TERM=linux",
 				"PATH=/sbin:/usr/sbin:/bin:/usr/bin",
@@ -181,20 +176,10 @@ int drbd_khelper(struct drbd_conf *mdev, char *cmd)
 
 	snprintf(mb, 12, "minor-%d", mdev_to_minor(mdev));
 
-	INFO("helper command: %s %s %s\n", usermode_helper, cmd, mb);
+	INFO("helper command: %s %s\n", usermode_helper, cmd);
 
 	drbd_bcast_ev_helper(mdev, cmd);
-	ret = call_usermodehelper(usermode_helper, argv, envp, 1);
-	if (ret)
-		drbd_WARN("helper command: %s %s %s exit code %u (0x%x)\n",
-				usermode_helper, cmd, mb,
-				(ret >> 8), ret);
-	else
-		INFO("helper command: %s %s %s exit code %u (0x%x)\n",
-				usermode_helper, cmd, mb,
-				(ret >> 8), ret);
-
-	return ret;
+	return call_usermodehelper(usermode_helper, argv, envp, 1);
 }
 
 enum drbd_disk_state drbd_try_outdate_peer(struct drbd_conf *mdev)
@@ -210,12 +195,12 @@ enum drbd_disk_state drbd_try_outdate_peer(struct drbd_conf *mdev)
 		fp = mdev->bc->dc.fencing;
 		dec_local(mdev);
 	} else {
-		drbd_WARN("Not outdating peer, I'm not even Consistent myself.\n");
+		WARN("Not outdating peer, I'm not even Consistent myself.\n");
 		return mdev->state.pdsk;
 	}
 
 	if (fp == Stonith)
-		_drbd_request_state(mdev, NS(susp, 1), ChgWaitComplete);
+		_drbd_request_state(mdev, NS(susp,1), ChgWaitComplete);
 
 	r = drbd_khelper(mdev, "outdate-peer");
 
@@ -237,7 +222,7 @@ enum drbd_disk_state drbd_try_outdate_peer(struct drbd_conf *mdev)
 		 * This is useful when an unconnected Secondary is asked to
 		 * become Primary, but findes the other peer being active. */
 		ex_to_string = "peer is active";
-		drbd_WARN("Peer is primary, outdating myself.\n");
+		WARN("Peer is primary, outdating myself.\n");
 		nps = DUnknown;
 		_drbd_request_state(mdev, NS(disk, Outdated), ChgWaitComplete);
 		break;
@@ -288,17 +273,17 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 			continue;
 		}
 
-		if (r == SS_NoUpToDateDisk && force &&
-		    (mdev->state.disk == Inconsistent ||
-		     mdev->state.disk == Outdated)) {
+		if( r == SS_NoUpToDateDisk && force &&
+		    ( mdev->state.disk == Inconsistent ||
+		      mdev->state.disk == Outdated ) ) {
 			mask.disk = disk_mask;
 			val.disk  = UpToDate;
 			forced = 1;
 			continue;
 		}
 
-		if (r == SS_NoUpToDateDisk &&
-		    mdev->state.disk == Consistent) {
+		if ( r == SS_NoUpToDateDisk &&
+		    mdev->state.disk == Consistent ) {
 			D_ASSERT(mdev->state.pdsk == DUnknown);
 			nps = drbd_try_outdate_peer(mdev);
 
@@ -319,7 +304,7 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 			nps = drbd_try_outdate_peer(mdev);
 
 			if (force && nps > Outdated) {
-				drbd_WARN("Forced into split brain situation!\n");
+				WARN("Forced into split brain situation!\n");
 				nps = Outdated;
 			}
 
@@ -331,10 +316,10 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 		if (r == SS_TwoPrimaries) {
 			/* Maybe the peer is detected as dead very soon...
 			   retry at most once more in this case. */
-			__set_current_state(TASK_INTERRUPTIBLE);
+			set_current_state(TASK_INTERRUPTIBLE);
 			schedule_timeout((mdev->net_conf->ping_timeo+1)*HZ/10);
 			if (try < max_tries)
-				try = max_tries - 1;
+				try = max_tries -1;
 			continue;
 		}
 		if (r < SS_Success) {
@@ -347,7 +332,7 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 	}
 
 	if (forced)
-		drbd_WARN("Forced to consider local data as UpToDate!\n");
+		WARN("Forced to consider local data as UpToDate!\n");
 
 	fsync_bdev(mdev->this_bdev);
 
@@ -361,7 +346,7 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 	 * */
 
 	if (new_role == Secondary) {
-		set_disk_ro(mdev->vdisk, TRUE);
+		set_disk_ro(mdev->vdisk, TRUE );
 		if (inc_local(mdev)) {
 			mdev->bc->md.uuid[Current] &= ~(u64)1;
 			dec_local(mdev);
@@ -371,9 +356,9 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 			mdev->net_conf->want_lose = 0;
 			dec_net(mdev);
 		}
-		set_disk_ro(mdev->vdisk, FALSE);
+		set_disk_ro(mdev->vdisk, FALSE );
 		if (inc_local(mdev)) {
-			if (((mdev->state.conn < Connected ||
+			if ( ((mdev->state.conn < Connected ||
 			       mdev->state.pdsk <= Failed)
 			      && mdev->bc->md.uuid[Bitmap] == 0) || forced)
 				drbd_uuid_new_current(mdev);
@@ -383,7 +368,8 @@ int drbd_set_role(struct drbd_conf *mdev, enum drbd_role new_role, int force)
 		}
 	}
 
-	if ((new_role == Secondary) && inc_local(mdev)) {
+	if ((new_role == Secondary) && inc_local(mdev) )
+	{
 		drbd_al_to_on_disk_bm(mdev);
 		dec_local(mdev);
 	}
@@ -477,7 +463,7 @@ char *ppsize(char *buf, unsigned long long size)
 	/* Needs 9 bytes at max. */
 	static char units[] = { 'K', 'M', 'G', 'T', 'P', 'E' };
 	int base = 0;
-	while (size >= 10000) {
+	while (size >= 10000 ) {
 		/* shift + round */
 		size = (size >> 10) + !!(size & (1<<9));
 		base++;
@@ -487,34 +473,6 @@ char *ppsize(char *buf, unsigned long long size)
 	return buf;
 }
 
-/* there is still a theoretical deadlock when called from receiver
- * on an Inconsistent Primary:
- *  remote READ does inc_ap_bio, receiver would need to receive answer
- *  packet from remote to dec_ap_bio again.
- *  receiver receive_sizes(), comes here,
- *  waits for ap_bio_cnt == 0. -> deadlock.
- * but this cannot happen, actually, because:
- *  Primary Inconsistent, and peer's disk is unreachable
- *  (not connected, *  or bad/no disk on peer):
- *  see drbd_fail_request_early, ap_bio_cnt is zero.
- *  Primary Inconsistent, and SyncTarget:
- *  peer may not initiate a resize.
- */
-void drbd_suspend_io(struct drbd_conf *mdev)
-{
-	int in_flight;
-	set_bit(SUSPEND_IO, &mdev->flags);
-	in_flight = atomic_read(&mdev->ap_bio_cnt);
-	if (in_flight)
-		wait_event(mdev->misc_wait, !atomic_read(&mdev->ap_bio_cnt));
-}
-
-void drbd_resume_io(struct drbd_conf *mdev)
-{
-	clear_bit(SUSPEND_IO, &mdev->flags);
-	wake_up(&mdev->misc_wait);
-}
-
 /**
  * drbd_determin_dev_size:
  * Evaluates all constraints and sets our correct device size.
@@ -530,20 +488,8 @@ enum determin_dev_size_enum drbd_determin_dev_size(struct drbd_conf *mdev) __mus
 	char ppb[10];
 
 	int md_moved, la_size_changed;
-	enum determin_dev_size_enum rv = unchanged;
+	enum determin_dev_size_enum rv=unchanged;
 
-	/* race:
-	 * application request passes inc_ap_bio,
-	 * but then cannot get an AL-reference.
-	 * this function later may wait on ap_bio_cnt == 0. -> deadlock.
-	 *
-	 * to avoid that:
-	 * Suspend IO right here.
-	 * still lock the act_log to not trigger ASSERTs there.
-	 */
-	drbd_suspend_io(mdev);
-
-	/* no wait necessary anymore, actually we could assert that */
 	wait_event(mdev->al_wait, lc_try_lock(mdev->act_log));
 
 	prev_first_sect = drbd_md_first_sector(mdev->bc);
@@ -555,8 +501,8 @@ enum determin_dev_size_enum drbd_determin_dev_size(struct drbd_conf *mdev) __mus
 
 	size = drbd_new_dev_size(mdev, mdev->bc);
 
-	if (drbd_get_capacity(mdev->this_bdev) != size ||
-	    drbd_bm_capacity(mdev) != size) {
+	if ( drbd_get_capacity(mdev->this_bdev) != size ||
+	    drbd_bm_capacity(mdev) != size ) {
 		int err;
 		err = drbd_bm_resize(mdev, size);
 		if (unlikely(err)) {
@@ -585,29 +531,27 @@ enum determin_dev_size_enum drbd_determin_dev_size(struct drbd_conf *mdev) __mus
 
 	la_size_changed = (la_size != mdev->bc->md.la_size_sect);
 
+	/* LGE: flexible device size!! is this the right thing to test? */
 	md_moved = prev_first_sect != drbd_md_first_sector(mdev->bc)
 		|| prev_size	   != mdev->bc->md.md_size_sect;
 
 	if (md_moved) {
-		drbd_WARN("Moving meta-data.\n");
+		WARN("Moving meta-data.\n");
 		/* assert: (flexible) internal meta data */
 	}
 
 	if (la_size_changed || md_moved) {
 		drbd_al_shrink(mdev); /* All extents inactive. */
 		INFO("Writing the whole bitmap, size changed\n");
-		rv = drbd_bitmap_io(mdev, &drbd_bm_write, "size changed");
+		rv = drbd_bitmap_io(mdev, &drbd_bm_write);
 		drbd_md_mark_dirty(mdev);
 	}
 
-	if (size > la_size)
-		rv = grew;
-	if (size < la_size)
-		rv = shrunk;
+	if (size > la_size) rv = grew;
+	if (size < la_size) rv = shrunk;
 out:
 	lc_unlock(mdev->act_log);
 	wake_up(&mdev->al_wait);
-	drbd_resume_io(mdev);
 
 	return rv;
 }
@@ -671,8 +615,8 @@ STATIC int drbd_check_al_size(struct drbd_conf *mdev)
 	ERR_IF(mdev->sync_conf.al_extents < 7)
 		mdev->sync_conf.al_extents = 127;
 
-	if (mdev->act_log &&
-	    mdev->act_log->nr_elements == mdev->sync_conf.al_extents)
+	if ( mdev->act_log &&
+	     mdev->act_log->nr_elements == mdev->sync_conf.al_extents )
 		return 0;
 
 	in_use = 0;
@@ -714,6 +658,7 @@ void drbd_setup_queue_param(struct drbd_conf *mdev, unsigned int max_seg_s) __mu
 	struct request_queue * const q = mdev->rq_queue;
 	struct request_queue * const b = mdev->bc->backing_bdev->bd_disk->queue;
 	/* unsigned int old_max_seg_s = q->max_segment_size; */
+	int max_segments = mdev->bc->dc.max_bio_bvecs;
 
 	if (b->merge_bvec_fn && !mdev->bc->dc.use_bmbv)
 		max_seg_s = PAGE_SIZE;
@@ -730,8 +675,13 @@ void drbd_setup_queue_param(struct drbd_conf *mdev, unsigned int max_seg_s) __mu
 	       );
 
 	q->max_sectors	     = max_seg_s >> 9;
-	q->max_phys_segments = max_seg_s >> PAGE_SHIFT;
-	q->max_hw_segments   = max_seg_s >> PAGE_SHIFT;
+	if (max_segments) {
+		q->max_phys_segments = max_segments;
+		q->max_hw_segments   = max_segments;
+	} else {
+		q->max_phys_segments = MAX_PHYS_SEGMENTS;
+		q->max_hw_segments   = MAX_HW_SEGMENTS;
+	}
 	q->max_segment_size  = max_seg_s;
 	q->hardsect_size     = 512;
 	q->seg_boundary_mask = PAGE_SIZE-1;
@@ -755,7 +705,7 @@ void drbd_setup_queue_param(struct drbd_conf *mdev, unsigned int max_seg_s) __mu
 	       );
 
 	if (b->merge_bvec_fn)
-		drbd_WARN("Backing device's merge_bvec_fn() = %p\n",
+		WARN("Backing device's merge_bvec_fn() = %p\n",
 		     b->merge_bvec_fn);
 	INFO("max_segment_size ( = BIO size ) = %u\n", q->max_segment_size);
 
@@ -773,9 +723,6 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 			     struct drbd_nl_cfg_reply *reply)
 {
 	enum ret_codes retcode;
-	enum determin_dev_size_enum dd;
-	sector_t max_possible_sectors;
-	sector_t min_md_device_sectors;
 	struct drbd_backing_dev *nbc = NULL; /* new_backing_conf */
 	struct inode *inode, *inode2;
 	struct lru_cache *resync_lru = NULL;
@@ -788,20 +735,20 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 		goto fail;
 	}
 
-	/*
-	* We may have gotten here very quickly from a detach. Wait for a bit
-	* then fail.
-	*/
+       /*
+        * We may have gotten here very quickly from a detach. Wait for a bit
+        * then fail.
+        */
 	while (1) {
 		__no_warn(local, nbc = mdev->bc; );
 		if (nbc == NULL)
 			break;
 		if (ntries++ >= 5) {
-			drbd_WARN("drbd_nl_disk_conf: mdev->bc not NULL.\n");
+			WARN("drbd_nl_disk_conf: mdev->bc not NULL.\n");
 			retcode = HaveDiskConfig;
 			goto fail;
 		}
-		__set_current_state(TASK_INTERRUPTIBLE);
+		set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(HZ/10);
 	}
 
@@ -811,7 +758,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 		goto fail;
 	}
 
-	if (!(nlp->flags & DRBD_NL_SET_DEFAULTS) && inc_local(mdev)) {
+	if ( !(nlp->flags & DRBD_NL_SET_DEFAULTS) && inc_local(mdev) ) {
 		memcpy(&nbc->dc, &mdev->bc->dc, sizeof(struct disk_conf));
 		dec_local(mdev);
 	} else {
@@ -819,6 +766,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 		nbc->dc.disk_size   = DRBD_DISK_SIZE_SECT_DEF;
 		nbc->dc.on_io_error = DRBD_ON_IO_ERROR_DEF;
 		nbc->dc.fencing     = DRBD_FENCING_DEF;
+		nbc->dc.max_bio_bvecs= DRBD_MAX_BIO_BVECS_DEF;
 	}
 
 	if (!disk_conf_from_tags(mdev, nlp->tag_list, &nbc->dc)) {
@@ -894,61 +842,42 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	if (bd_claim(nbc->md_bdev,
 		     (nbc->dc.meta_dev_idx == DRBD_MD_INDEX_INTERNAL ||
 		      nbc->dc.meta_dev_idx == DRBD_MD_INDEX_FLEX_INT) ?
-		     (void *)mdev : (void *) drbd_m_holder)) {
+		     (void *)mdev : (void *) drbd_m_holder )) {
 		retcode = MDMounted;
 		goto release_bdev_fail;
 	}
 
-	if ((nbc->backing_bdev == nbc->md_bdev) !=
-	    (nbc->dc.meta_dev_idx == DRBD_MD_INDEX_INTERNAL ||
-	     nbc->dc.meta_dev_idx == DRBD_MD_INDEX_FLEX_INT)) {
+	if ( (nbc->backing_bdev == nbc->md_bdev) !=
+	     (nbc->dc.meta_dev_idx == DRBD_MD_INDEX_INTERNAL ||
+	      nbc->dc.meta_dev_idx == DRBD_MD_INDEX_FLEX_INT) ) {
 		retcode = LDMDInvalid;
 		goto release_bdev2_fail;
 	}
 
-	if (drbd_get_max_capacity(nbc) < nbc->dc.disk_size) {
+	if ((drbd_get_capacity(nbc->backing_bdev)) < nbc->dc.disk_size) {
 		retcode = LDDeviceTooSmall;
 		goto release_bdev2_fail;
 	}
 
-	if (nbc->dc.meta_dev_idx < 0) {
-		max_possible_sectors = DRBD_MAX_SECTORS_FLEX;
-		/* at least one MB, otherwise it does not make sense */
-		min_md_device_sectors = (2<<10);
-	} else {
-		max_possible_sectors = DRBD_MAX_SECTORS;
-		min_md_device_sectors = MD_RESERVED_SECT * (nbc->dc.meta_dev_idx + 1);
-	}
-
-	if (drbd_get_capacity(nbc->md_bdev) > max_possible_sectors)
-		drbd_WARN("truncating very big lower level device "
-		     "to currently maximum possible %llu sectors\n",
-		     (unsigned long long) max_possible_sectors);
-
-	if (drbd_get_capacity(nbc->md_bdev) < min_md_device_sectors) {
-		retcode = MDDeviceTooSmall;
-		drbd_WARN("refusing attach: md-device too small, "
-		     "at least %llu sectors needed for this meta-disk type\n",
-		     (unsigned long long) min_md_device_sectors);
-		goto release_bdev2_fail;
-	}
+/* TODO check whether backing device size is within plausible limits */
 
 	/* Make sure the new disk is big enough
 	 * (we may currently be Primary with no local disk...) */
 	if (drbd_get_capacity(nbc->backing_bdev) <
-	    drbd_get_capacity(mdev->this_bdev)) {
+	    drbd_get_capacity(mdev->this_bdev) ) {
 		retcode = LDDeviceTooSmall;
 		goto release_bdev2_fail;
 	}
 
-	nbc->known_size = drbd_get_max_capacity(nbc);
+	nbc->known_size = drbd_get_capacity(nbc->backing_bdev);
 
 	retcode = _drbd_request_state(mdev, NS(disk, Attaching), ChgStateVerbose);
-	if (retcode < SS_Success)
+	if (retcode < SS_Success )
 		goto release_bdev2_fail;
 
-	if (!inc_local_if_state(mdev, Attaching))
+	if (!inc_local_if_state(mdev, Attaching)) {
 		goto force_diskless;
+	}
 
 	drbd_thread_start(&mdev->worker);
 	drbd_md_set_sector_offsets(mdev, nbc);
@@ -975,7 +904,6 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	/* Prevent shrinking of consistent devices ! */
 	if (drbd_md_test_flag(nbc, MDF_Consistent) &&
 	   drbd_new_dev_size(mdev, nbc) < nbc->md.la_size_sect) {
-		drbd_WARN("refusing to truncate a consistent device\n");
 		retcode = LDDeviceTooSmall;
 		goto force_diskless_dec;
 	}
@@ -1039,27 +967,25 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	 * degraded but active "cluster" after a certain timeout.
 	 */
 	clear_bit(USE_DEGR_WFC_T, &mdev->flags);
-	if (mdev->state.role != Primary &&
+	if ( mdev->state.role != Primary &&
 	     drbd_md_test_flag(mdev->bc, MDF_PrimaryInd) &&
-	    !drbd_md_test_flag(mdev->bc, MDF_ConnectedInd))
+	    !drbd_md_test_flag(mdev->bc, MDF_ConnectedInd) )
 		set_bit(USE_DEGR_WFC_T, &mdev->flags);
 
-	dd = drbd_determin_dev_size(mdev);
-	if (dd == dev_size_error) {
+	if (drbd_determin_dev_size(mdev) == dev_size_error) {
 		retcode = VMallocFailed;
 		goto force_diskless_dec;
-	} else if (dd == grew)
-		set_bit(RESYNC_AFTER_NEG, &mdev->flags);
+	}
 
 	if (drbd_md_test_flag(mdev->bc, MDF_FullSync)) {
 		INFO("Assuming that all blocks are out of sync "
 		     "(aka FullSync)\n");
-		if (drbd_bitmap_io(mdev, &drbd_bmio_set_n_write, "set_n_write from attaching")) {
+		if (drbd_bitmap_io(mdev, &drbd_bmio_set_n_write)) {
 			retcode = MDIOError;
 			goto force_diskless_dec;
 		}
 	} else {
-		if (drbd_bitmap_io(mdev, &drbd_bm_read, "read from attaching") < 0) {
+		if (drbd_bitmap_io(mdev, &drbd_bm_read) < 0) {
 			retcode = MDIOError;
 			goto force_diskless_dec;
 		}
@@ -1094,7 +1020,7 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 		ns.pdsk = Outdated;
 
 	if ( ns.disk == Consistent &&
-	    (ns.pdsk == Outdated || mdev->bc->dc.fencing == DontCare))
+	    ( ns.pdsk == Outdated || mdev->bc->dc.fencing == DontCare ) )
 		ns.disk = UpToDate;
 
 	/* All tests on MDF_PrimaryInd, MDF_ConnectedInd,
@@ -1117,10 +1043,8 @@ STATIC int drbd_nl_disk_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp
 	if (rv < SS_Success)
 		goto force_diskless_dec;
 
-	if (mdev->state.role == Primary)
-		mdev->bc->md.uuid[Current] |=  (u64)1;
-	else
-		mdev->bc->md.uuid[Current] &= ~(u64)1;
+	if(mdev->state.role == Primary) mdev->bc->md.uuid[Current] |=  (u64)1;
+	else                            mdev->bc->md.uuid[Current] &= ~(u64)1;
 
 	drbd_md_mark_dirty(mdev);
 	drbd_md_sync(mdev);
@@ -1161,7 +1085,7 @@ STATIC int drbd_nl_detach(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 	fsync_bdev(mdev->this_bdev);
 	reply->ret_code = drbd_request_state(mdev, NS(disk, Diskless));
 
-	__set_current_state(TASK_INTERRUPTIBLE);
+	set_current_state(TASK_INTERRUPTIBLE);
 	schedule_timeout(HZ/20); /* 50ms; Time for worker to finally terminate */
 
 	return 0;
@@ -1176,10 +1100,15 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 	enum ret_codes retcode;
 	struct net_conf *new_conf = NULL;
 	struct crypto_hash *tfm = NULL;
+	struct crypto_hash *integrity_w_tfm = NULL;
+	struct crypto_hash *integrity_r_tfm = NULL;
 	struct hlist_head *new_tl_hash = NULL;
 	struct hlist_head *new_ee_hash = NULL;
 	struct drbd_conf *odev;
 	char hmac_name[HMAC_NAME_L];
+	void *int_dig_out = NULL;
+	void *int_dig_in = NULL;
+	void *int_dig_vv = NULL;
 
 	if (mdev->state.conn > StandAlone) {
 		retcode = HaveNetConfig;
@@ -1192,7 +1121,7 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		goto fail;
 	}
 
-	if (!(nlp->flags & DRBD_NL_SET_DEFAULTS) && inc_net(mdev)) {
+	if ( !(nlp->flags & DRBD_NL_SET_DEFAULTS) && inc_net(mdev)) {
 		memcpy(new_conf, mdev->net_conf, sizeof(struct net_conf));
 		dec_net(mdev);
 	} else {
@@ -1240,13 +1169,13 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		odev = minor_to_mdev(i);
 		if (!odev || odev == mdev)
 			continue;
-		if (inc_net(odev)) {
-			if (M_ADDR(new_conf) == M_ADDR(odev->net_conf) &&
-			    M_PORT(new_conf) == M_PORT(odev->net_conf))
+		if ( inc_net(odev)) {
+			if ( M_ADDR(new_conf) == M_ADDR(odev->net_conf) &&
+			    M_PORT(new_conf) == M_PORT(odev->net_conf) )
 				retcode = LAAlreadyInUse;
 
 			if (O_ADDR(new_conf) == O_ADDR(odev->net_conf) &&
-			    O_PORT(new_conf) == O_PORT(odev->net_conf))
+			   O_PORT(new_conf) == O_PORT(odev->net_conf) )
 				retcode = OAAlreadyInUse;
 
 			dec_net(odev);
@@ -1276,6 +1205,26 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		}
 	}
 
+	if (new_conf->integrity_alg[0]) {
+		integrity_w_tfm = crypto_alloc_hash(new_conf->integrity_alg, 0, CRYPTO_ALG_ASYNC);
+		if (IS_ERR(integrity_w_tfm)) {
+			integrity_w_tfm = NULL;
+			retcode=IntegrityAlgNotAvail;
+			goto fail;
+		}
+
+		if (crypto_tfm_alg_type(crypto_hash_tfm(integrity_w_tfm)) != CRYPTO_ALG_TYPE_DIGEST) {
+			retcode=IntegrityAlgNotDigest;
+			goto fail;				
+		}
+
+		integrity_r_tfm = crypto_alloc_hash(new_conf->integrity_alg, 0, CRYPTO_ALG_ASYNC);
+		if (IS_ERR(integrity_r_tfm)) {
+			integrity_r_tfm = NULL;
+			retcode=IntegrityAlgNotAvail;
+			goto fail;
+		}
+	}
 
 	ns = new_conf->max_epoch_size/8;
 	if (mdev->tl_hash_s != ns) {
@@ -1287,7 +1236,7 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 	}
 
 	ns = new_conf->max_buffers/8;
-	if (new_conf->two_primaries && (mdev->ee_hash_s != ns)) {
+	if (new_conf->two_primaries && ( mdev->ee_hash_s != ns ) ) {
 		new_ee_hash = kzalloc(ns*sizeof(void *), GFP_KERNEL);
 		if (!new_ee_hash) {
 			retcode = KMallocFailed;
@@ -1321,6 +1270,32 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		new_conf->ping_int = new_conf->ping_int+1;
 #endif
 
+	if (integrity_w_tfm) {
+		i = crypto_hash_digestsize(integrity_w_tfm);
+		int_dig_out = kmalloc(i, GFP_KERNEL);
+		if (!int_dig_out) {
+			retcode = KMallocFailed;
+			goto fail;
+		}
+		int_dig_in = kmalloc(i, GFP_KERNEL);
+		if (!int_dig_in) {
+			retcode = KMallocFailed;
+			goto fail;
+		}
+		int_dig_vv = kmalloc(i, GFP_KERNEL);
+		if (!int_dig_vv) {
+			retcode = KMallocFailed;
+			goto fail;
+		}
+	}
+
+	if (!mdev->bitmap) {
+		if(drbd_bm_init(mdev)) {
+			retcode = KMallocFailed;
+			goto fail;
+		}
+	}
+
 	D_ASSERT(mdev->net_conf == NULL);
 	mdev->net_conf = new_conf;
 
@@ -1342,6 +1317,19 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 	crypto_free_hash(mdev->cram_hmac_tfm);
 	mdev->cram_hmac_tfm = tfm;
 
+	crypto_free_hash(mdev->integrity_w_tfm);
+	mdev->integrity_w_tfm = integrity_w_tfm;
+
+	crypto_free_hash(mdev->integrity_r_tfm);
+	mdev->integrity_r_tfm = integrity_r_tfm;
+
+	kfree(mdev->int_dig_out);
+	kfree(mdev->int_dig_in);
+	kfree(mdev->int_dig_vv);
+	mdev->int_dig_out=int_dig_out;
+	mdev->int_dig_in=int_dig_in;
+	mdev->int_dig_vv=int_dig_vv;
+
 	retcode = _drbd_request_state(mdev, NS(conn, Unconnected), ChgStateVerbose);
 	if (retcode >= SS_Success)
 		drbd_thread_start(&mdev->worker);
@@ -1350,7 +1338,12 @@ STATIC int drbd_nl_net_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 	return 0;
 
 fail:
+	kfree(int_dig_out);
+	kfree(int_dig_in);
+	kfree(int_dig_vv);
 	crypto_free_hash(tfm);
+	crypto_free_hash(integrity_w_tfm);
+	crypto_free_hash(integrity_r_tfm);
 	kfree(new_tl_hash);
 	kfree(new_ee_hash);
 	kfree(new_conf);
@@ -1383,13 +1376,14 @@ STATIC int drbd_nl_disconnect(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nl
 			drbd_force_state(mdev, NS(conn, Disconnecting));
 			retcode = SS_Success;
 		}
+
 	}
 
 	if (retcode < SS_Success)
 		goto fail;
 
 	if (wait_event_interruptible(mdev->state_wait,
-				     mdev->state.conn != Disconnecting)) {
+				     mdev->state.conn != Disconnecting) ) {
 		/* Do not test for mdev->state.conn == StandAlone, since
 		   someone else might connect us in the mean time! */
 		retcode = GotSignal;
@@ -1412,12 +1406,12 @@ void resync_after_online_grow(struct drbd_conf *mdev)
 	if (mdev->state.role != mdev->state.peer)
 		iass = (mdev->state.role == Primary);
 	else
-		iass = test_bit(DISCARD_CONCURRENT, &mdev->flags);
+		iass = test_bit(DISCARD_CONCURRENT,&mdev->flags);
 
 	if (iass)
-		drbd_start_resync(mdev, SyncSource);
+		drbd_start_resync(mdev,SyncSource);
 	else
-		_drbd_request_state(mdev, NS(conn, WFSyncUUID), ChgStateVerbose + ChgSerialize);
+		drbd_request_state(mdev,NS(conn,WFSyncUUID));
 }
 
 STATIC int drbd_nl_resize(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
@@ -1439,8 +1433,8 @@ STATIC int drbd_nl_resize(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		goto fail;
 	}
 
-	if (mdev->state.role == Secondary &&
-	    mdev->state.peer == Secondary) {
+	if ( mdev->state.role == Secondary &&
+	     mdev->state.peer == Secondary) {
 		retcode = APrimaryNodeNeeded;
 		goto fail;
 	}
@@ -1464,7 +1458,7 @@ STATIC int drbd_nl_resize(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 		goto fail;
 	}
 
-	if (mdev->state.conn == Connected && (dd != unchanged || ldsc)) {
+	if (mdev->state.conn == Connected && ( dd != unchanged || ldsc) ) {
 		drbd_send_uuids(mdev);
 		drbd_send_sizes(mdev);
 		if (dd == grew)
@@ -1483,6 +1477,9 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	struct syncer_conf sc;
 	struct drbd_conf *odev;
 	int err;
+	struct crypto_hash *verify_tfm = NULL, *old_verify_tfm = NULL;
+	int ovr; /* online verify running */
+	cpumask_t n_cpu_mask = CPU_MASK_NONE;
 
 	memcpy(&sc, &mdev->sync_conf, sizeof(struct syncer_conf));
 
@@ -1499,7 +1496,7 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	}
 
 	if (sc.after != -1) {
-		if (sc.after < -1 || minor_to_mdev(sc.after) == NULL) {
+		if ( sc.after < -1 || minor_to_mdev(sc.after) == NULL ) {
 			retcode = SyncAfterInvalid;
 			goto fail;
 		}
@@ -1515,6 +1512,38 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 		}
 	}
 
+	ovr = (mdev->state.conn == VerifyS || mdev->state.conn == VerifyT);
+
+	if (ovr) {
+		if (strcmp(sc.verify_alg, mdev->sync_conf.verify_alg)) {
+			retcode = VERIFYIsRunning;
+			goto fail;
+		}
+	}
+
+	if (!ovr && sc.verify_alg[0]) {
+		verify_tfm = crypto_alloc_hash(sc.verify_alg, 0, CRYPTO_ALG_ASYNC);
+		if (IS_ERR(verify_tfm)) {
+			verify_tfm = NULL;
+			retcode=VERIFYAlgNotAvail;
+			goto fail;
+		}
+
+		if (crypto_tfm_alg_type(crypto_hash_tfm(verify_tfm)) != CRYPTO_ALG_TYPE_DIGEST) {
+			retcode=VERIFYAlgNotDigest;
+			goto fail;
+		}
+	}
+
+	if (sc.cpu_mask[0] != 0) {
+		err = __bitmap_parse(sc.cpu_mask, 32, 0, (unsigned long *)&n_cpu_mask, NR_CPUS);
+		if (err) {
+			WARN("__bitmap_parse() failed with %d\n", err);
+			retcode = CPUMaskParseFailed;
+			goto fail;
+		}
+	}
+
 	ERR_IF (sc.rate < 1) sc.rate = 1;
 	ERR_IF (sc.al_extents < 7) sc.al_extents = 127; /* arbitrary minimum */
 #define AL_MAX ((MD_AL_MAX_SIZE-1) * AL_EXTENTS_PT)
@@ -1527,8 +1556,15 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 	spin_lock(&mdev->peer_seq_lock);
 	/* lock against receive_SyncParam() */
 	mdev->sync_conf = sc;
+	if (!ovr) {
+		old_verify_tfm = mdev->verify_tfm;
+		mdev->verify_tfm = verify_tfm;
+		verify_tfm = NULL;
+	}
 	spin_unlock(&mdev->peer_seq_lock);
 
+	crypto_free_hash(old_verify_tfm);
+
 	if (inc_local(mdev)) {
 		wait_event(mdev->al_wait, lc_try_lock(mdev->act_log));
 		drbd_al_shrink(mdev);
@@ -1550,7 +1586,16 @@ STATIC int drbd_nl_syncer_conf(struct drbd_conf *mdev, struct drbd_nl_cfg_req *n
 
 	drbd_alter_sa(mdev, sc.after);
 
+	if (!cpus_equal(mdev->cpu_mask, n_cpu_mask)) {
+		mdev->cpu_mask = n_cpu_mask;
+		n_cpu_mask = drbd_calc_cpu_mask(mdev);
+		drbd_thread_set_cpu(&mdev->receiver, n_cpu_mask);
+		drbd_thread_set_cpu(&mdev->worker, n_cpu_mask);
+		drbd_thread_set_cpu(&mdev->asender, n_cpu_mask);
+	}
+
 fail:
+	crypto_free_hash(verify_tfm);
 	reply->ret_code = retcode;
 	return 0;
 }
@@ -1558,8 +1603,26 @@ fail:
 STATIC int drbd_nl_invalidate(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
 			      struct drbd_nl_cfg_reply *reply)
 {
-	reply->ret_code = drbd_request_state(mdev, NS2(conn, StartingSyncT,
-						      disk, Inconsistent));
+	int retcode;
+
+	retcode = _drbd_request_state(mdev, NS(conn, StartingSyncT), ChgOrdered);
+
+	if (retcode < SS_Success && retcode != SS_NeedConnection)
+		retcode = drbd_request_state(mdev, NS(conn, StartingSyncT));
+
+	while (retcode == SS_NeedConnection) {
+		spin_lock_irq(&mdev->req_lock);
+		if (mdev->state.conn < Connected)
+			retcode = _drbd_set_state(_NS(mdev, disk, Inconsistent), ChgStateVerbose, NULL);
+		spin_unlock_irq(&mdev->req_lock);
+
+		if (retcode != SS_NeedConnection)
+			break;
+
+		retcode = drbd_request_state(mdev, NS(conn, StartingSyncT));
+	}
+
+	reply->ret_code = retcode;
 	return 0;
 }
 
@@ -1567,8 +1630,7 @@ STATIC int drbd_nl_invalidate_peer(struct drbd_conf *mdev, struct drbd_nl_cfg_re
 				   struct drbd_nl_cfg_reply *reply)
 {
 
-	reply->ret_code = drbd_request_state(mdev, NS2(conn, StartingSyncS,
-						      pdsk, Inconsistent));
+	reply->ret_code = drbd_request_state(mdev, NS(conn, StartingSyncS));
 
 	return 0;
 }
@@ -1697,6 +1759,15 @@ STATIC int drbd_nl_get_timeout_flag(struct drbd_conf *mdev, struct drbd_nl_cfg_r
 	return (int)((char *)tl - (char *)reply->tag_list);
 }
 
+STATIC int drbd_nl_start_ov(struct drbd_conf *mdev, struct drbd_nl_cfg_req *nlp,
+				    struct drbd_nl_cfg_reply *reply)
+{
+	reply->ret_code = drbd_request_state(mdev,NS(conn,VerifyS));
+
+	return 0;
+}
+
+
 STATIC struct drbd_conf *ensure_mdev(struct drbd_nl_cfg_req *nlp)
 {
 	struct drbd_conf *mdev;
@@ -1730,7 +1801,7 @@ STATIC struct drbd_conf *ensure_mdev(struct drbd_nl_cfg_req *nlp)
 struct cn_handler_struct {
 	int (*function)(struct drbd_conf *,
 			 struct drbd_nl_cfg_req *,
-			 struct drbd_nl_cfg_reply *);
+			 struct drbd_nl_cfg_reply* );
 	int reply_body_size;
 };
 
@@ -1761,7 +1832,7 @@ static struct cn_handler_struct cnd_table[] = {
 	[ P_get_timeout_flag ]	=
 		{ &drbd_nl_get_timeout_flag,
 		  sizeof(struct get_timeout_flag_tag_len_struct)},
-
+	[ P_start_ov ]          = { &drbd_nl_start_ov,          0 },
 };
 
 STATIC void drbd_connector_callback(void *data)
@@ -1796,13 +1867,6 @@ STATIC void drbd_connector_callback(void *data)
 	}
 
 	cm = cnd_table + nlp->packet_type;
-
-	/* This may happen if packet number is 0: */
-	if (cm->function == NULL) {
-		retcode = UnknownNetLinkPacket;
-		goto fail;
-	}
-
 	reply_size += cm->reply_body_size;
 
 	cn_reply = kmalloc(reply_size, GFP_KERNEL);
@@ -1842,6 +1906,61 @@ STATIC void drbd_connector_callback(void *data)
 
 static atomic_t drbd_nl_seq = ATOMIC_INIT(2); /* two. */
 
+static inline unsigned short *
+__tl_add_blob(unsigned short *tl, enum drbd_tags tag, const void *data,
+	int len, int nul_terminated)
+{
+	int l = tag_descriptions[tag_number(tag)].max_len;
+	l = (len < l) ? len :  l;
+	*tl++ = tag;
+	*tl++ = len;
+	memcpy(tl, data, len);
+	/* TODO
+	 * maybe we need to add some padding to the data stream.
+	 * otherwise we may get strange effects on architectures
+	 * that require certain data types to be strictly aligned,
+	 * because now the next "unsigned short" may be misaligned. */
+	tl = (unsigned short*)((char*)tl + len);
+	if (nul_terminated)
+		*((char*)tl - 1) = 0;
+	return tl;
+}
+
+static inline unsigned short *
+tl_add_blob(unsigned short *tl, enum drbd_tags tag, const void *data, int len)
+{
+	return __tl_add_blob(tl, tag, data, len, 0);
+}
+
+static inline unsigned short *
+tl_add_str(unsigned short *tl, enum drbd_tags tag, const char *str)
+{
+	return __tl_add_blob(tl, tag, str, strlen(str)+1, 0);
+}
+
+static inline unsigned short *
+tl_add_int(unsigned short *tl, enum drbd_tags tag, const void *val)
+{
+	switch(tag_type(tag)) {
+	case TT_INTEGER:
+		*tl++ = tag;
+		*tl++ = sizeof(int);
+		*(int*)tl = *(int*)val;
+		tl = (unsigned short*)((char*)tl+sizeof(int));
+		break;
+	case TT_INT64:
+		*tl++ = tag;
+		*tl++ = sizeof(u64);
+		*(u64*)tl = *(u64*)val;
+		tl = (unsigned short*)((char*)tl+sizeof(u64));
+		break;
+	default:
+		/* someone did something stupid. */
+		;
+	}
+	return tl;
+}
+
 void drbd_bcast_state(struct drbd_conf *mdev, union drbd_state_t state)
 {
 	char buffer[sizeof(struct cn_msg)+
@@ -1853,7 +1972,7 @@ void drbd_bcast_state(struct drbd_conf *mdev, union drbd_state_t state)
 		(struct drbd_nl_cfg_reply *)cn_reply->data;
 	unsigned short *tl = reply->tag_list;
 
-	/* drbd_WARN("drbd_bcast_state() got called\n"); */
+	/* WARN("drbd_bcast_state() got called\n"); */
 
 	tl = get_state_to_tags(mdev, (struct get_state *)&state, tl);
 	*tl++ = TT_END; /* Close the tag list */
@@ -1888,7 +2007,7 @@ void drbd_bcast_ev_helper(struct drbd_conf *mdev, char *helper_name)
 	unsigned short *tl = reply->tag_list;
 	int str_len;
 
-	/* drbd_WARN("drbd_bcast_state() got called\n"); */
+	/* WARN("drbd_bcast_state() got called\n"); */
 
 	str_len = strlen(helper_name)+1;
 	*tl++ = T_helper;
@@ -1915,6 +2034,77 @@ void drbd_bcast_ev_helper(struct drbd_conf *mdev, char *helper_name)
 	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
 }
 
+void drbd_bcast_ee(struct drbd_conf *mdev,
+		const char *reason, const int dgs,
+		const char* seen_hash, const char* calc_hash,
+		const struct Tl_epoch_entry* e)
+{
+	struct cn_msg *cn_reply;
+	struct drbd_nl_cfg_reply *reply;
+	struct bio_vec *bvec;
+	unsigned short *tl;
+	int i;
+
+	if (!e)
+		return;
+	if (!reason || !reason[0])
+		return;
+
+	/* aparently we have to memcpy twice, first to prepare the data for the
+	 * struct cn_msg, then within cn_netlink_send from the cn_msg to the
+	 * netlink skb. */
+	cn_reply = kmalloc(
+		sizeof(struct cn_msg)+
+		sizeof(struct drbd_nl_cfg_reply)+
+		sizeof(struct dump_ee_tag_len_struct)+
+		sizeof(short int)
+		, GFP_KERNEL);
+
+	if (!cn_reply) {
+		ERR("could not kmalloc buffer for drbd_bcast_ee, sector %llu, size %u\n",
+				(unsigned long long)e->sector, e->size);
+		return;
+	}
+
+	reply = (struct drbd_nl_cfg_reply*)cn_reply->data;
+	tl = reply->tag_list;
+
+	tl = tl_add_str(tl, T_dump_ee_reason, reason);
+	tl = tl_add_blob(tl, T_seen_digest, seen_hash, dgs);
+	tl = tl_add_blob(tl, T_calc_digest, calc_hash, dgs);
+	tl = tl_add_int(tl, T_ee_sector, &e->sector);
+	tl = tl_add_int(tl, T_ee_block_id, &e->block_id);
+
+	*tl++ = T_ee_data;
+	*tl++ = e->size;
+
+	__bio_for_each_segment(bvec, e->private_bio, i, 0) {
+		void *d = kmap(bvec->bv_page);
+		memcpy(tl, d + bvec->bv_offset, bvec->bv_len);
+		kunmap(bvec->bv_page);
+		tl=(unsigned short*)((char*)tl + bvec->bv_len);
+	}
+	*tl++ = TT_END; /* Close the tag list */
+
+	cn_reply->id.idx = CN_IDX_DRBD;
+	cn_reply->id.val = CN_VAL_DRBD;
+
+	cn_reply->seq = atomic_add_return(1,&drbd_nl_seq);
+	cn_reply->ack = 0; // not used here.
+	cn_reply->len = sizeof(struct drbd_nl_cfg_reply) +
+		(int)((char*)tl - (char*)reply->tag_list);
+	cn_reply->flags = 0;
+
+	reply->packet_type = P_dump_ee;
+	reply->minor = mdev_to_minor(mdev);
+	reply->ret_code = NoError;
+
+	TRACE(TraceTypeNl, TraceLvlSummary, nl_trace_reply(cn_reply););
+
+	cn_netlink_send(cn_reply, CN_IDX_DRBD, GFP_KERNEL);
+	kfree(cn_reply);
+}
+
 void drbd_bcast_sync_progress(struct drbd_conf *mdev)
 {
 	char buffer[sizeof(struct cn_msg)+
@@ -1965,24 +2155,15 @@ void __exit cn_fini(void);
 
 int __init drbd_nl_init(void)
 {
-	static struct cb_id cn_id_drbd;
-	int err, try=10;
+	static struct cb_id cn_id_drbd = { CN_IDX_DRBD, CN_VAL_DRBD };
+	int err;
 
 #ifdef NETLINK_ROUTE6
 	/* pre 2.6.16 */
 	err = cn_init();
-	if (err)
-		return err;
+	if(err) return err;
 #endif
-	cn_id_drbd.val = CN_VAL_DRBD;
-	do {
-		cn_id_drbd.idx = cn_idx;
-		err = cn_add_callback(&cn_id_drbd, "cn_drbd", &drbd_connector_callback);
-		if (!err)
-			break;
-		cn_idx = (cn_idx + CN_IDX_STEP);
-	} while (try--);
-
+	err = cn_add_callback(&cn_id_drbd, "cn_drbd", &drbd_connector_callback);
 	if (err) {
 		printk(KERN_ERR "drbd: cn_drbd failed to register\n");
 		return err;
@@ -1993,10 +2174,7 @@ int __init drbd_nl_init(void)
 
 void drbd_nl_cleanup(void)
 {
-	static struct cb_id cn_id_drbd;
-
-	cn_id_drbd.idx = cn_idx;
-	cn_id_drbd.val = CN_VAL_DRBD;
+	static struct cb_id cn_id_drbd = { CN_IDX_DRBD, CN_VAL_DRBD };
 
 	cn_del_callback(&cn_id_drbd);
 
@@ -2006,7 +2184,8 @@ void drbd_nl_cleanup(void)
 #endif
 }
 
-void drbd_nl_send_reply(struct cn_msg *req, int ret_code)
+void drbd_nl_send_reply( struct cn_msg *req,
+			 int ret_code)
 {
 	char buffer[sizeof(struct cn_msg)+sizeof(struct drbd_nl_cfg_reply)];
 	struct cn_msg *cn_reply = (struct cn_msg *) buffer;
