@@ -38,7 +38,7 @@
 #include "drbd_int.h"
 #include "lru_cache.h" /* for lc_sprintf_stats */
 
-STATIC int drbd_proc_open(struct inode *inode, struct file *file);
+int drbd_proc_open(struct inode *inode, struct file *file);
 
 
 struct proc_dir_entry *drbd_proc;
@@ -80,11 +80,11 @@ STATIC void drbd_syncer_progress(struct drbd_conf *mdev, struct seq_file *seq)
 	if (mdev->rs_total > 0x100000L)
 		seq_printf(seq, "(%lu/%lu)M\n\t",
 			    (unsigned long) Bit2KB(rs_left) >> 10,
-			    (unsigned long) Bit2KB(mdev->rs_total) >> 10);
+			    (unsigned long) Bit2KB(mdev->rs_total) >> 10 );
 	else
 		seq_printf(seq, "(%lu/%lu)K\n\t",
 			    (unsigned long) Bit2KB(rs_left),
-			    (unsigned long) Bit2KB(mdev->rs_total));
+			    (unsigned long) Bit2KB(mdev->rs_total) );
 
 	/* see drivers/md/md.c
 	 * We do not want to overflow, so the order of operands and
@@ -136,8 +136,8 @@ STATIC void drbd_syncer_progress(struct drbd_conf *mdev, struct seq_file *seq)
 	seq_printf(seq, " K/sec\n");
 }
 
-#ifdef DRBD_DUMP_RESYNC_DETAIL
-void resync_dump_detail(struct seq_file *seq, struct lc_element *e)
+#ifdef ENABLE_DYNAMIC_TRACE
+STATIC void resync_dump_detail(struct seq_file *seq, struct lc_element *e)
 {
 	struct bm_extent *bme = (struct bm_extent *)e;
 
@@ -154,8 +154,8 @@ STATIC int drbd_seq_show(struct seq_file *seq, void *v)
 	const char *sn;
 	struct drbd_conf *mdev;
 
-	seq_printf(seq, "version: " REL_VERSION " (api:%d/proto:%d)\n%s\n",
-		    API_VERSION, PRO_VERSION, drbd_buildtag());
+	seq_printf(seq, "version: " REL_VERSION " (api:%d/proto:%d-%d)\n%s\n",
+		   API_VERSION, PRO_VERSION_MIN, PRO_VERSION_MAX, drbd_buildtag());
 
 	/*
 	  cs .. connection state
@@ -178,20 +178,19 @@ STATIC int drbd_seq_show(struct seq_file *seq, void *v)
 		}
 		if (hole) {
 			hole = 0;
-			seq_printf(seq, "\n");
+			seq_printf( seq, "\n");
 		}
 
 		sn = conns_to_name(mdev->state.conn);
 
-		if (mdev->state.conn == StandAlone &&
-		    mdev->state.disk == Diskless &&
-		    mdev->state.role == Secondary) {
-			seq_printf(seq, "%2d: cs:Unconfigured\n", i);
+		if ( mdev->state.conn == StandAlone &&
+		     mdev->state.disk == Diskless) {
+			seq_printf( seq, "%2d: cs:Unconfigured\n", i);
 		} else {
-			seq_printf(seq,
+			seq_printf( seq,
 			   "%2d: cs:%s st:%s/%s ds:%s/%s %c %c%c%c%c\n"
 			   "    ns:%u nr:%u dw:%u dr:%u al:%u bm:%u "
-			   "lo:%d pe:%d ua:%d ap:%d\n",
+			   "lo:%d pe:%d ua:%d ap:%d",
 			   i, sn,
 			   roles_to_name(mdev->state.role),
 			   roles_to_name(mdev->state.peer),
@@ -215,23 +214,42 @@ STATIC int drbd_seq_show(struct seq_file *seq, void *v)
 			   atomic_read(&mdev->unacked_cnt),
 			   atomic_read(&mdev->ap_bio_cnt)
 			);
+			seq_printf(seq, " oos:%lu\n",
+				   drbd_bm_total_weight(mdev) << (BM_BLOCK_SIZE_B - 10));
 		}
-		if (mdev->state.conn == SyncSource ||
-		    mdev->state.conn == SyncTarget)
+		if ( mdev->state.conn == SyncSource ||
+		     mdev->state.conn == SyncTarget )
 			drbd_syncer_progress(mdev, seq);
 
-		if (inc_local_if_state(mdev, Failed)) {
+		if ( mdev->state.conn == VerifyS ||
+		     mdev->state.conn == VerifyT ) {
+			seq_printf(seq,"\t%3d%%      %lu/%lu\n",
+				   (int)((mdev->rs_total-mdev->ov_left) /
+					 (mdev->rs_total/100+1)),
+				   mdev->rs_total - mdev->ov_left,
+				   mdev->rs_total);
+		}
+
+#ifdef ENABLE_DYNAMIC_TRACE
+		if (proc_details >= 1 && inc_local_if_state(mdev, Failed)) {
 			lc_printf_stats(seq, mdev->resync);
 			lc_printf_stats(seq, mdev->act_log);
 			dec_local(mdev);
 		}
 
+		if (proc_details >= 2) {
+			if (mdev->resync) {
+				lc_dump(mdev->resync, seq, "rs_left",
+					resync_dump_detail);
+			}
+		}
+#endif
 	}
 
 	return 0;
 }
 
-STATIC int drbd_proc_open(struct inode *inode, struct file *file)
+int drbd_proc_open(struct inode *inode, struct file *file)
 {
 	return single_open(file, drbd_seq_show, PDE(inode)->data);
 }
