@@ -45,6 +45,8 @@ static const char *drbd_conn_s_names[] = {
 	[WFSyncUUID]     = "WFSyncUUID",
 	[SyncSource]     = "SyncSource",
 	[SyncTarget]     = "SyncTarget",
+	[VerifyS]        = "VerifyS",
+	[VerifyT]        = "VerifyT",
 	[PausedSyncS]    = "PausedSyncS",
 	[PausedSyncT]    = "PausedSyncT"
 };
@@ -75,16 +77,19 @@ static const char *drbd_state_sw_errors[] = {
 	[-SS_SyncingDiskless] = "Refusing to be syncing and diskless",
 	[-SS_ConnectedOutdates] = "Refusing to be Outdated while Connected",
 	[-SS_PrimaryNOP] = "Refusing to be Primary while peer is not outdated",
-	[-SS_ResyncRunning] = "Can not start resync since it is already active",
+	[-SS_ResyncRunning] = "Can not start OV/resync since it is already active",
 	[-SS_AlreadyStandAlone] = "Can not disconnect a StandAlone device",
 	[-SS_CW_FailedByPeer] = "State changed was refused by peer node",
 	[-SS_IsDiskLess] =
 		"Device is diskless, the requesed operation requires a disk",
 	[-SS_DeviceInUse] = "Device is held open by someone",
 	[-SS_NoNetConfig] = "Have no net/connection configuration",
+	[-SS_NoVerifyAlg] = "Need a verify algorithm to start online verify",
+	[-SS_NeedConnection] = "Need a connection to start verify or resync",
+	[-SS_NotSupported] = "Peer does not support protocol",
 	[-SS_LowerThanOutdated] = "Disk state is lower than outdated",
 	[-SS_InTransientState] = "In transient state, retry after next state change",
-	[-SS_ConcurrentStChg] = "Concurrent state changes detected and aborted"
+	[-SS_ConcurrentStChg] = "Concurrent state changes detected and aborted",
 };
 
 const char *conns_to_name(enum drbd_conns s)
@@ -105,7 +110,7 @@ const char *disks_to_name(enum drbd_disk_state s)
 
 const char *set_st_err_name(enum set_st_err err)
 {
-	return err <= SS_AfterLastError ? "TOO_SMALL" :
+	return err < SS_NotSupported ? "TOO_SMALL" :
 	       err > SS_TwoPrimaries ? "TOO_LARGE"
 			: drbd_state_sw_errors[-err];
 }
