@@ -37,6 +37,9 @@
 #include <linux/mm_inline.h>
 #include <linux/slab.h>
 #include <linux/random.h>
+#ifdef HAVE_LINUX_SCATTERLIST_H
+#include <linux/scatterlist.h>
+#endif
 
 #include <linux/drbd.h>
 #include "drbd_int.h"
@@ -67,6 +70,7 @@ BIO_ENDIO_TYPE drbd_md_io_complete BIO_ENDIO_ARGS(struct bio *bio, int error)
 	struct drbd_md_io *md_io;
 
 	BIO_ENDIO_FN_START;
+
 	/* error parameter ignored:
 	 * drbd_md_sync_page_io explicitly tests bio_uptodate(bio); */
 
@@ -77,6 +81,7 @@ BIO_ENDIO_TYPE drbd_md_io_complete BIO_ENDIO_ARGS(struct bio *bio, int error)
 	dump_internal_bio("Md", md_io->mdev, bio, 1);
 
 	complete(&md_io->event);
+
 	BIO_ENDIO_FN_RETURN;
 }
 
@@ -98,7 +103,7 @@ BIO_ENDIO_TYPE drbd_endio_read_sec BIO_ENDIO_ARGS(struct bio *bio, int error) __
 		/* strange behaviour of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
 		 * but do not return any error?!
-		 * do we want to drbd_WARN() on this? */
+		 * do we want to WARN() on this? */
 		error = -EIO;
 	}
 
@@ -109,8 +114,7 @@ BIO_ENDIO_TYPE drbd_endio_read_sec BIO_ENDIO_ARGS(struct bio *bio, int error) __
 	spin_lock_irqsave(&mdev->req_lock, flags);
 	mdev->read_cnt += e->size >> 9;
 	list_del(&e->w.list);
-	if (list_empty(&mdev->read_ee))
-		wake_up(&mdev->ee_wait);
+	if (list_empty(&mdev->read_ee)) wake_up(&mdev->ee_wait);
 	spin_unlock_irqrestore(&mdev->req_lock, flags);
 
 	drbd_chk_io_error(mdev, error, FALSE);
@@ -146,7 +150,7 @@ BIO_ENDIO_TYPE drbd_endio_write_sec BIO_ENDIO_ARGS(struct bio *bio, int error) _
 		/* strange behaviour of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
 		 * but do not return any error?!
-		 * do we want to drbd_WARN() on this? */
+		 * do we want to WARN() on this? */
 		error = -EIO;
 	}
 
@@ -219,7 +223,7 @@ BIO_ENDIO_TYPE drbd_endio_pri BIO_ENDIO_ARGS(struct bio *bio, int error)
 		/* strange behaviour of some lower level drivers...
 		 * fail the request by clearing the uptodate flag,
 		 * but do not return any error?!
-		 * do we want to drbd_WARN() on this? */
+		 * do we want to WARN() on this? */
 		error = -EIO;
 	}
 
@@ -234,6 +238,7 @@ BIO_ENDIO_TYPE drbd_endio_pri BIO_ENDIO_ARGS(struct bio *bio, int error)
 	spin_lock_irqsave(&mdev->req_lock, flags);
 	_req_mod(req, what, error);
 	spin_unlock_irqrestore(&mdev->req_lock, flags);
+
 	BIO_ENDIO_FN_RETURN;
 }
 
@@ -256,8 +261,7 @@ int w_io_error(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	drbd_req_free(req);
 
 	ok = drbd_io_error(mdev, FALSE);
-	if (unlikely(!ok))
-		ERR("Sending in w_io_error() failed\n");
+	if (unlikely(!ok)) ERR("Sending in w_io_error() failed\n");
 	return ok;
 }
 
@@ -268,14 +272,15 @@ int w_read_retry_remote(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	/* FIXME this is ugly. we should not detach for read io-error,
 	 * but try to WRITE the DataReply to the failed location,
 	 * to give the disk the chance to relocate that block */
-	drbd_io_error(mdev, FALSE); /* tries to schedule a detach and notifies peer */
+	drbd_io_error(mdev,FALSE); /* tries to schedule a detach and notifies peer */
 
 	spin_lock_irq(&mdev->req_lock);
-	if (cancel ||
-	    mdev->state.conn < Connected ||
-	    mdev->state.pdsk <= Inconsistent) {
+	if ( cancel ||
+	     mdev->state.conn < Connected ||
+	     mdev->state.pdsk <= Inconsistent ) {
 		_req_mod(req, send_canceled, 0); /* FIXME freeze? ... */
 		spin_unlock_irq(&mdev->req_lock);
+		drbd_khelper(mdev, "pri-on-incon-degr"); /* FIXME REALLY? */
 		ALERT("WE ARE LOST. Local IO failure, no peer.\n");
 		return 1;
 	}
@@ -291,6 +296,29 @@ int w_resync_inactive(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	return 1; /* Simply ignore this! */
 }
 
+STATIC void drbd_csum(struct drbd_conf *mdev, struct crypto_hash *tfm, struct bio *bio, void *digest)
+{
+	struct hash_desc desc;
+	struct scatterlist sg;
+	struct bio_vec *bvec;
+	int i;
+
+	desc.tfm=tfm;
+	desc.flags=0;
+
+	sg_init_table(&sg, 1);
+	crypto_hash_init(&desc);
+
+	__bio_for_each_segment(bvec, bio, i, 0) {
+		sg_set_page(&sg, bvec->bv_page, bvec->bv_len, bvec->bv_offset);
+		crypto_hash_update(&desc, &sg, sg.length);
+	}
+	crypto_hash_final(&desc, digest);
+}
+
+int w_make_ov_request(struct drbd_conf *mdev, struct drbd_work* w,int cancel);
+
+
 void resync_timer_fn(unsigned long data)
 {
 	unsigned long flags;
@@ -301,7 +329,9 @@ void resync_timer_fn(unsigned long data)
 
 	if (likely(!test_and_clear_bit(STOP_SYNC_TIMER, &mdev->flags))) {
 		queue = 1;
-		mdev->resync_work.cb = w_make_resync_request;
+		if(mdev->state.conn == VerifyS ) {
+			mdev->resync_work.cb = w_make_ov_request;
+		} else mdev->resync_work.cb = w_make_resync_request;
 	} else {
 		queue = 0;
 		mdev->resync_work.cb = w_resync_inactive;
@@ -328,8 +358,7 @@ int w_make_resync_request(struct drbd_conf *mdev,
 
 	PARANOIA_BUG_ON(w != &mdev->resync_work);
 
-	if (unlikely(cancel))
-		return 1;
+	if (unlikely(cancel)) return 1;
 
 	if (unlikely(mdev->state.conn < Connected)) {
 		ERR("Confused in w_make_resync_request()! cstate < Connected");
@@ -379,7 +408,7 @@ next_sector:
 			goto requeue;
 		}
 
-		if (unlikely(drbd_bm_test_bit(mdev, bit) == 0)) {
+		if (unlikely(drbd_bm_test_bit(mdev, bit) == 0 )) {
 			drbd_rs_complete_io(mdev, sector);
 			goto next_sector;
 		}
@@ -403,23 +432,22 @@ next_sector:
 				break;
 
 			/* Be always aligned */
-			if (sector & ((1<<(align+3))-1))
+			if (sector & ((1<<(align+3))-1) )
 				break;
 
 			/* do not cross extent boundaries */
-			if (((bit+1) & BM_BLOCKS_PER_BM_EXT_MASK) == 0)
+			if (( (bit+1) & BM_BLOCKS_PER_BM_EXT_MASK ) == 0)
 				break;
 			/* now, is it actually dirty, after all?
 			 * caution, drbd_bm_test_bit is tri-state for some
 			 * obscure reason; ( b == 0 ) would get the out-of-band
 			 * only accidentally right because of the "oddly sized"
 			 * adjustment below */
-			if (drbd_bm_test_bit(mdev, bit+1) != 1)
+			if ( drbd_bm_test_bit(mdev, bit+1) != 1 )
 				break;
 			bit++;
 			size += BM_BLOCK_SIZE;
-			if ((BM_BLOCK_SIZE << align) <= size)
-				align++;
+			if ( (BM_BLOCK_SIZE<<align) <= size) align++;
 			i++;
 		}
 		/* if we merged some,
@@ -459,6 +487,65 @@ next_sector:
 	return 1;
 }
 
+int w_make_ov_request(struct drbd_conf *mdev, struct drbd_work* w,int cancel)
+{
+	int number,i,size;
+	sector_t sector;
+	const sector_t capacity = drbd_get_capacity(mdev->this_bdev);
+
+	if(unlikely(cancel)) return 1;
+
+	if (unlikely(mdev->state.conn < Connected)) {
+		ERR("Confused in w_make_ov_request()! cstate < Connected");
+		return 0;
+	}
+
+	number = SLEEP_TIME*mdev->sync_conf.rate / ((BM_BLOCK_SIZE/1024)*HZ);
+	if (atomic_read(&mdev->rs_pending_cnt)>number) {
+		goto requeue;
+	}
+	number -= atomic_read(&mdev->rs_pending_cnt);
+
+	sector = mdev->ov_position;
+	for(i=0;i<number;i++) {
+		size = BM_BLOCK_SIZE;
+
+		if (drbd_try_rs_begin_io(mdev, sector)) {
+			mdev->ov_position = sector;
+			goto requeue;
+		}
+
+		if (sector + (size>>9) > capacity) size = (capacity-sector)<<9;
+
+		inc_rs_pending(mdev);
+		if(!drbd_send_ov_request(mdev, sector, size)) {
+			dec_rs_pending(mdev);
+			return 0;
+		}
+		sector += BM_SECT_PER_BIT;
+		if(sector >= capacity) {
+			mdev->resync_work.cb = w_resync_inactive;
+
+			return 1;
+		}
+	}
+	mdev->ov_position = sector;
+
+ requeue:
+	mod_timer(&mdev->resync_timer, jiffies + SLEEP_TIME);
+	return 1;
+}
+
+
+int w_ov_finished(struct drbd_conf *mdev, struct drbd_work* w,int cancel)
+{
+	kfree(w);
+	ov_oos_print(mdev);
+	drbd_resync_finished(mdev);
+
+	return 1;
+}
+
 STATIC int w_resync_finished(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
 	kfree(w);
@@ -470,9 +557,10 @@ STATIC int w_resync_finished(struct drbd_conf *mdev, struct drbd_work *w, int ca
 
 int drbd_resync_finished(struct drbd_conf *mdev)
 {
-	unsigned long db, dt, dbdt;
+	unsigned long db,dt,dbdt;
 	union drbd_state_t os, ns;
 	struct drbd_work *w;
+	int art = 0;
 
 	/* Remove all elements from the resync LRU. Since future actions
 	 * might set bits in the (main) bitmap, then the entries in the
@@ -484,7 +572,7 @@ int drbd_resync_finished(struct drbd_conf *mdev)
 		 * is not finished by now).   Retry in 100ms. */
 
 		drbd_kick_lo(mdev);
-		__set_current_state(TASK_INTERRUPTIBLE);
+		set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(HZ / 10);
 		w = kmalloc(sizeof(struct drbd_work), GFP_ATOMIC);
 		if (w) {
@@ -516,10 +604,20 @@ int drbd_resync_finished(struct drbd_conf *mdev)
 	ns = os;
 	ns.conn = Connected;
 
-	INFO("Resync done (total %lu sec; paused %lu sec; %lu K/sec)\n",
+	INFO("%s done (total %lu sec; paused %lu sec; %lu K/sec)\n",
+	     (os.conn == VerifyS || os.conn == VerifyT) ?
+	     "Online verify ": "Resync",
 	     dt + mdev->rs_paused, mdev->rs_paused, dbdt);
 
-	D_ASSERT((drbd_bm_total_weight(mdev)-mdev->rs_failed) == 0);
+	if (os.conn == VerifyS || os.conn == VerifyT) {
+		if (drbd_bm_total_weight(mdev)) {
+			ALERT("Online verify found %lu %dk block out of sync!\n",
+			      drbd_bm_total_weight(mdev),BM_BLOCK_SIZE/1024);
+			drbd_khelper(mdev,"out-of-sync");
+		}
+	} else {
+		D_ASSERT((drbd_bm_total_weight(mdev)-mdev->rs_failed) == 0);
+	}
 
 	if (mdev->rs_failed) {
 		INFO("            %lu failed blocks\n", mdev->rs_failed);
@@ -558,22 +656,27 @@ int drbd_resync_finished(struct drbd_conf *mdev)
 		}
 	}
 
+	art = os.conn == SyncTarget || os.conn == PausedSyncT;
+
 	_drbd_set_state(mdev, ns, ChgStateVerbose, NULL);
-out_unlock:
+  out_unlock:
 	spin_unlock_irq(&mdev->req_lock);
 	dec_local(mdev);
-out:
+  out:
 	mdev->rs_total  = 0;
 	mdev->rs_failed = 0;
 	mdev->rs_paused = 0;
 
 	if (test_and_clear_bit(WRITE_BM_AFTER_RESYNC, &mdev->flags)) {
-		drbd_WARN("Writing the whole bitmap, due to failed kmalloc\n");
-		drbd_queue_bitmap_io(mdev, &drbd_bm_write, NULL, "write from resync_finished");
+		WARN("Writing the whole bitmap, due to failed kmalloc\n");
+		drbd_queue_bitmap_io(mdev, &drbd_bm_write, NULL);
 	}
 
 	drbd_bm_recount_bits(mdev);
 
+	if (art)
+		drbd_khelper(mdev, "after-resync-target");
+
 	return 1;
 }
 
@@ -617,8 +720,7 @@ int w_e_end_data_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	}
 	spin_unlock_irq(&mdev->req_lock);
 
-	if (unlikely(!ok))
-		ERR("drbd_send_block() failed\n");
+	if (unlikely(!ok)) ERR("drbd_send_block() failed\n");
 	return ok;
 }
 
@@ -642,7 +744,7 @@ int w_e_end_rsdata_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	}
 
 	if (likely(drbd_bio_uptodate(e->private_bio))) {
-		if (likely(mdev->state.pdsk >= Inconsistent)) {
+		if (likely( mdev->state.pdsk >= Inconsistent )) {
 			inc_rs_pending(mdev);
 			ok = drbd_send_block(mdev, RSDataReply, e);
 		} else {
@@ -675,11 +777,117 @@ int w_e_end_rsdata_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	}
 	spin_unlock_irq(&mdev->req_lock);
 
-	if (unlikely(!ok))
-		ERR("drbd_send_block() failed\n");
+	if (unlikely(!ok)) ERR("drbd_send_block() failed\n");
 	return ok;
 }
 
+
+int w_e_end_ov_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
+{
+	struct Tl_epoch_entry *e = (struct Tl_epoch_entry*)w;
+	int digest_size;
+	void *digest;
+	int ok=1;
+
+	if(unlikely(cancel)) {
+		drbd_free_ee(mdev,e);
+		dec_unacked(mdev);
+		return 1;
+	}
+
+	if(likely(drbd_bio_uptodate(e->private_bio))) {
+		digest_size = crypto_hash_digestsize(mdev->verify_tfm);
+		digest = kmalloc(digest_size,GFP_KERNEL);
+		if(digest) {
+			drbd_csum(mdev,mdev->verify_tfm,e->private_bio,digest);
+			ok = drbd_send_drequest_csum(mdev, e->sector, e->size,
+						     digest, digest_size, OVReply);
+			if (ok) inc_rs_pending(mdev);
+			kfree(digest);
+		}
+	}
+
+	dec_unacked(mdev);
+
+	spin_lock_irq(&mdev->req_lock);
+	drbd_free_ee(mdev,e);
+	spin_unlock_irq(&mdev->req_lock);
+
+	return ok;
+}
+
+void drbd_ov_oos_found(struct drbd_conf *mdev, sector_t sector, int size)
+{
+	if (mdev->ov_last_oos_start + mdev->ov_last_oos_size == sector) {
+		mdev->ov_last_oos_size += size>>9;
+	} else {
+		mdev->ov_last_oos_start = sector;
+		mdev->ov_last_oos_size = size>>9;
+	}
+	drbd_set_out_of_sync(mdev, sector, size);
+	set_bit(WRITE_BM_AFTER_RESYNC, &mdev->flags);
+}
+
+int w_e_end_ov_reply(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
+{
+	struct Tl_epoch_entry *e = (struct Tl_epoch_entry*)w;
+	struct digest_info *di;
+	int digest_size;
+	void *digest;
+	int ok,eq=0;
+
+	if(unlikely(cancel)) {
+		drbd_free_ee(mdev,e);
+		dec_unacked(mdev);
+		return 1;
+	}
+
+	/* after "cancel", because after drbd_disconnect/drbd_rs_cancel_all
+	 * the resync lru has been cleaned up already */
+	drbd_rs_complete_io(mdev,e->sector);
+
+	di = (struct digest_info *)(unsigned long)e->block_id;
+
+	if(likely(drbd_bio_uptodate(e->private_bio))) {
+		digest_size = crypto_hash_digestsize(mdev->verify_tfm);
+		digest = kmalloc(digest_size,GFP_KERNEL);
+		if(digest) {
+			drbd_csum(mdev, mdev->verify_tfm, e->private_bio, digest);
+
+			D_ASSERT(digest_size == di->digest_size);
+			eq = !memcmp(digest, di->digest, digest_size);
+			kfree(digest);
+		}
+	} else {
+		ok=drbd_send_ack(mdev,NegRSDReply,e);
+		if (DRBD_ratelimit(5*HZ,5))
+			ERR("Sending NegDReply. I guess it gets messy.\n");
+		drbd_io_error(mdev, FALSE);
+	}
+
+	dec_unacked(mdev);
+
+	kfree(di);
+
+	if (!eq) drbd_ov_oos_found(mdev,e->sector,e->size);
+	else ov_oos_print(mdev);
+
+	ok = drbd_send_ack_ex(mdev,OVResult,e->sector,e->size,
+			      eq ? ID_IN_SYNC : ID_OUT_OF_SYNC);
+
+	spin_lock_irq(&mdev->req_lock);
+	drbd_free_ee(mdev,e);
+	spin_unlock_irq(&mdev->req_lock);
+
+	if( --mdev->ov_left == 0 ) {
+		ov_oos_print(mdev);
+		drbd_resync_finished(mdev);
+	}
+
+	return ok;
+}
+
+
 int w_prev_work_done(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 {
 	clear_bit(WORK_PENDING, &mdev->flags);
@@ -760,13 +968,17 @@ int w_send_read_req(struct drbd_conf *mdev, struct drbd_work *w, int cancel)
 	ok = drbd_send_drequest(mdev, DataRequest, req->sector, req->size,
 				(unsigned long)req);
 
-	if (!ok) {
-		/* ?? we set Timeout or BrokenPipe in drbd_send();
-		 * so this is probably redundant */
+	if (ok) {
+		req_mod(req, handed_over_to_network, 0);
+	} else {
+		/* ?? we set Timeout or BrokenPipe in drbd_send() */
 		if (mdev->state.conn >= Connected)
 			drbd_force_state(mdev, NS(conn, NetworkFailure));
+		/* req_mod(req, send_failed); we should not fail it here,
+		 * we might have to "freeze" on disconnect.
+		 * handled by req_mod(req, connection_lost_while_pending);
+		 * in drbd_fail_pending_reads soon enough. */
 	}
-	req_mod(req, ok ? handed_over_to_network : send_failed, 0);
 
 	return ok;
 }
@@ -812,11 +1024,10 @@ STATIC int _drbd_may_sync_now(struct drbd_conf *mdev)
 			return 1;
 		odev = minor_to_mdev(odev->sync_conf.after);
 		ERR_IF(!odev) return 1;
-		if ((odev->state.conn >= SyncSource &&
+		if ( (odev->state.conn >= SyncSource &&
 		     odev->state.conn <= PausedSyncT) ||
 		    odev->state.aftr_isp || odev->state.peer_isp ||
-		    odev->state.user_isp)
-			return 0;
+		    odev->state.user_isp ) return 0;
 	}
 }
 
@@ -838,8 +1049,8 @@ STATIC int _drbd_pause_after(struct drbd_conf *mdev)
 		if (odev->state.conn == StandAlone && odev->state.disk == Diskless)
 			continue;
 		if (!_drbd_may_sync_now(odev))
-			rv |= (_drbd_set_state(_NS(odev, aftr_isp, 1), ChgStateHard, NULL)
-				!= SS_NothingToDo);
+			rv |= ( _drbd_set_state(_NS(odev, aftr_isp, 1), ChgStateHard, NULL)
+				!= SS_NothingToDo ) ;
 	}
 
 	return rv;
@@ -910,11 +1121,11 @@ void drbd_alter_sa(struct drbd_conf *mdev, int na)
 void drbd_start_resync(struct drbd_conf *mdev, enum drbd_conns side)
 {
 	union drbd_state_t ns;
-	int r = 0;
+	int r;
 
 	MTRACE(TraceTypeResync, TraceLvlSummary,
 	       INFO("Resync starting: side=%s\n",
-		    side == SyncTarget ? "SyncTarget" : "SyncSource");
+		    side == SyncTarget?"SyncTarget":"SyncSource");
 	    );
 
 	drbd_bm_recount_bits(mdev);
@@ -922,6 +1133,20 @@ void drbd_start_resync(struct drbd_conf *mdev, enum drbd_conns side)
 	/* In case a previous resync run was aborted by an IO error... */
 	drbd_rs_cancel_all(mdev);
 
+	if (side == SyncTarget) {
+		/* Since application IO was locked out during WFBitMapT and
+		   WFSyncUUID we are still unmodified. Before going to SyncTarget
+		   we check that we might make the data inconsistent. */
+		r = drbd_khelper(mdev, "before-resync-target");
+		r = (r >> 8) & 0xff;
+		if (r > 0) {
+			INFO("before-resync-target handler returned %d, "
+			     "dropping connection.\n", r);
+			drbd_force_state(mdev, NS(conn, Disconnecting));
+			return;
+		}
+	}
+
 	drbd_state_lock(mdev);
 
 	if (!inc_local_if_state(mdev, Negotiating)) {
@@ -1000,13 +1225,14 @@ int drbd_worker(struct Drbd_thread *thi)
 	int intr = 0, i;
 
 	sprintf(current->comm, "drbd%d_worker", mdev_to_minor(mdev));
+	set_cpus_allowed(current, drbd_calc_cpu_mask(mdev));
 
 	while (get_t_state(thi) == Running) {
 
 		if (down_trylock(&mdev->data.work.s)) {
 			down(&mdev->data.mutex);
 			if (mdev->data.socket)
-				drbd_tcp_uncork(mdev->data.socket);
+				drbd_tcp_flush(mdev->data.socket);
 			up(&mdev->data.mutex);
 
 			intr = down_interruptible(&mdev->data.work.s);
@@ -1025,8 +1251,7 @@ int drbd_worker(struct Drbd_thread *thi)
 			break;
 		}
 
-		if (get_t_state(thi) != Running)
-			break;
+		if (get_t_state(thi) != Running) break;
 		/* With this break, we have done a down() but not consumed
 		   the entry from the list. The cleanup code takes care of
 		   this...   */
@@ -1052,8 +1277,8 @@ int drbd_worker(struct Drbd_thread *thi)
 		list_del_init(&w->list);
 		spin_unlock_irq(&mdev->data.work.q_lock);
 
-		if (!w->cb(mdev, w, mdev->state.conn < Connected)) {
-			/* drbd_WARN("worker: a callback failed! \n"); */
+		if (!w->cb(mdev, w, mdev->state.conn < Connected )) {
+			/* WARN("worker: a callback failed! \n"); */
 			if (mdev->state.conn >= Connected)
 				drbd_force_state(mdev,
 						NS(conn, NetworkFailure));
@@ -1089,9 +1314,6 @@ int drbd_worker(struct Drbd_thread *thi)
 	 * device.. */
 
 	D_ASSERT(mdev->state.disk == Diskless && mdev->state.conn == StandAlone);
-	/* _drbd_set_state only uses stop_nowait.
-	 * wait here for the Exiting receiver. */
-	drbd_thread_stop(&mdev->receiver);
 	drbd_mdev_cleanup(mdev);
 
 	INFO("worker terminated\n");
