@@ -1,3 +1,5 @@
+#include <linux/ctype.h>
+
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 # error "use a 2.6 kernel, please"
@@ -45,8 +47,7 @@ static inline int drbd_bio_has_active_page(struct bio *bio)
 	int i;
 
 	__bio_for_each_segment(bvec, bio, i, 0) {
-		if (page_count(bvec->bv_page) > 1)
-			return 1;
+		if (page_count(bvec->bv_page) > 1) return 1;
 	}
 
 	return 0;
@@ -63,11 +64,11 @@ static inline int drbd_bio_has_active_page(struct bio *bio)
 #else
 #define BIO_ENDIO_TYPE void
 #define BIO_ENDIO_ARGS(b,e) (b,e)
-#define BIO_ENDIO_FN_START do {} while (0)
+#define BIO_ENDIO_FN_START while(0) {}
 #define BIO_ENDIO_FN_RETURN return
 #endif
 
-/* bi_end_io handlers */
+// bi_end_io handlers
 extern BIO_ENDIO_TYPE drbd_md_io_complete BIO_ENDIO_ARGS(struct bio *bio, int error);
 extern BIO_ENDIO_TYPE drbd_endio_read_sec BIO_ENDIO_ARGS(struct bio *bio, int error);
 extern BIO_ENDIO_TYPE drbd_endio_write_sec BIO_ENDIO_ARGS(struct bio *bio, int error);
@@ -79,11 +80,6 @@ extern BIO_ENDIO_TYPE drbd_endio_pri BIO_ENDIO_ARGS(struct bio *bio, int error);
 #define kmem_cache_create(N,S,A,F,C) kmem_cache_create(N,S,A,F,C,NULL)
 #endif
 
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
-# undef HAVE_bvec_merge_data
-# define HAVE_bvec_merge_data 1
-#endif
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
 static inline void sg_set_page(struct scatterlist *sg, struct page *page,
 			       unsigned int len, unsigned int offset)
@@ -156,7 +152,7 @@ static inline void drbd_plug_device(struct drbd_conf *mdev)
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,23)
 static inline void drbd_unregister_blkdev(unsigned int major, const char *name)
 {
-	int ret = unregister_blkdev(major, name);
+	int ret = unregister_blkdev(major,name);
 	if (ret)
 		printk(KERN_ERR "drbd: unregister of device failed\n");
 }
@@ -180,7 +176,7 @@ static __inline__ int atomic_add_return(int i, atomic_t *v)
 
 static __inline__ int atomic_sub_return(int i, atomic_t *v)
 {
-	return atomic_add_return(-i, v);
+	return atomic_add_return(-i,v);
 }
 
 #define atomic_inc_return(v)  (atomic_add_return(1,v))
@@ -216,7 +212,7 @@ no_xadd: /* Legacy 386 processor */
 
 static __inline__ int atomic_sub_return(int i, atomic_t *v)
 {
-	return atomic_add_return(-i, v);
+	return atomic_add_return(-i,v);
 }
 
 #define atomic_inc_return(v)  (atomic_add_return(1,v))
@@ -255,16 +251,22 @@ crypto_alloc_hash(char *alg_name, u32 type, u32 mask)
 	struct crypto_hash *ch;
 	char *closing_bracket;
 
-	/* "hmac(xxx)" is in alg_name we need that xxx. */
-	closing_bracket = strchr(alg_name, ')');
-	if (!closing_bracket)
-		return ERR_PTR(-ENOENT);
-	if (closing_bracket-alg_name < 6)
-		return ERR_PTR(-ENOENT);
+	// "hmac(xxx)" is in alg_name we need that xxx.
+	closing_bracket = strchr(alg_name,')');
+	if(!closing_bracket) {
+		ch = kmalloc(sizeof(struct crypto_hash),GFP_KERNEL);
+		if(!ch) return ERR_PTR(-ENOMEM);
+		ch->base = crypto_alloc_tfm(alg_name, 0);
+		if (ch->base == NULL) {
+			kfree(ch);
+			return ERR_PTR(-ENOMEM);
+		}
+		return ch;
+	}
+	if(closing_bracket-alg_name < 6) return ERR_PTR(-ENOENT);
 
-	ch = kmalloc(sizeof(struct crypto_hash), GFP_KERNEL);
-	if (!ch)
-		return ERR_PTR(-ENOMEM);
+	ch = kmalloc(sizeof(struct crypto_hash),GFP_KERNEL);
+	if(!ch) return ERR_PTR(-ENOMEM);
 
 	*closing_bracket = 0;
 	ch->base = crypto_alloc_tfm(alg_name + 5, 0);
@@ -279,7 +281,7 @@ crypto_alloc_hash(char *alg_name, u32 type, u32 mask)
 }
 
 static inline int
-crypto_hash_setkey(struct crypto_hash *hash, const u8 *key, unsigned int keylen)
+crypto_hash_setkey(struct crypto_hash *hash,const u8 *key,unsigned int keylen)
 {
 	hash->key = key;
 	hash->keylen = keylen;
@@ -292,7 +294,7 @@ crypto_hash_digest(struct hash_desc *desc, struct scatterlist *sg,
 		   unsigned int nbytes, u8 *out)
 {
 
-	crypto_hmac(desc->tfm->base, (u8 *)desc->tfm->key,
+	crypto_hmac(desc->tfm->base, (u8*)desc->tfm->key,
 		    &desc->tfm->keylen, sg, 1 /* ! */ , out);
 	/* ! this is not generic. Would need to convert nbytes -> nsg */
 
@@ -301,9 +303,8 @@ crypto_hash_digest(struct hash_desc *desc, struct scatterlist *sg,
 
 static inline void crypto_free_hash(struct crypto_hash *tfm)
 {
-	if (!tfm)
-		return;
-	crypto_free_tfm(tfm->base);
+	if (!tfm) return;
+	crypto_free_tfm(tfm->base); 
 	kfree(tfm);
 }
 
@@ -317,19 +318,119 @@ static inline struct crypto_tfm *crypto_hash_tfm(struct crypto_hash *tfm)
 	return tfm->base;
 }
 
+static inline int crypto_hash_init(struct hash_desc *desc)
+{
+	crypto_digest_init(desc->tfm->base);
+	return 0;
+}
+
+static inline int crypto_hash_update(struct hash_desc *desc,
+				     struct scatterlist *sg,
+				     unsigned int nbytes)
+{
+	crypto_digest_update(desc->tfm->base,sg,1 /* ! */ );
+	/* ! this is not generic. Would need to convert nbytes -> nsg */
+
+	return 0;
+}
+
+static inline int crypto_hash_final(struct hash_desc *desc, u8 *out)
+{
+	crypto_digest_final(desc->tfm->base, out);
+	return 0;
+}
+
 #endif
 
 #ifdef NEED_BACKPORT_OF_KZALLOC
 static inline void *kzalloc(size_t size, int flags)
 {
-	void *rv = kmalloc(size, flags);
-	if (rv)
-		memset(rv, 0, size);
+	void *rv = kmalloc(size,flags);
+	if(rv) memset(rv,0,size);
 
 	return rv;
 }
 #endif
 
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#define __bitmap_parse(BUF, BUFLEN, ISUSR, MASKP, NMASK) \
+	backport_bitmap_parse(BUF, BUFLEN, ISUSR, MASKP, NMASK)
+
+#define CHUNKSZ                         32
+#define nbits_to_hold_value(val)        fls(val)
+#define unhex(c)                        (isdigit(c) ? (c - '0') : (toupper(c) - 'A' + 10))
+
+static inline int backport_bitmap_parse(const char *buf, unsigned int buflen,
+		int is_user, unsigned long *maskp,
+		int nmaskbits)
+{
+	int c, old_c, totaldigits, ndigits, nchunks, nbits;
+	u32 chunk;
+	const char __user *ubuf = buf;
+
+	bitmap_zero(maskp, nmaskbits);
+
+	nchunks = nbits = totaldigits = c = 0;
+	do {
+		chunk = ndigits = 0;
+
+		/* Get the next chunk of the bitmap */
+		while (buflen) {
+			old_c = c;
+			if (is_user) {
+				if (__get_user(c, ubuf++))
+					return -EFAULT;
+			}
+			else
+				c = *buf++;
+			buflen--;
+			if (isspace(c))
+				continue;
+
+			/*
+			 * If the last character was a space and the current
+			 * character isn't '\0', we've got embedded whitespace.
+			 * This is a no-no, so throw an error.
+			 */
+			if (totaldigits && c && isspace(old_c))
+				return -EINVAL;
+
+			/* A '\0' or a ',' signal the end of the chunk */
+			if (c == '\0' || c == ',')
+				break;
+
+			if (!isxdigit(c))
+				return -EINVAL;
+
+			/*
+			 * Make sure there are at least 4 free bits in 'chunk'.
+			 * If not, this hexdigit will overflow 'chunk', so
+			 * throw an error.
+			 */
+			if (chunk & ~((1UL << (CHUNKSZ - 4)) - 1))
+				return -EOVERFLOW;
+
+			chunk = (chunk << 4) | unhex(c);
+			ndigits++; totaldigits++;
+		}
+		if (ndigits == 0)
+			return -EINVAL;
+		if (nchunks == 0 && chunk == 0)
+			continue;
+
+		bitmap_shift_left(maskp, maskp, CHUNKSZ, nmaskbits);
+		*maskp |= chunk;
+		nchunks++;
+		nbits += (nchunks == 1) ? nbits_to_hold_value(chunk) : CHUNKSZ;
+		if (nbits > nmaskbits)
+			return -EOVERFLOW;
+	} while (buflen && c == ',');
+
+	return 0;
+}
+#endif
+
 #ifndef __CHECKER__
 # undef __cond_lock
 # define __cond_lock(x,c) (c)
