@@ -26,7 +26,6 @@
 #ifndef DRBD_H
 #define DRBD_H
 #include <linux/drbd_config.h>
-#include <linux/connector.h>
 
 #include <asm/types.h>
 
@@ -38,7 +37,7 @@
 #include <sys/wait.h>
 #include <limits.h>
 
-/* Altough the Linux source code makes a difference between
+/* Altough the Linux source code makes a difference between 
    generic endiness and the bitfields' endianess, there is no
    architecture as of Linux-2.6.24-rc4 where the bitfileds' endianess
    does not match the generic endianess. */
@@ -130,19 +129,19 @@ enum ret_codes {
 	HaveNoDiskConfig,
 	ProtocolCRequired,
 	VMallocFailed,
-	IntegrityAlgNotAvail,	/* DRBD 8.2 only */
-	IntegrityAlgNotDigest,	/* DRBD 8.2 only */
-	CPUMaskParseFailed,	/* DRBD 8.2 only */
-	CSUMSAlgNotAvail,	/* DRBD 8.2 only */
-	CSUMSAlgNotDigest,	/* DRBD 8.2 only */
-	VERIFYAlgNotAvail,	/* DRBD 8.2 only */
-	VERIFYAlgNotDigest,	/* DRBD 8.2 only */
-	CSUMSResyncRunning,	/* DRBD 8.2 only */
-	VERIFYIsRunning,	/* DRBD 8.2 only */
+	IntegrityAlgNotAvail,
+	IntegrityAlgNotDigest,
+	CPUMaskParseFailed,
+	CSUMSAlgNotAvail,
+	CSUMSAlgNotDigest,
+	VERIFYAlgNotAvail,
+	VERIFYAlgNotDigest,
+	CSUMSResyncRunning,
+	VERIFYIsRunning,
 	DataOfWrongCurrent,
 
 	/* insert new ones above this line */
-	AfterLastRetCode
+	AfterLastRetCode,
 };
 
 #define DRBD_PROT_A   1
@@ -189,10 +188,14 @@ enum drbd_conns {
 	WFBitMapT,
 	WFSyncUUID,
 
-	/* All SyncStates are tested with this comparison
+	/* The distance between original state and pause
+	 * state must be the same for source and target. (+2)
+	 * All SyncStates are tested with this comparison
 	 * xx >= SyncSource && xx <= PausedSyncT */
 	SyncSource,
 	SyncTarget,
+	VerifyS,
+	VerifyT,
 	PausedSyncS,
 	PausedSyncT,
 	conn_mask = 31
@@ -223,27 +226,27 @@ union drbd_state_t {
  */
 	struct {
 #if defined(__LITTLE_ENDIAN_BITFIELD)
-		unsigned role:2 ;   /* 3/4	 primary/secondary/unknown */
-		unsigned peer:2 ;   /* 3/4	 primary/secondary/unknown */
-		unsigned conn:5 ;   /* 17/32	 cstates */
-		unsigned disk:4 ;   /* 8/16	 from Diskless to UpToDate */
-		unsigned pdsk:4 ;   /* 8/16	 from Diskless to UpToDate */
-		unsigned susp:1 ;   /* 2/2	 IO suspended  no/yes */
-		unsigned aftr_isp:1 ; /* isp .. imposed sync pause */
-		unsigned peer_isp:1 ;
-		unsigned user_isp:1 ;
-		unsigned _pad:11;   /* 0	 unused */
+		unsigned role : 2 ;   /* 3/4	 primary/secondary/unknown */
+		unsigned peer : 2 ;   /* 3/4	 primary/secondary/unknown */
+		unsigned conn : 5 ;   /* 17/32	 cstates */
+		unsigned disk : 4 ;   /* 8/16	 from Diskless to UpToDate */
+		unsigned pdsk : 4 ;   /* 8/16	 from Diskless to UpToDate */
+		unsigned susp : 1 ;   /* 2/2	 IO suspended  no/yes */
+		unsigned aftr_isp : 1 ; /* isp .. imposed sync pause */
+		unsigned peer_isp : 1 ;
+		unsigned user_isp : 1 ;
+		unsigned _pad : 11;   /* 0	 unused */
 #elif defined(__BIG_ENDIAN_BITFIELD)
-		unsigned _pad:11;   /* 0	 unused */
-		unsigned user_isp:1 ;
-		unsigned peer_isp:1 ;
-		unsigned aftr_isp:1 ; /* isp .. imposed sync pause */
-		unsigned susp:1 ;   /* 2/2	 IO suspended  no/yes */
-		unsigned pdsk:4 ;   /* 8/16	 from Diskless to UpToDate */
-		unsigned disk:4 ;   /* 8/16	 from Diskless to UpToDate */
-		unsigned conn:5 ;   /* 17/32	 cstates */
-		unsigned peer:2 ;   /* 3/4	 primary/secondary/unknown */
-		unsigned role:2 ;   /* 3/4	 primary/secondary/unknown */
+		unsigned _pad : 11;   /* 0	 unused */
+		unsigned user_isp : 1 ;
+		unsigned peer_isp : 1 ;
+		unsigned aftr_isp : 1 ; /* isp .. imposed sync pause */
+		unsigned susp : 1 ;   /* 2/2	 IO suspended  no/yes */
+		unsigned pdsk : 4 ;   /* 8/16	 from Diskless to UpToDate */
+		unsigned disk : 4 ;   /* 8/16	 from Diskless to UpToDate */
+		unsigned conn : 5 ;   /* 17/32	 cstates */
+		unsigned peer : 2 ;   /* 3/4	 primary/secondary/unknown */
+		unsigned role : 2 ;   /* 3/4	 primary/secondary/unknown */
 #else
 # error "this endianess is not supported"
 #endif
@@ -275,9 +278,9 @@ enum set_st_err {
 	SS_NotSupported = -17,      /* drbd-8.2 only */
 	SS_InTransientState = -18,  /* Retry after the next state change */
 	SS_ConcurrentStChg = -19,   /* Concurrent cluster side state change! */
-	SS_AfterLastError = -20,    /* Keep this at bottom */
 };
 
+
 /* from drbd_strings.c */
 extern const char *conns_to_name(enum drbd_conns);
 extern const char *roles_to_name(enum drbd_role);
@@ -332,15 +335,9 @@ enum UuidIndex {
 
 /* The following line should be moved over to linux/connector.h
  * when the time comes */
-#ifndef CN_IDX_DRBD
-# define CN_IDX_DRBD			0x4
-/* Ubuntu's Trapid release defined CN_IDX_DRBD as 0x6 */
-#endif
+#define CN_IDX_DRBD			0x4
 #define CN_VAL_DRBD			0x1
 
-/* For searching a vacant cn_idx value */
-#define CN_IDX_STEP			6977
-
 struct drbd_nl_cfg_req {
 	int packet_type;
 	int drbd_minor;
