@@ -65,7 +65,7 @@
 #define DRBD_MAX_EPOCH_SIZE_DEF 2048
 
   /* I don't think that a tcp send buffer of more than 10M is usefull */
-#define DRBD_SNDBUF_SIZE_MIN  0
+#define DRBD_SNDBUF_SIZE_MIN  1
 #define DRBD_SNDBUF_SIZE_MAX  10000000
 #define DRBD_SNDBUF_SIZE_DEF  (2*65535)
 
@@ -107,11 +107,10 @@
 
 /* drbdsetup XY resize -d Z
  * you are free to reduce the device size to nothing, if you want to.
- * the upper limit with 64bit kernel, enough ram and flexible meta data
- * is 8 TB, currently. */
+ * but more than 3998G are currently not possible */
 /* DRBD_MAX_SECTORS */
 #define DRBD_DISK_SIZE_SECT_MIN  0
-#define DRBD_DISK_SIZE_SECT_MAX  (8 * (2LLU << 30))
+#define DRBD_DISK_SIZE_SECT_MAX  ((128LLU*1024*2 - 72)*512LLU*8*8)
 #define DRBD_DISK_SIZE_SECT_DEF  0 /* = disabled = no user size... */
 
 #define DRBD_ON_IO_ERROR_DEF PassOn
@@ -121,5 +120,9 @@
 #define DRBD_AFTER_SB_2P_DEF Disconnect
 #define DRBD_RR_CONFLICT_DEF Disconnect
 
+#define DRBD_MAX_BIO_BVECS_MIN 0
+#define DRBD_MAX_BIO_BVECS_MAX 128
+#define DRBD_MAX_BIO_BVECS_DEF 0
+
 #undef RANGE
 #endif
