@@ -63,6 +63,15 @@ struct ifi_info {
   struct ifi_info *ifi_next;    /* next ifi_info structure */
 };
 
+struct d_proxy_info
+{
+  char* name;
+  char* inside_addr;
+  char* inside_port;
+  char* outside_addr;
+  char* outside_port;
+};
+
 struct d_host_info
 {
   char* name;
@@ -74,6 +83,8 @@ struct d_host_info
   int meta_major;
   int meta_minor;
   char* meta_index;
+  struct d_proxy_info *proxy;
+  struct d_host_info* next;
 };
 
 struct d_option
@@ -89,13 +100,19 @@ struct d_resource
 {
   char* name;
   char* protocol;
+  char* device;     // gets propagated to host_info sections later.
+  char* disk;       // gets propagated to host_info sections later.
+  char* meta_disk;  // gets propagated to host_info sections later.
+  char* meta_index; // gets propagated to host_info sections later.
   struct d_host_info* me;
   struct d_host_info* peer;
+  struct d_host_info* all_hosts;
   struct d_option* net_options;
   struct d_option* disk_options;
   struct d_option* sync_options;
   struct d_option* startup_options;
   struct d_option* handlers;
+  struct d_option* proxy_options;
   struct d_resource* next;
 };
 
