@@ -68,18 +68,10 @@ char* progname;
 struct adm_cmd {
   const char* name;
   int (* function)(struct d_resource*,const char* );
-  /* which level this command is for.
-   * 0: don't show this command, ever
-   * 1: normal administrative commands, shown in normal help
-   * 2-4: shown on "drbdadm hidden-commands"
-   * 2: usefull for shell scripts
-   * 3: callbacks potentially called from kernel module on certain events
-   * 4: advanced, experts and developers only */
   unsigned int show_in_usage     :3;
-  /* if set, command requires an explicit resource name */
   unsigned int res_name_required :1;
-  /* error out if the ip specified is not available/active now */
   unsigned int verify_ips        :1;
+  unsigned int me_is_localhost   :1;
 };
 
 struct deferred_cmd
@@ -106,6 +98,7 @@ static int adm_dump(struct d_resource* ,const char* );
 static int adm_dump_xml(struct d_resource* ,const char* );
 static int adm_wait_c(struct d_resource* ,const char* );
 static int adm_wait_ci(struct d_resource* ,const char* );
+static int adm_proxy_up(struct d_resource* ,const char* );
 static int sh_nop(struct d_resource* ,const char* );
 static int sh_resources(struct d_resource* ,const char* );
 static int sh_resource(struct d_resource* ,const char* );
@@ -120,7 +113,7 @@ static int admm_generic(struct d_resource* ,const char* );
 static int adm_khelper(struct d_resource* ,const char* );
 static int adm_generic_b(struct d_resource* ,const char* );
 static int hidden_cmds(struct d_resource* ,const char* );
-static int adm_outdate(struct d_resource* ,const char* );
+static int adm_outdate(struct d_resource* res,const char* cmd);
 
 static char* get_opt_val(struct d_option*,const char*,char*);
 
@@ -144,6 +137,7 @@ int verbose;
 int do_verify_ips;
 char* drbdsetup;
 char* drbdmeta;
+char* drbd_proxy_ctl;
 char* sh_varname = NULL;
 char* setup_opts[10];
 int soi=0;
@@ -204,112 +198,66 @@ struct option admopt[] = {
   { "config-file",  required_argument,0, 'c' },
   { "drbdsetup",    required_argument,0, 's' },
   { "drbdmeta",     required_argument,0, 'm' },
+  { "drbd-proxy-ctl",required_argument,0,'p' },
   { "sh-varname",   required_argument,0, 'n' },
   { 0,              0,                0, 0   }
 };
 
-/* DRBD adm_cmd flags shortcuts,
- * to avoid merge conflicts and unreadable diffs
- * when we add the next flag */
-
-#define DRBD_acf1_default		\
-	.show_in_usage = 1,		\
-	.res_name_required = 1,		\
-	.verify_ips = 0,
-
-#define DRBD_acf1_defnet		\
-	.show_in_usage = 1,		\
-	.res_name_required = 1,		\
-	.verify_ips = 1,
-
-#define DRBD_acf3_handler		\
-	.show_in_usage = 3,		\
-	.res_name_required = 1,		\
-	.verify_ips = 0,
-
-#define DRBD_acf4_advanced		\
-	.show_in_usage = 4,		\
-	.res_name_required = 1,		\
-	.verify_ips = 0,
-
-#define DRBD_acf1_dump			\
-	.show_in_usage = 1,		\
-	.res_name_required = 1,		\
-	.verify_ips = 1,
-
-#define DRBD_acf2_shell			\
-	.show_in_usage = 2,		\
-	.res_name_required = 1,		\
-	.verify_ips = 0,
-
-#define DRBD_acf2_gen_shell		\
-	.show_in_usage = 2,		\
-	.res_name_required = 0,		\
-	.verify_ips = 0,
-
 struct adm_cmd cmds[] = {
-        /*  name, function, flags
-	 *  sort order:
-	 *  - normal config commands,
-	 *  - normal meta data manipulation
-	 *  - sh-*
-	 *  - handler
-	 *  - advanced
-	 ***/
-        { "attach",                adm_attach,      DRBD_acf1_default   },
-        { "detach",                adm_generic_l,   DRBD_acf1_default   },
-        { "connect",               adm_connect,     DRBD_acf1_defnet    },
-        { "disconnect",            adm_generic_s,   DRBD_acf1_default   },
-        { "up",                    adm_up,          DRBD_acf1_defnet    },
-        { "down",                  adm_generic_l,   DRBD_acf1_default   },
-        { "primary",               adm_generic_l,   DRBD_acf1_default   },
-        { "secondary",             adm_generic_l,   DRBD_acf1_default   },
-        { "invalidate",            adm_generic_b,   DRBD_acf1_default   },
-        { "invalidate-remote",     adm_generic_l,   DRBD_acf1_defnet    },
-        { "outdate",               adm_outdate,     DRBD_acf1_default   },
-        { "resize",                adm_resize,      DRBD_acf1_defnet    },
-        { "syncer",                adm_syncer,      DRBD_acf1_defnet    },
-        { "pause-sync",            adm_generic_s,   DRBD_acf1_defnet    },
-        { "resume-sync",           adm_generic_s,   DRBD_acf1_defnet    },
-        { "adjust",                adm_adjust,      DRBD_acf1_defnet    },
-        { "wait-connect",          adm_wait_c,      DRBD_acf1_defnet    },
-        { "wait-con-int",          adm_wait_ci,
-		.show_in_usage = 1, .verify_ips = 1, },
-        { "state",                 adm_generic_s,   DRBD_acf1_default   },
-        { "cstate",                adm_generic_s,   DRBD_acf1_default   },
-        { "dstate",                adm_generic_b,   DRBD_acf1_default   },
-
-        { "dump",                  adm_dump,        DRBD_acf1_dump      },
-        { "dump-xml",              adm_dump_xml,    DRBD_acf1_dump      },
-
-        { "create-md",             adm_create_md,   DRBD_acf1_default   },
-        { "show-gi",               adm_generic_b,   DRBD_acf1_default   },
-        { "get-gi",                adm_generic_b,   DRBD_acf1_default   },
-        { "dump-md",               admm_generic,    DRBD_acf1_default   },
-        { "wipe-md",               admm_generic,    DRBD_acf1_default   },
-        { "hidden-commands",       hidden_cmds,     .show_in_usage = 1, },
-
-        { "sh-nop",                sh_nop,          DRBD_acf2_gen_shell },
-        { "sh-resources",          sh_resources,    DRBD_acf2_gen_shell },
-        { "sh-resource",           sh_resource,     DRBD_acf2_shell     },
-        { "sh-mod-parms",          sh_mod_parms,    DRBD_acf2_gen_shell },
-        { "sh-dev",                sh_dev,          DRBD_acf2_shell     },
-        { "sh-ll-dev",             sh_ll_dev,       DRBD_acf2_shell     },
-        { "sh-md-dev",             sh_md_dev,       DRBD_acf2_shell     },
-        { "sh-md-idx",             sh_md_idx,       DRBD_acf2_shell     },
-        { "sh-ip",                 sh_ip,           DRBD_acf2_shell     },
-        { "sh-b-pri",              sh_b_pri,        DRBD_acf2_shell     },
-
-        { "pri-on-incon-degr",     adm_khelper,     DRBD_acf3_handler   },
-        { "pri-lost-after-sb",     adm_khelper,     DRBD_acf3_handler   },
-        { "outdate-peer",          adm_khelper,     DRBD_acf3_handler   },
-        { "local-io-error",        adm_khelper,     DRBD_acf3_handler   },
-        { "pri-lost",              adm_khelper,     DRBD_acf3_handler   },
-        { "split-brain",           adm_khelper,     DRBD_acf3_handler   },
-
-        { "suspend-io",            adm_generic_s,   DRBD_acf4_advanced  },
-        { "resume-io",             adm_generic_s,   DRBD_acf4_advanced  },
-        { "set-gi",                admm_generic,    DRBD_acf4_advanced  },
+/*   name, function,                  show, needs res, verify_ips, me_localhost */
+  { "attach",            adm_attach,    1,1,0,1 },
+  { "detach",            adm_generic_s, 1,1,0,1 },
+  { "connect",           adm_connect,   1,1,1,1 },
+  { "disconnect",        adm_generic_s, 1,1,0,1 },
+  { "up",                adm_up,        1,1,1,1 },
+  { "down",              adm_generic_s, 1,1,0,1 },
+  { "primary",           adm_generic_l, 1,1,0,1 },
+  { "secondary",         adm_generic_l, 1,1,0,1 },
+  { "invalidate",        adm_generic_b, 1,1,0,1 },
+  { "invalidate-remote", adm_generic_l, 1,1,1,1 },
+  { "outdate",           adm_outdate,   1,1,0,1 },
+  { "resize",            adm_resize,    1,1,1,1 },
+  { "syncer",            adm_syncer,    1,1,1,1 },
+  { "pause-sync",        adm_generic_s, 1,1,1,1 },
+  { "resume-sync",       adm_generic_s, 1,1,1,1 },
+  { "adjust",            adm_adjust,    1,1,1,1 },
+  { "wait-connect",      adm_wait_c,    1,1,1,1 },
+  { "state",             adm_generic_s, 1,1,0,1 },
+  { "cstate",            adm_generic_s, 1,1,0,1 },
+  { "dstate",            adm_generic_b, 1,1,0,1 },
+  { "dump",              adm_dump,      1,1,1,0 },
+  { "dump-xml",          adm_dump_xml,  1,1,1,0 },
+  { "create-md",         adm_create_md, 1,1,0,1 },
+  { "show-gi",           adm_generic_b, 1,1,0,1 },
+  { "get-gi",            adm_generic_b, 1,1,0,1 },
+  { "dump-md",           admm_generic,  1,1,0,1 },
+  { "wipe-md",           admm_generic,  1,1,0,1 },
+  { "wait-con-int",      adm_wait_ci,   1,0,1,1 },
+  { "hidden-commands",   hidden_cmds,   1,0,0,0 },
+  { "verify",            adm_generic_s, 1,1,0,0 },
+  { "proxy-up",          adm_proxy_up,  2,1,0,0 },
+  { "sh-nop",            sh_nop,        2,0,0,0 },
+  { "sh-resources",      sh_resources,  2,0,0,0 },
+  { "sh-resource",       sh_resource,   2,1,0,0 },
+  { "sh-mod-parms",      sh_mod_parms,  2,0,0,0 },
+  { "sh-dev",            sh_dev,        2,1,0,0 },
+  { "sh-ll-dev",         sh_ll_dev,     2,1,0,0 },
+  { "sh-md-dev",         sh_md_dev,     2,1,0,0 },
+  { "sh-md-idx",         sh_md_idx,     2,1,0,0 },
+  { "sh-ip",             sh_ip,         0,1,0,0 },
+  { "sh-b-pri",          sh_b_pri,      2,1,0,0 },
+  { "pri-on-incon-degr", adm_khelper,   3,1,0,1 },
+  { "pri-lost-after-sb", adm_khelper,   3,1,0,1 },
+  { "outdate-peer",      adm_khelper,   3,1,0,1 },
+  { "local-io-error",    adm_khelper,   3,1,0,1 },
+  { "pri-lost",          adm_khelper,   3,1,0,1 },
+  { "split-brain",       adm_khelper,   3,1,0,1 },
+  { "out-of-sync",       adm_khelper,   3,1,0,1 },
+  { "before-resync-target",adm_khelper,   3,1,0,1 },
+  { "after-resync-target",adm_khelper,   3,1,0,1 },
+  { "set-gi",            admm_generic,  4,1,0,1 },
+  { "suspend-io",        adm_generic_s, 4,1,0,1 },
+  { "resume-io",         adm_generic_s, 4,1,0,1 },
 };
 
 /*** These functions are used to the print the config ***/
@@ -352,32 +300,32 @@ static char* esc_xml(char* str)
     while(*ue) {
       if (*ue == '"' || *ue == '\\') {
 	  *e++ = '\\';
-          if (e-buffer >= 1021) {
+	  if (e-buffer >= 1021) {
 	     fprintf(stderr,"string too long.\n");
 	     exit(E_syntax);
 	  }
-          *e++ = *ue++;
+	  *e++ = *ue++;
       } else if (*ue == '\'' || *ue == '<' || *ue == '>' || *ue == '&') {
-          if (*ue == '\'' && e-buffer < 1017) {
-            strcpy(e, "&apos;");
-            e += 6;
-          } else if (*ue == '<' && e-buffer < 1019) {
-            strcpy(e, "&lt;");
-            e += 4;
-          } else if (*ue == '>' && e-buffer < 1019) {
-            strcpy(e, "&gt;");
-            e += 4;
-          } else if (*ue == '&' && e-buffer < 1018) {
-            strcpy(e, "&amp;");
-            e += 5;
-          } else {
-            fprintf(stderr,"string too long.\n");
+	  if (*ue == '\'' && e-buffer < 1017) {
+	    strcpy(e, "&apos;");
+	    e += 6;
+	  } else if (*ue == '<' && e-buffer < 1019) {
+	    strcpy(e, "&lt;");
+	    e += 4;
+	  } else if (*ue == '>' && e-buffer < 1019) {
+	    strcpy(e, "&gt;");
+	    e += 4;
+	  } else if (*ue == '&' && e-buffer < 1018) {
+	    strcpy(e, "&amp;");
+	    e += 5;
+	  } else {
+	    fprintf(stderr,"string too long.\n");
 	    exit(E_syntax);
 	  }
 	  ue++;
       } else {
-          *e++ = *ue++;
-          if (e-buffer >= 1022) {
+	  *e++ = *ue++;
+	  if (e-buffer >= 1022) {
 	    fprintf(stderr,"string too long.\n");
 	    exit(E_syntax);
 	  }
@@ -427,10 +375,19 @@ static void dump_common_info()
   dump_options("disk",common->disk_options);
   dump_options("syncer",common->sync_options);
   dump_options("startup",common->startup_options);
+  dump_options("proxy",common->proxy_options);
   dump_options("handlers",common->handlers);
   --indent; printf("}\n\n");
 }
 
+static void dump_proxy_info(struct d_proxy_info* pi)
+{
+  printI("proxy on %s {\n",esc(pi->name)); ++indent;
+  printI(IPFMT,"inside", pi->inside_addr, pi->inside_port);
+  printI(IPFMT,"outside", pi->outside_addr, pi->outside_port);
+  --indent; printI("}\n");
+}
+
 static void dump_host_info(struct d_host_info* hi)
 {
   if(!hi) {
@@ -448,6 +405,7 @@ static void dump_host_info(struct d_host_info* hi)
     printA("meta-disk", "internal");
   else
     printI(MDISK,"meta-disk", esc(hi->meta_disk), hi->meta_index);
+  if(hi->proxy) dump_proxy_info(hi->proxy);
   --indent; printI("}\n");
 }
 
@@ -490,10 +448,19 @@ static void dump_common_info_xml()
   dump_options_xml("disk",common->disk_options);
   dump_options_xml("syncer",common->sync_options);
   dump_options_xml("startup",common->startup_options);
+  dump_options_xml("proxy",common->proxy_options);
   dump_options_xml("handlers",common->handlers);
   --indent; printI("</common>\n");
 }
 
+static void dump_proxy_info_xml(struct d_proxy_info* pi)
+{
+  printI("<proxy hostname=\"%s\">\n",esc_xml(pi->name)); ++indent;
+  printI("<inside port=\"%s\">%s</inside>\n", pi->inside_port, pi->inside_addr);
+  printI("<outside port=\"%s\">%s</outside>\n", pi->outside_port, pi->outside_addr);
+  --indent; printI("</proxy>\n");
+}
+
 static void dump_host_info_xml(struct d_host_info* hi)
 {
   if(!hi) {
@@ -512,6 +479,7 @@ static void dump_host_info_xml(struct d_host_info* hi)
   else {
     printI("<meta-disk index=\"%s\">%s</meta-disk>\n", hi->meta_index, esc_xml(hi->meta_disk));
   }
+  if(hi->proxy) dump_proxy_info_xml(hi->proxy);
   --indent; printI("</host>\n");
 }
 
@@ -526,6 +494,7 @@ static int adm_dump(struct d_resource* res,const char* unused __attribute((unuse
   dump_options("disk",res->disk_options);
   dump_options("syncer",res->sync_options);
   dump_options("startup",res->startup_options);
+  dump_options("proxy",res->proxy_options);
   dump_options("handlers",res->handlers);
   --indent; printf("}\n\n");
 
@@ -544,6 +513,7 @@ static int adm_dump_xml(struct d_resource* res,const char* unused __attribute((u
   dump_options_xml("disk",res->disk_options);
   dump_options_xml("syncer",res->sync_options);
   dump_options_xml("startup",res->startup_options);
+  dump_options_xml("proxy",res->proxy_options);
   dump_options_xml("handlers",res->handlers);
   --indent; printI("</resource>\n");
 
@@ -620,19 +590,11 @@ static int sh_md_idx(struct d_resource* res,const char* unused __attribute((unus
 static int sh_b_pri(struct d_resource *res,const char* unused __attribute((unused)))
 {
   char* val;
-  int i, rv;
 
   val = get_opt_val(res->startup_options, "become-primary-on", NULL);
   if ( val && ( !strcmp(val,nodeinfo.nodename) ||
 		!strcmp(val,"both") ) ) {
-    /* Opon connect resync starts, and both sides become primary at the same time.
-       One's try might be declined since an other state transition happens. Retry. */
-    for (i=0; i<5; i++) {
-      rv = adm_generic_s(res,"primary");
-      if (rv == 0) return rv;
-      sleep(1);
-    }
-    return rv;
+    return adm_generic_s(res,"primary");
   }
   return 0;
 }
@@ -678,12 +640,17 @@ static void free_config(struct d_resource* res)
   for_each_resource(f,t,res) {
     free(f->name);
     free(f->protocol);
+    free(f->device);
+    free(f->disk);
+    free(f->meta_disk);
+    free(f->meta_index);
     free_host_info(f->me);
     free_host_info(f->peer);
     free_options(f->net_options);
     free_options(f->disk_options);
     free_options(f->sync_options);
     free_options(f->startup_options);
+    free_options(f->proxy_options);
     free_options(f->handlers);
     free(f);
   }
@@ -692,6 +659,7 @@ static void free_config(struct d_resource* res)
     free_options(common->disk_options);
     free_options(common->sync_options);
     free_options(common->startup_options);
+    free_options(common->proxy_options);
     free_options(common->handlers);
     free(common);
   }
@@ -724,6 +692,7 @@ static void expand_common(void)
     expand_opts(common->disk_options,    &res->disk_options);
     expand_opts(common->sync_options,    &res->sync_options);
     expand_opts(common->startup_options, &res->startup_options);
+    expand_opts(common->proxy_options,   &res->proxy_options);
     expand_opts(common->handlers,        &res->handlers);
     if(common->protocol && ! res->protocol) {
       res->protocol = strdup(common->protocol);
@@ -1109,7 +1078,11 @@ int adm_connect(struct d_resource* res,const char* unused __attribute((unused)))
   argv[NA(argc)]=res->me->device;
   argv[NA(argc)]="net";
   ssprintf(argv[NA(argc)],"%s:%s",res->me->address,res->me->port);
-  ssprintf(argv[NA(argc)],"%s:%s",res->peer->address,res->peer->port);
+  if(res->me->proxy) {
+    ssprintf(argv[NA(argc)],"%s:%s",res->me->proxy->inside_addr,res->me->proxy->inside_port);
+  } else {
+    ssprintf(argv[NA(argc)],"%s:%s",res->peer->address,res->peer->port);
+  }
   argv[NA(argc)]=res->protocol;
 
   argv[NA(argc)]="--set-defaults";
@@ -1143,6 +1116,53 @@ void convert_after_option(struct d_resource* res)
   }
 }
 
+static int adm_proxy_up(struct d_resource* res, const char* unused __attribute((unused)))
+{
+  char* argv[MAX_ARGS];
+  int argc=0, rv;
+  struct d_option* opt;
+
+  if(!res->me->proxy) {
+    fprintf(stderr,"There is no proxy config for host %s in resource %s.\n",
+	    nodeinfo.nodename, res->name);
+    exit(E_config_invalid);
+  }
+
+  if(strcmp(res->me->proxy->name,nodeinfo.nodename)) {
+    fprintf(stderr,"The proxy config in resource %s is for %s, this is %s.\n",
+	    res->name, res->me->proxy->name,nodeinfo.nodename);
+    exit(E_config_invalid);
+  }
+
+  argv[NA(argc)]=drbd_proxy_ctl;
+  argv[NA(argc)]="-c";
+  ssprintf(argv[NA(argc)],
+	   "add connection %s-%s-%s %s:%s %s:%s %s:%s %s:%s",
+	   res->me->name, res->name, res->peer->name,
+	   res->me->proxy->inside_addr, res->me->proxy->inside_port,
+	   res->peer->proxy->outside_addr, res->peer->proxy->outside_port,
+	   res->me->proxy->outside_addr, res->me->proxy->outside_port,
+	   res->me->address, res->me->port);
+  argv[NA(argc)]=0;
+
+  rv = m_system(argv, SLEEPS_SHORT, res);
+  if(rv != 0) return rv;
+
+  argc=0;
+  argv[NA(argc)]=drbd_proxy_ctl;
+  opt = res->proxy_options;
+  while(opt) {
+    argv[NA(argc)]="-c";
+    ssprintf(argv[NA(argc)], "set %s %s-%s-%s %s",
+	     opt->name, res->me->name, res->name, res->peer->name, opt->value);
+    opt=opt->next; 
+  }
+  argv[NA(argc)]=0;
+  if(argc > 2) return m_system(argv, SLEEPS_SHORT, res);  
+  return rv;
+}
+
+
 int adm_syncer(struct d_resource* res,const char* unused __attribute((unused)))
 {
   char* argv[MAX_ARGS];
@@ -1605,6 +1625,7 @@ void verify_ips(struct d_resource *res)
 }
 
 static char* conf_file[] = {
+    "/etc/drbd-82.conf",
     "/etc/drbd-08.conf",
     "/etc/drbd.conf",
     0
@@ -1817,8 +1838,9 @@ void validate_resource(struct d_resource * res)
       }
     }
   }
-  /* IP verification (check for existence)
-   * moved to just before command execution */
+  if (res->me && res->peer) {
+    verify_ips(res);
+  }
 
   if ( (opt = find_opt(res->handlers, "outdate-peer")) ) {
     if(strstr(opt->value,"drbd-peer-outdater")) sanity_check_perm();
@@ -1833,6 +1855,13 @@ void validate_resource(struct d_resource * res)
 	    "is not set.\n", res->name);
     config_valid = 0;
   }
+
+  if (( res->me->proxy == NULL ) != (res->peer->proxy == NULL)) {
+	fprintf(stderr,
+		"Incomplete proxy configuration. in resource %s.\n",
+		res->name);
+	config_valid = 0;
+  }
 }
 
 static void global_validate(void)
@@ -1867,8 +1896,8 @@ int main(int argc, char** argv)
     strncpy(nodeinfo.nodename,env_drbd_nodename,sizeof(nodeinfo.nodename)-1);
     nodeinfo.nodename[sizeof(nodeinfo.nodename)-1] = 0;
     fprintf(stderr, "\n"
-            "   found __DRBD_NODE__ in environment\n"
-            "   PRETENDING that I am >>%s<<\n\n",nodeinfo.nodename);
+	    "   found __DRBD_NODE__ in environment\n"
+	    "   PRETENDING that I am >>%s<<\n\n",nodeinfo.nodename);
   }
 
   /* in case drbdadm is called with an absolut or relative pathname
@@ -1878,6 +1907,7 @@ int main(int argc, char** argv)
     progname=argv[0];
     drbdsetup = strdup("drbdsetup");
     drbdmeta = strdup("drbdmeta");
+    drbd_proxy_ctl = strdup("drbd-proxy-ctl");
   } else {
     size_t len = strlen(argv[0]) + 1;
     ++progname;
@@ -1896,12 +1926,19 @@ int main(int argc, char** argv)
       strcpy(drbdmeta + (progname - argv[0]), "drbdmeta");
     }
 
+    len += strlen("drbd-proxy-ctl") - strlen(progname);
+    drbd_proxy_ctl = malloc(len);
+    if (drbd_proxy_ctl) {
+      strncpy(drbd_proxy_ctl, argv[0], (progname - argv[0]));
+      strcpy(drbd_proxy_ctl + (progname - argv[0]), "drbd-proxy-ctl");
+    }
+
     argv[0] = progname;
   }
 
   if(argc == 1) print_usage_and_exit("missing arguments"); // arguments missing.
 
-  if (drbdsetup == NULL || drbdmeta == NULL) {
+  if (drbdsetup == NULL || drbdmeta == NULL || drbd_proxy_ctl == NULL) {
     fprintf(stderr,"could not strdup argv[0].\n");
     exit(E_exec_error);
   }
@@ -1951,6 +1988,14 @@ int main(int argc, char** argv)
 	    find_drbdcmd(&drbdmeta,pathes);
 	  }
 	  break;
+	case 'p':
+	  {
+	    char* pathes[2];
+	    pathes[0]=optarg;
+	    pathes[1]=0;
+	    find_drbdcmd(&drbd_proxy_ctl,pathes);
+	  }
+	  break;
 	case 'n':
 	  {
 	    char *c;
@@ -2093,48 +2138,46 @@ int main(int argc, char** argv)
   if(cmd->res_name_required)
     {
       if (optind + 1 > argc && !is_dump)
-        print_usage_and_exit("missing arguments"); // arguments missing.
+	print_usage_and_exit("missing arguments"); // arguments missing.
 
       global_validate();
       if (!is_dump) {
-	if(!config_valid) exit(E_config_invalid);
-
-        for_each_resource(res,tmp,config) {
+	for_each_resource(res,tmp,config) {
 	  convert_after_option(res);
 	  convert_discard_opt(res);
+
+	  if(cmd->me_is_localhost) {
+	    if(strcmp(res->me->name, nodeinfo.nodename)) {
+	      fprintf(stderr,"In resource %s there is no host section for this"
+		      " host %s\n.", res->name, nodeinfo.nodename);
+	      config_valid=0;
+	    }
+	  }
 	}
+	if(!config_valid) exit(E_config_invalid);
       }
 
-      /* either no resorce arguments at all,
-       * but command is dump / dump-xml, so implitict "all",
-       * or an explicit "all" argument is given */
       if ( optind==argc || !strcmp(argv[optind],"all") ) {
-	/* verify ips first, for all of them */
-        for_each_resource(res,tmp,config) {
-	  verify_ips(res);
+	if (is_dump) {
+	  if (is_dump_xml) {
+	    printf("<config file=\"%s\">\n", config_file); ++indent;
+	    dump_global_info_xml();
+	    dump_common_info_xml();
+	  } else {
+	    printf("# %s\n",config_file);
+	    dump_global_info();
+	    dump_common_info();
+	  }
 	}
-	if (!is_dump && !config_valid)
-	  exit(E_config_invalid);
-        if (is_dump) {
-          if (is_dump_xml) {
-            printf("<config file=\"%s\">\n", config_file); ++indent;
-            dump_global_info_xml();
-            dump_common_info_xml();
-          } else {
-            printf("# %s\n",config_file);
-            dump_global_info();
-            dump_common_info();
-          }
-	}
-        for_each_resource(res,tmp,config) {
+	for_each_resource(res,tmp,config) {
 	  if( (rv |= cmd->function(res,cmd->name)) >= 10 ) {
-	    fprintf(stderr,"drbdsetup exited with code %d\n",rv);
+	    fprintf(stderr,"command exited with code %d\n",rv);
 	    exit(E_exec_error);
 	  }
 	}
-        if (is_dump_xml) {
-            --indent; printf("</config>\n");
-        }
+	if (is_dump_xml) {
+	    --indent; printf("</config>\n");
+	}
       } else {
 	for(i=optind;(int)i<argc;i++) {
 	  res = res_by_name(argv[i]);
@@ -2143,9 +2186,6 @@ int main(int argc, char** argv)
 	    fprintf(stderr,"'%s' not defined in your config.\n",argv[i]);
 	    exit(E_usage);
 	  }
-	  verify_ips(res);
-	  if (!is_dump && !config_valid)
-            exit(E_config_invalid);
 	  if( (rv=cmd->function(res,cmd->name)) >= 20 ) {
 	    fprintf(stderr,"drbdadm aborting\n");
 	    exit(rv);
@@ -2154,7 +2194,7 @@ int main(int argc, char** argv)
       }
     } else { // Commands which do not need a resource name
       if( (rv=cmd->function(config,cmd->name)) >= 10) {
-	fprintf(stderr,"drbdsetup exited with code %d\n",rv);
+	fprintf(stderr,"command exited with code %d\n",rv);
 	exit(E_exec_error);
       }
     }
