@@ -92,9 +92,9 @@ void range_check(const enum range_checks what, const char *name,
 				DRBD_DIALOG_REFRESH_MAX);
 		break;
 	case R_DISK_SIZE:
-		m_strtoll_range(value, 's', name,
-				DRBD_DISK_SIZE_SECT_MIN,
-				DRBD_DISK_SIZE_SECT_MAX);
+		m_strtoll_range(value, 'K', name,
+				DRBD_DISK_SIZE_SECT_MIN >> 1,
+				DRBD_DISK_SIZE_SECT_MAX >> 1);
 		break;
 	case R_TIMEOUT:
 		m_strtoll_range(value, 1, name, DRBD_TIMEOUT_MIN,
@@ -196,74 +196,44 @@ void check_meta_disk(struct d_host_info *host)
 	}
 }
 
+#define EXP(TOKEN1)						\
+({ 								\
+	int token;						\
+	token = yylex();					\
+	if(token != TOKEN1)					\
+		pe_expected_got( #TOKEN1, token);		\
+	token;							\
+})
+
+#define EXP2(TOKEN1,TOKEN2)						\
+({ 									\
+	int token;							\
+	token = yylex();						\
+	if(token != TOKEN1 && token != TOKEN2)				\
+		pe_expected_got( #TOKEN1 "|" # TOKEN2, token);		\
+	token;								\
+})
+
 static void pe_expected(const char *exp)
 {
-	const char *s = yytext;
 	fprintf(stderr, "%s:%u: Parse error: '%s' expected,\n\t"
-		"but got '%.20s%s'\n", config_file, line, exp, s,
-		strlen(s) > 20 ? "..." : "");
-	exit(E_config_invalid);
-}
-
-static void check_string_error(int got)
-{
-	const char *msg;
-	switch(got) {
-	case TK_ERR_STRING_TOO_LONG:
-		msg = "Token too long";
-		break;
-	case TK_ERR_DQSTRING_TOO_LONG:
-		msg = "Double quoted string too long";
-		break;
-	case TK_ERR_DQSTRING:
-		msg = "Unterminated double quoted string\n  we don't allow embeded newlines\n ";
-		break;
-	default:
-		return;
-	}
-	fprintf(stderr,"%s:%u: %s >>>%.20s...<<<\n", config_file, line, msg, yytext);
+		"but got '%s'\n", config_file, line, exp, yytext);
 	exit(E_config_invalid);
 }
 
 static void pe_expected_got(const char *exp, int got)
 {
 	static char tmp[2] = "\0";
-	const char *s = yytext;
 	if (exp[0] == '\'' && exp[1] && exp[2] == '\'' && exp[3] == 0) {
 		tmp[0] = exp[1];
 	}
 	fprintf(stderr, "%s:%u: Parse error: '%s' expected,\n\t"
-		"but got '%.20s%s' (TK %d)\n",
+		"but got '%s' (TK %d)\n",
 		config_file, line,
-		tmp[0] ? tmp : exp, s, strlen(s) > 20 ? "..." : "", got);
+		tmp[0] ? tmp : exp, yytext, got);
 	exit(E_config_invalid);
 }
 
-#define EXP(TOKEN1)						\
-({								\
-	int token;						\
-	token = yylex();					\
-	if (token != TOKEN1) {					\
-		if (TOKEN1 == TK_STRING)			\
-			check_string_error(token);		\
-		pe_expected_got( #TOKEN1, token);		\
-	}							\
-	token;							\
-})
-
-static void expect_STRING_or_INT(void)
-{
-	int token = yylex();
-	switch(token) {
-	case TK_INTEGER:
-	case TK_STRING:
-		return;
-	default:
-		check_string_error(token);
-		pe_expected_got("TK_STRING | TK_INTEGER", token);
-	}
-}
-
 static void parse_global(void)
 {
 	fline = line;
@@ -325,7 +295,6 @@ static struct d_option *parse_options(int token_switch, int token_option)
 	struct d_option *options = NULL, *ro = NULL;
 	fline = line;
 
-	EXP('{');
 	while (1) {
 		token = yylex();
 		if (token == token_switch) {
@@ -333,7 +302,7 @@ static struct d_option *parse_options(int token_switch, int token_option)
 		} else if (token == token_option) {
 			opt_name = yylval.txt;
 			rc = yylval.rc;
-			expect_STRING_or_INT();
+			EXP2(TK_STRING, TK_INTEGER);
 			range_check(rc, opt_name, yylval.txt);
 			ro = new_opt(opt_name, yylval.txt);
 			options = APPEND(options, ro);
@@ -355,9 +324,76 @@ static struct d_option *parse_options(int token_switch, int token_option)
 	}
 }
 
-static void parse_host_body(struct d_host_info *host,
-			    struct d_resource *res, int require_all)
+static void parse_proxy_section(struct d_host_info *host)
 {
+	struct d_proxy_info *proxy;
+
+	proxy=calloc(1,sizeof(struct d_proxy_info));
+	host->proxy = proxy;
+
+	EXP(TK_ON);
+	EXP(TK_STRING);
+	proxy->name = yylval.txt;
+
+	EXP('{');
+	while (1) {
+		switch (yylex()) {
+		case TK_INSIDE:
+			EXP(TK_IPADDR);
+			proxy->inside_addr = yylval.txt;
+			EXP(':');
+			EXP(TK_INTEGER);
+			proxy->inside_port = yylval.txt;
+			EXP(';');
+			break;
+		case TK_OUTSIDE:
+			EXP(TK_IPADDR);
+			proxy->outside_addr = yylval.txt;
+			EXP(':');
+			EXP(TK_INTEGER);
+			proxy->outside_port = yylval.txt;
+			EXP(';');
+			break;
+		case '}':
+			goto break_loop;
+		default:
+			pe_expected("inside | outside");
+
+		}
+	}
+	
+ break_loop:
+	return;
+}
+
+static void parse_meta_disk(char **disk, char** index)
+{
+	EXP(TK_STRING);
+	*disk = yylval.txt;
+	if (strcmp("internal", yylval.txt)) {
+		EXP('[');
+		EXP(TK_INTEGER);
+		*index = yylval.txt;
+		EXP(']');
+		EXP(';');
+	} else {
+		EXP(';');
+	}
+}
+
+static void parse_host_section(struct d_resource *res, 
+			       char *host_name, int require_all)
+{
+	struct d_host_info *host;
+
+	c_section_start = line;
+	fline = line;
+
+	host=calloc(1,sizeof(struct d_host_info));
+	host->name = host_name;
+	check_uniq("host section", "%s: on %s", res->name, host->name);
+	res->all_hosts = APPEND(res->all_hosts, host);
+
 	EXP('{');
 	while (1) {
 		switch (yylex()) {
@@ -399,17 +435,7 @@ static void parse_host_body(struct d_host_info *host,
 		case TK_META_DISK:
 			check_uniq("meta-disk statement", "%s:%s:meta-disk",
 				   res->name, host->name);
-			EXP(TK_STRING);
-			host->meta_disk = yylval.txt;
-			if (strcmp("internal", yylval.txt)) {
-				EXP('[');
-				EXP(TK_INTEGER);
-				host->meta_index = yylval.txt;
-				EXP(']');
-				EXP(';');
-			} else {
-				EXP(';');
-			}
+			parse_meta_disk(&host->meta_disk, &host->meta_index);
 			check_meta_disk(host);
 			break;
 		case TK_FLEX_META_DISK:
@@ -433,6 +459,9 @@ static void parse_host_body(struct d_host_info *host,
 				break;
 			}
 			break;
+		case TK_PROXY:
+			parse_proxy_section(host);
+			break;
 		case '}':
 			goto break_loop;
 		default:
@@ -441,6 +470,26 @@ static void parse_host_body(struct d_host_info *host,
 		}
 	}
       break_loop:
+
+	/* Inerit device, disk, meta_disk and meta_index from the resource. */
+	if(!host->disk && res->disk) {
+		host->disk = strdup(res->disk);
+		check_uniq("disk", "%s:%s:%s", "disk",
+			   host->name, host->disk);
+	}
+
+	if(!host->device && res->device) {
+		host->device = strdup(res->device);
+		check_uniq("device", "%s:%s:%s", "device",
+			   host->name, host->device);
+	}
+
+	if(!host->meta_disk && res->meta_disk) {
+		host->meta_disk = strdup(res->meta_disk);
+		if(res->meta_index) host->meta_index = strdup(res->meta_index);
+		check_meta_disk(host);
+	}
+
 	if (!require_all)
 		return;
 	if (!host->device)
@@ -456,18 +505,15 @@ static void parse_host_body(struct d_host_info *host,
 void parse_skip()
 {
 	int level;
-	int token;
 	fline = line;
 
-	token = yylex();
-	switch (token) {
+	switch (yylex()) {
 	case TK_STRING:
 		EXP('{');
 		break;
 	case '{':
 		break;
 	default:
-		check_string_error(token);
 		pe_expected("[ some_text ] {");
 	}
 
@@ -492,59 +538,10 @@ void parse_skip()
 	while (level) ;
 }
 
-
-static void parse_host_section(struct d_resource* res)
-{
-	struct d_host_info *host;
-
-	c_section_start = line;
-	fline = line;
-
-	host=calloc(1,sizeof(struct d_host_info));
-	EXP(TK_STRING);
-	host->name = yylval.txt;
-
-	check_uniq("host section", "%s: on %s", res->name, host->name);
-	if(strcmp(host->name, nodeinfo.nodename) == 0) {
-		// if (res->me) die duplicate entry ... done by check_uniq above
-		res->me = host;
-	} else {
-		if (res->peer) {
-			config_valid = 0;
-			fprintf(stderr,
-		"%s:%d: in resource %s, on %s { ... } ... on %s { ... }:\n"
-		"\tThere are multiple host sections for the peer.\n"
-		"\tMaybe misspelled local host name '%s'?\n",
-				config_file, c_section_start, res->name,
-				res->peer->name, host->name, nodeinfo.nodename);
-		}
-		res->peer = host;
-	}
-	parse_host_body(host,res,1);
-}
-
-static void parse_drbdsetup_host_dump(struct d_resource* res, int local)
-{
-	struct d_host_info *host;
-
-	c_section_start = line;
-
-	host=calloc(1,sizeof(struct d_host_info));
-
-	if(local) {
-		res->me = host;
-		host->name = strdup("_this_host");
-	} else {
-		res->peer = host;
-		host->name = strdup("_remote_host");
-	}
-
-	parse_host_body(host,res,0);
-}
-
 struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 {
 	struct d_resource* res;
+	struct d_host_info *host;
 	int token;
 
 	fline = line;
@@ -562,38 +559,75 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 			EXP(';');
 			break;
 		case TK_ON:
-			parse_host_section(res);
+			EXP(TK_STRING);
+			parse_host_section(res, yylval.txt, 1);
 			break;
 		case TK__THIS_HOST:
-			parse_drbdsetup_host_dump(res, 1);
+			parse_host_section(res, strdup("_this_host"), 0);
 			break;
 		case TK__REMOTE_HOST:
-			parse_drbdsetup_host_dump(res, 0);
+			parse_host_section(res, strdup("_remote_host"), 0);
 			break;
 		case TK_DISK:
-			check_uniq("disk section", "%s:disk", res->name);
-			res->disk_options = parse_options(TK_DISK_SWITCH,
-							  TK_DISK_OPTION);
+			switch (token=yylex()) {
+			case TK_STRING:
+				res->disk = yylval.txt;
+				EXP(';');
+				break;
+			case '{':
+				check_uniq("disk section", "%s:disk", res->name);
+				res->disk_options = parse_options(TK_DISK_SWITCH,
+								  TK_DISK_OPTION);
+				break;
+			default: 
+				pe_expected_got( "TK_STRING | {", token);
+			}
 			break;
 		case TK_NET:
 			check_uniq("net section", "%s:net", res->name);
+			EXP('{');
 			res->net_options = parse_options(TK_NET_SWITCH,
 							 TK_NET_OPTION);
 			break;
 		case TK_SYNCER:
 			check_uniq("syncer section", "%s:syncer", res->name);
+			EXP('{');
 			res->sync_options = parse_options(TK_SYNCER_SWITCH,
 							  TK_SYNCER_OPTION);
 			break;
 		case TK_STARTUP:
 			check_uniq("startup section", "%s:startup", res->name);
+			EXP('{');
 			res->startup_options=parse_options(TK_STARTUP_SWITCH,
 							   TK_STARTUP_OPTION);
 			break;
 		case TK_HANDLER:
 			check_uniq("handlers section", "%s:handlers", res->name);
+			EXP('{');
 			res->handlers =  parse_options(0, TK_HANDLER_OPTION);
 			break;
+		case TK_PROXY:
+			check_uniq("proxy section", "%s:proxy", res->name);
+			EXP('{');
+			res->proxy_options =  parse_options(TK_PROXY_SWITCH,
+							    TK_PROXY_OPTION);
+			break;
+		case TK_DEVICE:
+			EXP(TK_STRING);
+			res->device = yylval.txt;
+			EXP(';');
+			break;
+		case TK_META_DISK:
+			parse_meta_disk(&res->meta_disk, &res->meta_index);
+			break;
+		case TK_FLEX_META_DISK:
+			EXP(TK_STRING);
+			res->meta_disk = yylval.txt;
+			if (strcmp("internal", yylval.txt)) {
+				res->meta_index = strdup("flexible");
+			}
+			EXP(';');
+			break;
 		case '}':
 		case 0:
 			goto exit_loop;
@@ -604,6 +638,41 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 	}
 
  exit_loop:
+
+	/* Determin the local host section and the peer host section. */
+	host = res->all_hosts;
+	while(host) {
+		if(!strcmp(host->name, nodeinfo.nodename) ||
+		   !strcmp(host->name, "_this_host") ||
+		   ( host->proxy && 
+		     !strcmp(host->proxy->name, nodeinfo.nodename))) {
+			if(res->me) {
+				config_valid = 0;
+				fprintf(stderr,
+					"%s:%d: in resource %s, on %s { ... } ... on %s { ... }:\n"
+					"\tThere are multiple host sections for this node.\n"
+					"\tMaybe misspelled local host name '%s'?\n",
+					config_file, c_section_start, res->name,
+						res->me->name, host->name, nodeinfo.nodename);
+			}
+			res->me = host;
+		} else {
+			/* This needs to be refined as soon as we support 
+			   multiple peers in the resource section */
+			if(res->peer) {
+				config_valid = 0;
+				fprintf(stderr,
+					"%s:%d: in resource %s, on %s { ... } ... on %s { ... }:\n"
+					"\tThere are multiple host sections for the peer.\n"
+					"\tMaybe misspelled local host name '%s'?\n",
+					config_file, c_section_start, res->name,
+						res->peer->name, host->name, nodeinfo.nodename);
+			}
+			res->peer = host;
+		}
+		host=host->next;
+	}
+
 	if(flags & ThisHRequired && !res->me) {
 		config_valid = 0;
 
