@@ -45,7 +45,6 @@
 #include <sys/time.h>
 #include <time.h>
 
-#define __bitwise /* Build-workaround for broken RHEL4 kernels (2.6.9_78.0.1) */
 #include <linux/netlink.h>
 #include <linux/connector.h>
 
@@ -182,7 +181,6 @@ int conv_protocol(struct drbd_argument *ad, struct drbd_tag_list *tl, char* arg)
 
 // convert functions for options
 int conv_numeric(struct drbd_option *od, struct drbd_tag_list *tl, char* arg);
-int conv_sndbuf(struct drbd_option *od, struct drbd_tag_list *tl, char* arg);
 int conv_handler(struct drbd_option *od, struct drbd_tag_list *tl, char* arg);
 int conv_bit(struct drbd_option *od, struct drbd_tag_list *tl, char* arg);
 int conv_string(struct drbd_option *od, struct drbd_tag_list *tl, char* arg);
@@ -194,7 +192,7 @@ void show_bit(struct drbd_option *od, unsigned short* tp);
 void show_string(struct drbd_option *od, unsigned short* tp);
 
 // sub functions for events_cmd
-int print_state(unsigned int seq, int, struct drbd_nl_cfg_reply *reply);
+int print_broadcast_events(unsigned int seq, int, struct drbd_nl_cfg_reply *reply);
 int w_connected_state(unsigned int seq, int, struct drbd_nl_cfg_reply *reply);
 int w_synced_state(unsigned int seq, int, struct drbd_nl_cfg_reply *reply);
 
@@ -211,7 +209,7 @@ const char *fencing_n[] = {
 };
 
 const char *asb0p_n[] = {
-        [Disconnect]        = "disconnect",
+	[Disconnect]        = "disconnect",
 	[DiscardYoungerPri] = "discard-younger-primary",
 	[DiscardOlderPri]   = "discard-older-primary",
 	[DiscardZeroChg]    = "discard-zero-changes",
@@ -251,10 +249,6 @@ struct option wait_cmds_options[] = {
 	conv_numeric, show_numeric, numeric_opt_usage, numeric_opt_xml, \
 	{ .numeric_param = { DRBD_ ## N ## _MIN, DRBD_ ## N ## _MAX, \
 		DRBD_ ## N ## _DEF ,U,UN  } }
-#define EN_sndbuf(N,U,UN) \
-	conv_sndbuf, show_numeric, numeric_opt_usage, numeric_opt_xml, \
-	{ .numeric_param = { DRBD_ ## N ## _MIN, DRBD_ ## N ## _MAX, \
-		DRBD_ ## N ## _DEF ,U,UN  } }
 #define EH(N,D) \
 	conv_handler, show_handler, handler_opt_usage, handler_opt_xml, \
 	{ .handler_param = { N, ARRY_SIZE(N), \
@@ -271,7 +265,7 @@ struct drbd_cmd commands[] = {
 	{"primary", P_primary, F_CONFIG_CMD, {{ NULL,
 	 (struct drbd_option[]) {
 		 { "overwrite-data-of-peer",'o',T_overwrite_peer, EB   },
-                 CLOSE_OPTIONS }} }, },
+		 CLOSE_OPTIONS }} }, },
 
 	{"secondary", P_secondary, F_CONFIG_CMD, {{NULL, NULL}} },
 
@@ -288,6 +282,7 @@ struct drbd_cmd commands[] = {
 		 { "use-bmbv",'b',	T_use_bmbv,     EB },
 		 { "no-disk-flushes",'i',T_no_disk_flush,EB },
 		 { "no-md-flushes",'m', T_no_md_flush,  EB },
+		 { "max-bio-bvecs",'s',	T_max_bio_bvecs,EN(MAX_BIO_BVECS,1,NULL) },
 		 CLOSE_OPTIONS }} }, },
 
 	{"detach", P_detach, F_CONFIG_CMD, {{NULL, NULL}} },
@@ -297,7 +292,7 @@ struct drbd_cmd commands[] = {
 		 { "local_addr",	T_my_addr,	conv_address },
 		 { "remote_addr",	T_peer_addr,	conv_address },
 		 { "protocol",		T_wire_protocol,conv_protocol },
- 		 { NULL,                0,           	NULL}, },
+		 { NULL,                0,           	NULL}, },
 	 (struct drbd_option[]) {
 		 { "timeout",'t',	T_timeout,	EN(TIMEOUT,1,"1/10 seconds") },
 		 { "max-epoch-size",'e',T_max_epoch_size,EN(MAX_EPOCH_SIZE,1,NULL) },
@@ -305,7 +300,7 @@ struct drbd_cmd commands[] = {
 		 { "unplug-watermark",'u',T_unplug_watermark, EN(UNPLUG_WATERMARK,1,NULL) },
 		 { "connect-int",'c',	T_try_connect_int, EN(CONNECT_INT,1,"seconds") },
 		 { "ping-int",'i',	T_ping_int,	   EN(PING_INT,1,"seconds") },
-		 { "sndbuf-size",'S',	T_sndbuf_size,	   EN_sndbuf(SNDBUF_SIZE,1,"bytes") },
+		 { "sndbuf-size",'S',	T_sndbuf_size,	   EN(SNDBUF_SIZE,1,"bytes") },
 		 { "ko-count",'k',	T_ko_count,	   EN(KO_COUNT,1,NULL) },
 		 { "allow-two-primaries",'m',T_two_primaries, EB },
 		 { "cram-hmac-alg",'a',	T_cram_hmac_alg,   ES },
@@ -317,6 +312,7 @@ struct drbd_cmd commands[] = {
 		 { "rr-conflict",'R',	T_rr_conflict,EH(rrcf_n,RR_CONFLICT) },
 		 { "ping-timeout",'p',  T_ping_timeo,	   EN(PING_TIMEO,1,"1/10 seconds") },
 		 { "discard-my-data",'D', T_want_lose,     EB },
+		 { "data-integrity-alg",'d', T_integrity_alg,     ES },
 		 CLOSE_OPTIONS }} }, },
 
 	{"disconnect", P_disconnect, F_CONFIG_CMD, {{NULL, NULL}} },
@@ -331,6 +327,8 @@ struct drbd_cmd commands[] = {
 		 { "rate",'r',T_rate,			EN(RATE,'k',"bytes/second") },
 		 { "after",'a',T_after,			EN(AFTER,1,NULL) },
 		 { "al-extents",'e',T_al_extents,	EN(AL_EXTENTS,1,NULL) },
+		 { "verify-alg", 'v',T_verify_alg,      ES },
+		 { "cpu-mask",'c',T_cpu_mask,           ES },
 		 CLOSE_OPTIONS }} }, },
 
 	{"invalidate", P_invalidate, F_CONFIG_CMD, {{ NULL, NULL }} },
@@ -340,6 +338,7 @@ struct drbd_cmd commands[] = {
 	{"suspend-io", P_suspend_io, F_CONFIG_CMD, {{ NULL, NULL }} },
 	{"resume-io", P_resume_io, F_CONFIG_CMD, {{ NULL, NULL }} },
 	{"outdate", P_outdate, F_CONFIG_CMD, {{ NULL, NULL }} },
+	{"verify", P_start_ov, F_CONFIG_CMD, {{NULL, NULL}} },
 	{"down",            0, down_cmd, get_usage, { {NULL, NULL }} },
 	{"state", P_get_state, F_GET_CMD, { .gp={ state_scmd} } },
 	{"cstate", P_get_state, F_GET_CMD, {.gp={ cstate_scmd} } },
@@ -352,7 +351,7 @@ struct drbd_cmd commands[] = {
 			{ "unfiltered", no_argument, 0, 'u' },
 			{ "all-devices",no_argument, 0, 'a' },
 			{ 0,            0,           0,  0  } },
-		print_state } } },
+		print_broadcast_events } } },
 	{"wait-connect", 0, F_EVENTS_CMD, { .ep = {
 		wait_cmds_options, w_connected_state } } },
 	{"wait-sync", 0, F_EVENTS_CMD, { .ep = {
@@ -407,6 +406,13 @@ static const char *error_messages[] = {
 	EM(HaveNoDiskConfig) = "Device does not have a disk-config",
 	EM(ProtocolCRequired) = "Protocol C required",
 	EM(VMallocFailed) = "vmalloc() failed. Out of memory?",
+	EM(IntegrityAlgNotAvail) = "The 'data-integrity-alg' you specified is not known in "
+	"the kernel. (Maybe you need to modprobe it, or modprobe hmac?)",
+	EM(IntegrityAlgNotDigest) = "The 'data-integrity-alg' you specified is not a digest.",
+	EM(CPUMaskParseFailed) = "Invalid cpu-mask.",
+	EM(VERIFYAlgNotAvail) = "VERIFYAlgNotAvail",
+	EM(VERIFYAlgNotDigest) = "VERIFYAlgNotDigest",
+	EM(VERIFYIsRunning) = "Can not change verify-alg while online verify runs",
 	EM(DataOfWrongCurrent) = "Can only attach to the data we lost last (see kernel log).",
 };
 #define MAX_ERROR (sizeof(error_messages)/sizeof(*error_messages))
@@ -422,7 +428,6 @@ char *cmdname = NULL; /* "drbdsetup" for reporting in usage etc. */
 char *devname = NULL; /* "/dev/drbd12" for reporting in print_config_error */
 int debug_dump_argv = 0; /* enabled by setting DRBD_DEBUG_DUMP_ARGV in the environment */
 int lock_fd;
-unsigned int cn_idx;
 
 int dump_tag_list(unsigned short *tlc)
 {
@@ -643,21 +648,6 @@ int conv_bit(struct drbd_option *od, struct drbd_tag_list *tl, char* arg __attri
 	return NoError;
 }
 
-int conv_sndbuf(struct drbd_option *od, struct drbd_tag_list *tl, char* arg)
-{
-	int err = conv_numeric(od, tl, arg);
-	long long l = m_strtoll(arg, 0);
-	char bit = 0;
-
-	if (err != NoError || l != 0)
-		return err;
-	/* this is a mandatory bit,
-	 * to avoid newer userland to configure older modules with
-	 * a sndbuf size of zero, which would lead to Oops. */
-	add_tag(tl, T_auto_sndbuf_size, &bit, sizeof(bit));
-	return NoError;
-}
-
 int conv_numeric(struct drbd_option *od, struct drbd_tag_list *tl, char* arg)
 {
 	const long long min = od->numeric_param.min;
@@ -688,6 +678,7 @@ int conv_numeric(struct drbd_option *od, struct drbd_tag_list *tl, char* arg)
 		fprintf(stderr, "internal error in conv_numeric()\n");
 	}
 	return NoError;
+
 }
 
 int conv_handler(struct drbd_option *od, struct drbd_tag_list *tl, char* arg)
@@ -785,10 +776,9 @@ int print_config_error(int err_no)
 		return 20;
 
 	if ( ( err_no >= AfterLastRetCode || err_no <= RetCodeBase ) &&
-	     ( err_no > SS_CW_NoNeed || err_no < SS_LowerThanOutdated) ) {
-		fprintf(stderr,"%s: Error code %d unknown.\n"
-			"You should updated the drbd userland tools.\n",
-			devname, err_no);
+	     ( err_no > SS_CW_NoNeed || err_no < SS_NotSupported) ) {
+		fprintf(stderr,"Error code %d unknown.\n"
+			"You should update the drbd userland tools.\n",err_no);
 		rv = 20;
 	} else {
 		if(err_no > RetCodeBase ) {
@@ -881,7 +871,7 @@ int _generic_config_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
 			rv = OTHER_ERROR;
 			goto error;
 		}
-		rv = ad->convert_function(ad,tl,argv[i++]);
+		rv |= ad->convert_function(ad,tl,argv[i++]);
 		if (rv != NoError)
 			goto error;
 		ad++;
@@ -892,8 +882,7 @@ int _generic_config_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
 	opterr=0;
 	while( (c=getopt_long(argc,argv,make_optstring(lo,':'),lo,0)) != -1 ) {
 		od = find_opt_by_short_name(cm->cp.options,c);
-		if (od)
-			rv = od->convert_function(od,tl,optarg);
+		if(od) rv |= od->convert_function(od,tl,optarg);
 		else {
 			if(c=='(') flags |= DRBD_NL_SET_DEFAULTS;
 			else if(c==')') flags |= DRBD_NL_CREATE_DEVICE;
@@ -1092,6 +1081,32 @@ int consume_tag_int(enum drbd_tags tag, unsigned short *tlc, int* val)
 	return 0;
 }
 
+int consume_tag_u64(enum drbd_tags tag, unsigned short *tlc, unsigned long long* val)
+{
+	unsigned short *tp;
+	unsigned short len;
+	tp = look_for_tag(tlc,tag);
+	if(tp) {
+		*tp++ = TT_REMOVED;
+		len = *tp++;
+		/* check the data size.
+		 * actually it has to be long long, but I'm paranoid */
+		if (len == sizeof(int))
+			*val = *(unsigned int*)tp;
+		else if (len == sizeof(long))
+			*val = *(unsigned long *)tp;
+		else if (len == sizeof(long long))
+			*val = *(unsigned long long *)tp;
+		else {
+			fprintf(stderr, "%s: unexpected tag len: %u\n",
+					__func__ , len);
+			return 0;
+		}
+		return 1;
+	}
+	return 0;
+}
+
 int consume_tag_bit(enum drbd_tags tag, unsigned short *tlc, int* val)
 {
 	unsigned short *tp;
@@ -1202,7 +1217,6 @@ int show_scmd(struct drbd_cmd *cm, int minor, unsigned short *rtl)
 		       ntohs(addr->sin_port));
 		printf("}\n");
 	}
-	consume_tag_bit(T_auto_sndbuf_size, rtl, &idx); /* consume it, its value has no relevance */
 
 	return 0;
 }
@@ -1315,8 +1329,113 @@ int down_cmd(struct drbd_cmd *cm, int minor, int argc, char **argv)
 	return cm->function(cm,minor,argc,argv);
 }
 
-int print_state(unsigned int seq, int u __attribute((unused)),
-		struct drbd_nl_cfg_reply *reply)
+
+void print_digest(const char* label, const int len, const unsigned char *hash)
+{
+	int i;
+	printf("\t%s: ", label);
+	for (i = 0; i < len; i++)
+		printf("%02x",hash[i]);
+	printf("\n");
+}
+
+static inline char printable_or_dot(char c)
+{
+	return (' ' < c && c <= '~') ? c : '.';
+}
+
+void print_hex_line(int offset, unsigned char *data)
+{
+
+	printf(	" %04x:"
+		" %02x %02x %02x %02x %02x %02x %02x %02x "
+		" %02x %02x %02x %02x %02x %02x %02x %02x"
+		"  %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c\n",
+		offset,
+		data[0], data[1], data[2], data[3],
+		data[4], data[5], data[6], data[7],
+		data[8], data[9], data[10], data[11],
+		data[12], data[13], data[14], data[15],
+		printable_or_dot(data[0]), printable_or_dot(data[1]),
+		printable_or_dot(data[2]), printable_or_dot(data[3]),
+		printable_or_dot(data[4]), printable_or_dot(data[5]),
+		printable_or_dot(data[6]), printable_or_dot(data[7]),
+		printable_or_dot(data[8]), printable_or_dot(data[9]),
+		printable_or_dot(data[10]), printable_or_dot(data[11]),
+		printable_or_dot(data[12]), printable_or_dot(data[13]),
+		printable_or_dot(data[14]), printable_or_dot(data[15]));
+}
+
+/* successive identical lines are collapsed into just printing one star */
+void print_hex_dump(int len, void *data)
+{
+	int i;
+	int star = 0;
+	for (i = 0; i < len-15; i += 16) {
+		if (i == 0 || memcmp(data + i, data + i - 16, 16)) {
+			print_hex_line(i, data + i);
+			star = 0;
+		} else if (!star)  {
+			printf(" *\n");
+			star = 1;
+		}
+	}
+	/* yes, I ignore remainders of len not modulo 16 here.
+	 * so what, usage is currently to dump bios, which are
+	 * multiple of 512. */
+	/* for good measure, print the total size as offset now,
+	 * last line may have been a '*' */
+	printf(" %04x.\n", len);
+}
+
+void print_dump_ee(struct drbd_nl_cfg_reply *reply)
+{
+	unsigned long long sector = -1ULL;
+	unsigned long long block_id = 0;
+	char *reason = "UNKNOWN REASON";
+	char *dig_in = NULL;
+	char *dig_vv = NULL;
+	unsigned int dgs_in = 0, dgs_vv = 0;
+	unsigned int size = 0;
+	char *data = NULL;
+
+	if (!consume_tag_string(T_dump_ee_reason, reply->tag_list, &reason))
+		printf("\tno reason?\n");
+	if (!consume_tag_blob(T_seen_digest, reply->tag_list, &dig_in, &dgs_in))
+		printf("\tno digest in?\n");
+	if (!consume_tag_blob(T_calc_digest, reply->tag_list, &dig_vv, &dgs_vv))
+		printf("\tno digest out?\n");
+	if (!consume_tag_u64(T_ee_sector, reply->tag_list, &sector))
+		printf("\tno sector?\n");
+	if (!consume_tag_u64(T_ee_block_id, reply->tag_list, &block_id))
+		printf("\tno block_id?\n");
+	if (!consume_tag_blob(T_ee_data, reply->tag_list, &data, &size))
+		printf("\tno data?\n");
+
+	printf("\tdumping ee, reason: %s\n", reason);
+	printf("\tsector: %llu block_id: 0x%llx size: %u\n",
+			sector, block_id, size);
+	
+	/* "input sanitation". Did I mention yet that I'm paranoid? */
+	if (!data) size = 0;
+	if (!dig_in) dgs_in = 0;
+	if (!dig_vv) dgs_vv = 0;
+	if (dgs_in > SHARED_SECRET_MAX) dgs_in = SHARED_SECRET_MAX;
+	if (dgs_vv > SHARED_SECRET_MAX) dgs_vv = SHARED_SECRET_MAX;
+
+	print_digest("received digest", dgs_in, (unsigned char*)dig_in);
+	print_digest("verified digest", dgs_vv, (unsigned char*)dig_vv);
+
+	/* dump at most 32 K */
+	if (size > 0x8000) {
+		size = 0x8000;
+		printf("\tWARNING truncating data to %u!\n", 0x8000);
+	}
+	print_hex_dump(size,data);
+}
+
+int print_broadcast_events(unsigned int seq, int u __attribute((unused)),
+			   struct drbd_nl_cfg_reply *reply)
 {
 	union drbd_state_t state;
 	char* str;
@@ -1350,13 +1469,17 @@ int print_state(unsigned int seq, int u __attribute((unused)),
 		break;
 	case P_sync_progress:
 		if (consume_tag_int(T_sync_progress, reply->tag_list, &synced)) {
-			printf("%u SP %d %i.%i\n", 
+			printf("%u SP %d %i.%i\n",
 				seq,
 				reply->minor,
 				synced / 10,
 				synced % 10);
 		} else fprintf(stderr,"Missing tag !?\n");
 		break;
+	case P_dump_ee:
+		printf("%u DE %d\n", seq, reply->minor);
+		print_dump_ee(reply);
+		break;
 	default:
 		printf("%u ?? %d <other message>\n",seq, reply->minor);
 		break;
@@ -1400,12 +1523,12 @@ int w_synced_state(unsigned int seq __attribute((unused)),
 
 int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
 {
-	char buffer[ 4096 ];
+	void *buffer;
 	struct cn_msg *cn_reply;
 	struct drbd_nl_cfg_reply *reply;
 	struct drbd_tag_list *tl;
 	struct option *lo;
-	unsigned int b_seq=0, r_seq=0;
+	unsigned int seq=0;
 	int sk_nl,c,cont=1,rr,i,last;
 	int unfiltered=0, all_devices=0;
 	int wfc_timeout=0, degr_wfc_timeout=0,timeout_ms;
@@ -1469,7 +1592,15 @@ int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
 	tl->drbd_p_header->drbd_minor = minor;
 	tl->drbd_p_header->flags = 0;
 
-	call_drbd(sk_nl,tl, (struct nlmsghdr*)buffer,4096,NL_TIME);
+	/* allocate 64k to be on the safe side. */
+#define NL_BUFFER_SIZE (64 << 10)
+	buffer = malloc(NL_BUFFER_SIZE);
+	if (!buffer) {
+		fprintf(stderr, "could not allocate buffer of %u bytes\n", NL_BUFFER_SIZE);
+		exit(20);
+	}
+
+	call_drbd(sk_nl,tl, buffer, NL_BUFFER_SIZE, NL_TIME);
 
 	cn_reply = (struct cn_msg *)NLMSG_DATA(buffer);
 	reply = (struct drbd_nl_cfg_reply *)cn_reply->data;
@@ -1509,9 +1640,9 @@ int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
 
 	do {
 		gettimeofday(&before,NULL);
-		rr = receive_cn(sk_nl, (struct nlmsghdr*)buffer, 4096,timeout_ms );
+		rr = receive_cn(sk_nl, buffer, NL_BUFFER_SIZE, timeout_ms);
 		gettimeofday(&after,NULL);
-		if(rr == -2) return 5; // timeout expired.
+		if(rr == -2) break; // timeout expired.
 
 		if(timeout_ms > 0 ) {
 			timeout_ms -= ( (after.tv_sec - before.tv_sec) * 1000 +
@@ -1523,26 +1654,22 @@ int events_cmd(struct drbd_cmd *cm, int minor, int argc ,char **argv)
 
 		// dump_tag_list(reply->tag_list);
 
-		/* There are two value spaces for sequence numbers. The first
-		   is the one created by this drbdsetup instance, the kernel's
-		   reply packets simply echo those sequence numbers.
-		   The second is created by the kernel's broadcast packets. */
-		if(cn_reply->ack==0) { // broadcasts
-			if(!unfiltered && cn_reply->seq <= b_seq) continue;
-			b_seq = cn_reply->seq;
-		} else { // replies to drbdsetup packes
-			if(!unfiltered && cn_reply->seq <= r_seq) continue;
-			r_seq = cn_reply->seq;
-		}
+		if(!unfiltered && cn_reply->seq <= seq) continue;
+		seq = cn_reply->seq;
 
 		if( all_devices || minor == reply->minor ) {
 			cont=cm->ep.proc_event(cn_reply->seq, wasb, reply);
 		}
 	} while(cont);
 
+	free(buffer);
+
 	close_cn(sk_nl);
 
-	return 0;
+	/* return code becomes exit code.
+	 * timeout? => exit 5
+	 * else     => exit 0 */
+	return (rr == -2) ? 5 : 0;
 }
 
 int numeric_opt_usage(struct drbd_option *option, char* str, int strlen)
@@ -1790,7 +1917,7 @@ int open_cn()
 	}
 
 	my_nla.nl_family = AF_NETLINK;
-	my_nla.nl_groups = -1; //cn_idx
+	my_nla.nl_groups = -1; //CN_IDX_DRBD;
 	my_nla.nl_pid = getpid();
 
 	err = bind(sk_nl, (struct sockaddr *)&my_nla, sizeof(my_nla));
@@ -1826,7 +1953,7 @@ void prepare_nl_header(struct nlmsghdr* nl_hdr, int size)
 	nl_hdr->nlmsg_pid = getpid();
 	/* fill the connector header */
 	cn_hdr->id.val = CN_VAL_DRBD;
-	cn_hdr->id.idx = cn_idx;
+	cn_hdr->id.idx = CN_IDX_DRBD;
 	cn_hdr->seq = cn_seq++;
 	get_random_bytes(&cn_hdr->ack,sizeof(cn_hdr->ack));
 	cn_hdr->len = size - sizeof(struct nlmsghdr) - sizeof(struct cn_msg);
@@ -1921,30 +2048,28 @@ void close_cn(int sk_nl)
 
 void ensure_drbd_driver_is_present(void)
 {
-	struct stat sb;
-	FILE *cn_idx_file;
-	int err;
+	struct drbd_tag_list *tl;
+	char buffer[4096];
+	int sk_nl, rr;
 
-	cn_idx = CN_IDX_DRBD;
-	cn_idx_file = fopen("/sys/module/drbd/parameters/cn_idx", "r");
-	if (cn_idx_file) {
-		fscanf(cn_idx_file, "%u", &cn_idx);
-		fclose(cn_idx_file);
-	}
+	sk_nl = open_cn();
+	/* Might print:
+	   Missing privileges? You should run this as root.
+	   Connector module not loaded? try 'modprobe cn'. */
+	if (sk_nl < 0) exit(20);
 
-	err = stat("/proc/drbd", &sb);
-	if (!err)
-		return;
+	tl = create_tag_list(2);
+	add_tag(tl, TT_END, NULL, 0); // close the tag list
 
-	if (err == ENOENT)
-		fprintf(stderr, "DRBD driver appears to be missing\n");
-	else
-		fprintf(stderr, "Could not stat(\"/proc/drbd\"): %m\n");
+	tl->drbd_p_header->packet_type = P_get_state;
+	tl->drbd_p_header->drbd_minor = 0;
+	tl->drbd_p_header->flags = 0;
 
-	fprintf(stderr, "do you need to load the module?\n"
-			"try: modprobe drbd\n");
-
-	exit(20);
+	rr = call_drbd(sk_nl, tl, (struct nlmsghdr*)buffer, 4096, 500);
+	/* Might print: (after 500ms)
+	   No response from the DRBD driver! Is the module loaded? */
+	close_cn(sk_nl);
+	if (rr == -2) exit(20);
 }
 
 int main(int argc, char** argv)
